#pragma once

using namespace System;
using namespace System::ComponentModel;
using namespace System::Collections;
using namespace System::Windows::Forms;
using namespace System::Data;
using namespace System::Drawing;


namespace IDVR
{
	/**
	<B>GUI class for creating the <i>About</i> dialog window.</B>.

	This class is part of the main application namespace IDVR and contains the definition of the <i>About</i> dialog window.
	<br><br>
	This dialog includes the genral information about the creaters of the application and about other issues of the creation process.
	<br><br>
	The API documentation for this class shows only the most important member variables and function. Event handling functions and GUI
	components (e.g. labels, textboxes,...) will not be included but all essential parts of the class are surely documented.

	@author <i>Bernhard Pflugfelder</i>
	@version 1.0
	@date 02-21-2005
	*/
	public __gc class AboutWin : public System::Windows::Forms::Form
	{
	public: 
		/**
		This is the standard constructor.
		It implements the standard initialization of the GUI window and actually start a 
		thread to create and control a specific dialog window.
		*/
		AboutWin(void)
		{
			InitializeComponent();

			this->lab_GeneralInfo->Text = S"This Importance Driven Volume Renderer (IDVR) has been developed as a student project between September 2004 and March 2005 at the Computer Graphics institute of the Technology University Vienna.";
			this->lab_NamesDescription->Text = S"The following students have been involved in the implementation of this IDVR RayCaster:";
			this->lab_Names->Text = S"Christopher Thurnher\nBernhard Pflugfelder";
			this->lab_Notice->Text = S"The GUI of this application has been created with the help of the SandBar toolkit:\nSandBar Copyright (c) 2004 by Tim Dawson, http://www.divil.co.uk/net/";
		}
        
	protected: 
		/**
		Destroys all GUI components before this current instance will be actually discarded.
		@param disposing is a flag variable which is true if the instance have to be discarded.
		*/
		void Dispose(Boolean disposing)
		{
			if (disposing && components)
			{
				components->Dispose();
			}
			__super::Dispose(disposing);
		}

	private: System::Windows::Forms::Label *  lab_GeneralInfo;
	private: System::Windows::Forms::Label *  lab_Notice;
	private: System::Windows::Forms::Button *  but_Close;
	private: System::Windows::Forms::Label *  lab_NamesDescription;
	private: System::Windows::Forms::Label *  lab_Names;

	private:

		/// Erforderliche Designervariable.
		System::ComponentModel::Container* components;

		/**
		The main function to create the all needed GUI components.
		Furthermore, the linking of those created GUI components (e.g. labels, textboxes,...) to the current container
		is implemented. After the correct linking all GUI items will be shown on the screen.
		@note This function must not be changed in a code editor because otherwise the GUI builder could be working incorrectly.
		*/
		void InitializeComponent(void)
		{
			this->lab_NamesDescription = new System::Windows::Forms::Label();
			this->lab_GeneralInfo = new System::Windows::Forms::Label();
			this->lab_Notice = new System::Windows::Forms::Label();
			this->but_Close = new System::Windows::Forms::Button();
			this->lab_Names = new System::Windows::Forms::Label();
			this->SuspendLayout();
			// 
			// lab_NamesDescription
			// 
			this->lab_NamesDescription->Location = System::Drawing::Point(16, 88);
			this->lab_NamesDescription->Name = S"lab_NamesDescription";
			this->lab_NamesDescription->Size = System::Drawing::Size(264, 32);
			this->lab_NamesDescription->TabIndex = 0;
			// 
			// lab_GeneralInfo
			// 
			this->lab_GeneralInfo->Location = System::Drawing::Point(16, 16);
			this->lab_GeneralInfo->Name = S"lab_GeneralInfo";
			this->lab_GeneralInfo->Size = System::Drawing::Size(264, 64);
			this->lab_GeneralInfo->TabIndex = 1;
			// 
			// lab_Notice
			// 
			this->lab_Notice->BorderStyle = System::Windows::Forms::BorderStyle::FixedSingle;
			this->lab_Notice->Location = System::Drawing::Point(8, 168);
			this->lab_Notice->Name = S"lab_Notice";
			this->lab_Notice->Size = System::Drawing::Size(272, 56);
			this->lab_Notice->TabIndex = 2;
			this->lab_Notice->TextAlign = System::Drawing::ContentAlignment::TopCenter;
			// 
			// but_Close
			// 
			this->but_Close->FlatStyle = System::Windows::Forms::FlatStyle::Popup;
			this->but_Close->Location = System::Drawing::Point(104, 232);
			this->but_Close->Name = S"but_Close";
			this->but_Close->Size = System::Drawing::Size(72, 24);
			this->but_Close->TabIndex = 3;
			this->but_Close->Text = S"OK";
			this->but_Close->Click += new System::EventHandler(this, but_Close_Click);
			// 
			// lab_Names
			// 
			this->lab_Names->Font = new System::Drawing::Font(S"Microsoft Sans Serif", 8.25F, System::Drawing::FontStyle::Bold, System::Drawing::GraphicsUnit::Point, (System::Byte)0);
			this->lab_Names->Location = System::Drawing::Point(14, 128);
			this->lab_Names->Name = S"lab_Names";
			this->lab_Names->Size = System::Drawing::Size(264, 32);
			this->lab_Names->TabIndex = 4;
			this->lab_Names->TextAlign = System::Drawing::ContentAlignment::TopCenter;
			// 
			// AboutWin
			// 
			this->AutoScaleBaseSize = System::Drawing::Size(5, 13);
			this->ClientSize = System::Drawing::Size(292, 266);
			this->Controls->Add(this->lab_Names);
			this->Controls->Add(this->but_Close);
			this->Controls->Add(this->lab_Notice);
			this->Controls->Add(this->lab_GeneralInfo);
			this->Controls->Add(this->lab_NamesDescription);
			this->FormBorderStyle = System::Windows::Forms::FormBorderStyle::FixedSingle;
			this->MaximizeBox = false;
			this->MinimizeBox = false;
			this->Name = S"AboutWin";
			this->StartPosition = System::Windows::Forms::FormStartPosition::CenterScreen;
			this->Text = S"About IDVR Volume RayCaster";
			this->ResumeLayout(false);

		}		
	private: System::Void but_Close_Click(System::Object *  sender, System::EventArgs *  e)
	{
		this->Dispose(true);
		this->Close();
	}

	};
}
