#include "stdafx.h"
#include "DataObject.h"

using namespace System;
using namespace System::Collections;
using namespace IDVR::Interfaces;
using namespace IDVR::Volume::DataStructures;

/*
This is the standard constructor.
All member variables of this new instance will be set to zero/null.
*/
DataObject::DataObject(void) 
{ 
	this->m_ObjectDescription = NULL;
	this->m_Importance = 0;
	this->m_RenderModel = NULL;
	this->m_DensityHistogram = NULL;
}

/*
Constructor initialization by all needed specified parameters.
All member variables will be set to the corresponding given parameters.
*/
DataObject::DataObject(String *objectDescription_temp, unsigned short n_Importance, unsigned short n_DensityScope, IRenderer* n_Renderer)
{	
	this->m_ObjectDescription = objectDescription_temp;
	this->m_Importance = n_Importance;
	this->m_RenderModel = n_Renderer;
	this->m_DensityHistogram = new Histogram(n_DensityScope);
}

/*
This is the standard deconstructor.
Espacially the histogram and render model will be deleted.
*/
DataObject::~DataObject(void)
{
	this->m_ObjectDescription = NULL;
	this->m_RenderModel = NULL;
	this->m_DensityHistogram = NULL;
}
	
/**********************************************************************************************************************/
/**********************************************************************************************************************/

//Returns the current importance value.
unsigned short DataObject::get_Importance(void) 
{ 
	return this->m_Importance; 
}

//Returns the current object description.
String* DataObject::get_ObjectDescription(void)
{ 
	return this->m_ObjectDescription; 
}
		
//Sets a new importance value for the current object.
void DataObject::set_Importance( unsigned short n_Importance) 
{ 
	m_Importance = n_Importance; 
}

/*
Returns the entire saved histogram as an one dimensional int array.
Use for fast access to Histogram instance values of the current volume object.
*/
int DataObject::getHistogramValues(void) __gc[]
{
	return this->m_DensityHistogram->get_HistogramValues();
}
	
//Returns a spcific density count of the object's histogram.
int DataObject::getHistogramValue(unsigned short n_Density)
{
	return this->m_DensityHistogram->getHistogramValue(n_Density);
}

//Returns the current density histogram.
Histogram* DataObject::get_DensityHistogram(void)
{ 
	return this->m_DensityHistogram; 
}
		
//Sets a new density histogram for the current object.
void DataObject::set_DensityHistogram(Histogram *n_DensityHistogram) 
{ 
	this->m_DensityHistogram = n_DensityHistogram; 
}

//Sets a new render model for the current object.
void DataObject::set_RenderModel(IRenderer* n_RenderModel) 
{ 
	this->m_RenderModel = n_RenderModel; 
}
	
//Returns the current render model.
IRenderer* DataObject::get_RenderModel(void) 
{ 
	return this->m_RenderModel; 
}