#include "StdAfx.h"
#include ".\DrawmodelGLSW.h"
#using <mscorlib.dll>

using namespace IDVR::Tools::OpenGLTools;

DrawModelGLSW::DrawModelGLSW(void)
{
	this->m_CanvasWidth = 0;
	this->m_CanvasHeight = 0;
	this->m_TextureWidth = 0;
	this->m_TextureHeight = 0;
	this->m_FrameBuffer = 0;
	this->m_CanvasBackColor = new float __gc[3];
	this->m_ResultTex = 0;
}

DrawModelGLSW::DrawModelGLSW(short n_CanvasWidth, short n_CanvasHeight, short n_TextureWidth, short n_TextureHeight)
{
	this->m_CanvasWidth = n_CanvasWidth;
	this->m_CanvasHeight = n_CanvasHeight;
	this->m_TextureWidth = n_TextureWidth;
	this->m_TextureHeight = n_TextureHeight;
	this->m_FrameBuffer = 0;
	this->m_CanvasBackColor = new float __gc[3];
	this->m_ResultTex = 0;
}

DrawModelGLSW::DrawModelGLSW(short n_CanvasWidth, short n_CanvasHeight, Color n_CanvasBackColor)
{
	this->m_CanvasWidth = n_CanvasWidth;
	this->m_CanvasHeight = n_CanvasHeight;
	this->m_CanvasBackColor = new float __gc[3];
	this->m_CanvasBackColor[R] = (float)n_CanvasBackColor.get_R() / 255.0f;
	this->m_CanvasBackColor[G] = (float)n_CanvasBackColor.get_G() / 255.0f;
	this->m_CanvasBackColor[B] = (float)n_CanvasBackColor.get_B() / 255.0f;
	this->m_ResultTex = 0;
	this->m_TextureWidth = 0;
	this->m_TextureHeight = 0;
	this->m_FrameBuffer = NULL;
}

DrawModelGLSW::DrawModelGLSW(short n_CanvasWidth, short n_CanvasHeight, short n_TextureWidth, short n_TextureHeight, unsigned char n_FrameBuffer __gc[], Color n_CanvasBackColor)
{
	this->m_CanvasWidth = n_CanvasWidth;
	this->m_CanvasHeight = n_CanvasHeight;
	this->m_TextureWidth = n_TextureWidth;
	this->m_TextureHeight = n_TextureHeight;
	this->m_FrameBuffer = n_FrameBuffer;
	this->m_CanvasBackColor = new float __gc[3];
	this->m_CanvasBackColor[R] = (float)n_CanvasBackColor.get_R() / 255.0f;
	this->m_CanvasBackColor[G] = (float)n_CanvasBackColor.get_G() / 255.0f;
	this->m_CanvasBackColor[B] = (float)n_CanvasBackColor.get_B() / 255.0f;
	this->m_ResultTex = 0;
	
}

DrawModelGLSW::~DrawModelGLSW(void)
{
	this->m_FrameBuffer = NULL;
	
	if(glIsTexture(this->m_ResultTex))
	{
		GLuint tempTexure = this->m_ResultTex;
		glDeleteTextures(GL_TEXTURE_2D, &tempTexure);
	}
}

void DrawModelGLSW::Init(void)
{
	glViewport(0, 0, this->m_CanvasWidth, this->m_CanvasHeight);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluOrtho2D(0.0, this->m_CanvasWidth, this->m_CanvasHeight, 0.0);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glClearColor(this->m_CanvasBackColor[0], this->m_CanvasBackColor[1], this->m_CanvasBackColor[2], 0.0);

	//Disable unnecessary shading features
	glDisable( GL_DEPTH_TEST );
    glDisable( GL_LINE_SMOOTH );
    glDisable( GL_POLYGON_SMOOTH );
	glDisable( GL_LIGHT0 );
	glDisable( GL_LIGHTING );
	glDisable( GL_COLOR_MATERIAL );
	glDisable( GL_AUTO_NORMAL );
    glDisable( GL_NORMALIZE );

	GLuint texture;

	if(glIsTexture(this->m_ResultTex))
	{
		GLuint tempTexure = this->m_ResultTex;
		glDeleteTextures(GL_TEXTURE_2D, &tempTexure);
	}

	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
	glGenTextures( 1, &texture );
	glBindTexture(GL_TEXTURE_2D, texture);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);

	gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGB, this->m_TextureWidth, this->m_TextureHeight,
                      GL_RGB, GL_UNSIGNED_BYTE, this->m_FrameBuffer );

	this->m_ResultTex = texture;
}

void DrawModelGLSW::Draw(void)
{
	glClear(GL_COLOR_BUFFER_BIT);

	glEnable( GL_TEXTURE_2D );
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
	glBindTexture(GL_TEXTURE_2D, this->m_ResultTex);

	glPushMatrix();
		glLoadIdentity();	
		glBegin(GL_QUADS);
			glTexCoord2d(0.0,0.0); glVertex2d(0.0,0.0);
			glTexCoord2d(1.0,0.0); glVertex2d(this->m_CanvasWidth,0.0);
			glTexCoord2d(1.0,1.0); glVertex2d(this->m_CanvasWidth,this->m_CanvasHeight);
			glTexCoord2d(0.0,1.0); glVertex2d(0.0,this->m_CanvasHeight);
		glEnd();
	glPopMatrix();

	glFlush();
	glDisable( GL_TEXTURE_2D );
	
}
				

void DrawModelGLSW::set_CanvasWidth(short n_CanvasWidth)
{
	this->m_CanvasWidth = n_CanvasWidth;
}

void DrawModelGLSW::set_CanvasHeight(short n_CanvasHeight)
{
	this->m_CanvasHeight = n_CanvasHeight;
}

void DrawModelGLSW::set_TextureWidth(short n_TextureWidth)
{
	this->m_TextureWidth = n_TextureWidth;
}

void DrawModelGLSW::set_TextureHeight(short n_TextureHeight)
{
	this->m_TextureHeight = n_TextureHeight;
}

void DrawModelGLSW::setFrameBuffer(unsigned char n_FrameBuffer __gc[])
{
	this->m_FrameBuffer = n_FrameBuffer;
}

void DrawModelGLSW::set_CanvasBackColor(Color n_CanvasBackColor)
{
	this->m_CanvasBackColor[R] = (float)n_CanvasBackColor.get_R() / 255.0f;
	this->m_CanvasBackColor[G] = (float)n_CanvasBackColor.get_G() / 255.0f;
	this->m_CanvasBackColor[B] = (float)n_CanvasBackColor.get_B() / 255.0f;
}

short DrawModelGLSW::get_CanvasWidth(void)
{
	return this->m_CanvasWidth;
}

short DrawModelGLSW::get_CanvasHeight(void)
{
	return this->m_CanvasHeight;
}

short DrawModelGLSW::get_TextureWidth(void)
{
	return this->m_TextureWidth;
}

short DrawModelGLSW::get_TextureHeight(void)
{
	return this->m_TextureHeight;
}

unsigned char DrawModelGLSW::getFrameBuffer(void) __gc[]
{
	return this->m_FrameBuffer;
}

Bitmap* DrawModelGLSW::GetBitmap(unsigned char n_FrameBuffer __gc[], int n_CanvasWidth, int n_CanvasHeight, int n_Resolution)
{

	double samplingFactor = (double)n_CanvasWidth / n_Resolution;  
	int sampledX = 0, sampledY = 0, bufferIndex = 0, red = 0, green = 0, blue = 0;

	Bitmap *newBitmap = new Bitmap(n_CanvasWidth, n_CanvasHeight);

	//canvas resolution is bigger than imageplane resolution -> linear up sampling is needed
	if (samplingFactor >= 1)
	{
		for (int currentY = 0; currentY < n_CanvasHeight; currentY++)
			for (int currentX = 0; currentX < n_CanvasWidth; currentX++)	
			{
				sampledX = Convert::ToInt32(Math::Floor((double)currentX / (double)samplingFactor));
				sampledY = Convert::ToInt32(Math::Floor((double)currentY / (double)samplingFactor));

				newBitmap->SetPixel(currentX, currentY, Color::FromArgb(n_FrameBuffer[bufferIndex++],
																		n_FrameBuffer[bufferIndex++],
																		n_FrameBuffer[bufferIndex++]));
			}
	}
    else
	{
		double inverse_samplingFactor = 1.0 / samplingFactor;

		for (int currentY = 0; currentY < n_CanvasHeight; currentY++)
			for (int currentX = 0; currentX < n_CanvasWidth; currentX++)	
			{
				for (int sampledY = Convert::ToInt32(Math::Floor(currentY * inverse_samplingFactor)); sampledY < Convert::ToInt32(Math::Floor((currentY * inverse_samplingFactor) + inverse_samplingFactor)); sampledY++)
					for (int sampledX = Convert::ToInt32(Math::Floor(currentX * inverse_samplingFactor)); sampledX < Convert::ToInt32(Math::Floor((currentX * inverse_samplingFactor) + inverse_samplingFactor)); sampledX++)
					{
						red += n_FrameBuffer[bufferIndex++];
						green += n_FrameBuffer[bufferIndex++];
						blue += n_FrameBuffer[bufferIndex++];
					}

				red = Convert::ToInt32(Math::Round((float)red / (inverse_samplingFactor * inverse_samplingFactor)));
				green = Convert::ToInt32(Math::Round((float)green / (inverse_samplingFactor * inverse_samplingFactor)));
				blue = Convert::ToInt32(Math::Round((float)blue / (inverse_samplingFactor * inverse_samplingFactor)));
				
				newBitmap->SetPixel(currentX, currentY, Color::FromArgb(red,green,blue));
			}
	}

	return newBitmap;
}