#pragma once

#include ".\ICompositingModel.h"

namespace IDVR {
namespace VolumeRenderer {
namespace SWVolumeRenderer {
namespace CompositingModels {

using namespace IDVR::Interfaces;

/**<B>Class for the dvr compositing model.</B>

Namespace: IDVR::VolumeRenderer::SWVolumeRenderer::CompositingModels

Baseclass: IDVR::Interfaces::ICompositingModel

This class blends the color and opacity values of a given raysample to obtain the final pixel color according to
normal direct volume renderind compositing.

@see LMipCompositingModelSW - the LMip compositing model
@author <i>Christopher Thurnher</i>
@version 1.0
@date 02-18-2005
*/

public __gc class DvrCompositingModelSW : public ICompositingModel
{
private:

	float m_Opacity; /**< the opacity value. */
	float m_Intensity __gc[]; /**< the intensity array. */
	float m_BackgroundOpacity; /**< the background opacity */

public:

	__value enum colorIndices{RED, GREEN, BLUE};

	/**
	Standard constructor.
	*/
	DvrCompositingModelSW(void);
	/**
	Standard destructor.
	*/
	~DvrCompositingModelSW(void);

	/**
	Blends the final intensity and opacity values of a given raysample.
	@param n_RaySample the raysample that should be blended.
	*/
	virtual void doCompositing(RaySample* n_RaySample);

	/**
	Blends the final pixelcolor with the given background color.
	@param n_BackColor the background color which should be blended.
	*/
	virtual void doCompositingBackground(float n_BackColor __gc[]);

	/**
	Resets the values of the composition model.
	*/
	virtual void resetValues(void);

	/**
	Sets the intensity array.
	@param n_Intensity the intensity array.
	*/
	__property virtual void set_Intensity(float n_Intensity __gc[]);
	/**
	Sets the opacity value.
	@param n_Opacity the opacity value.
	*/
	__property virtual void set_Opacity(float n_Opacity);

	/**
	Returns the intensity array.
	@return The intensity array.
	*/
	__property virtual float get_Intensity(void) __gc[];
	/**
	Returns the opacity value.
	@return The opacity value.
	*/
	__property virtual float get_Opacity(void);
};
}
}
}
}
