#include "StdAfx.h"
#include ".\footprint.h"
#using <mscorlib.dll>

using namespace IDVR::VolumeRenderer::RenderPrimitives;

//Standard constructor.
Footprint::Footprint(void)
{
	this->m_Footprint = NULL;
}

//Constructor.
Footprint::Footprint(int n_Width, int n_Height, ImagePlane* n_ImagePlane, double n_SamplingDistance, VolumeData *n_Data)
{
	this->m_Footprint = new ArrayList* __gc[n_Width, n_Height];
	this->m_Width = n_Width;
	this->m_Height = n_Height;
	this->m_ImagePlane = n_ImagePlane;
	this->m_SamplingDistance = n_SamplingDistance;
	this->m_Data = n_Data;
	this->m_Interpolation = new InterpolationSW();

	calculateObjectIDWithMaxImportance();
}

//Standrad destructor.
Footprint::~Footprint(void)
{
	this->m_Footprint = NULL;
	this->m_Interpolation = NULL;
	this->m_DistanceToObjectWithMaxImportance = NULL;
}

//Creates the Footprint for a ray at the specified location.
void Footprint::createFootprint(int n_X, int n_Y, Vector3 *n_ExitPoint, int n_NumOfSamples, unsigned char n_InterpolationMode)
{
	RaySample* samplePoint = new RaySample(n_ExitPoint);
	
	for(int i = 0; i < n_NumOfSamples; i++)
	{
		short x_down = Convert::ToInt16(Math::Floor(samplePoint->get_Coords()->X)),
			  x_up = Convert::ToInt16(Math::Ceiling(samplePoint->get_Coords()->X)), 
			  y_down = Convert::ToInt16(Math::Floor(samplePoint->get_Coords()->Y)), 
			  y_up = Convert::ToInt16(Math::Ceiling(samplePoint->get_Coords()->Y)), 
			  z_down = Convert::ToInt16(Math::Floor(samplePoint->get_Coords()->Z)), 
			  z_up = Convert::ToInt16(Math::Ceiling(samplePoint->get_Coords()->Z));

		//check if sample point is exactly on a border of two voxels
		if (x_down == x_up) x_up++;
		if (y_down == y_up) y_up++;
		if (z_down == z_up) z_up++;

		try
		{
			//interpolate object ID of current ray sample
			samplePoint->set_ObjectID(m_Interpolation->interpolate_ObjectID(n_InterpolationMode, samplePoint, this->m_Data, x_down, x_up, y_down, y_up, z_down, z_up));
		}
		catch(Exception *e)
		{
			throw new Exception(e->Message, e);
		}
		unsigned short temp_ObjectID = samplePoint->get_ObjectID();
		unsigned short temp_Importance = this->m_Data->getObjectMember(temp_ObjectID)->get_Importance();
		double temp_Distance =this->m_ImagePlane->calculateDistance(samplePoint->get_Coords());

		if(this->m_Footprint[n_X, n_Y] == NULL || this->m_Footprint[n_X, n_Y]->get_Count() == 0)
		{
			this->m_Footprint[n_X, n_Y] = new ArrayList(0);
			this->m_Footprint[n_X, n_Y]->Add(new FootprintElement(temp_ObjectID, temp_Importance, temp_Distance));
		}
		else
		{
			bool isAlreadyInside = false;
			for(int j = 0; j < this->m_Footprint[n_X, n_Y]->get_Count(); j++)
			{
				if(__try_cast<FootprintElement*>(this->m_Footprint[n_X, n_Y]->get_Item(j))->getObjectID() == temp_ObjectID)
				{
					isAlreadyInside = true;
					break;
				}
			}
			if(!isAlreadyInside)
			{
				this->m_Footprint[n_X, n_Y]->Add(new FootprintElement(temp_ObjectID, temp_Importance, temp_Distance));
			}
		}
		samplePoint = new RaySample(Vector3::Addition(samplePoint->get_Coords(), this->m_ImagePlane->get_ViewDirection()->getNegativeVector()->multSkalar(m_SamplingDistance)));
	}
}

//Returns the Footprint array.
ArrayList* Footprint::getFootprint(void) __gc[,]
{
	return this->m_Footprint;
}

//Returns a Footprint list at the specified location.
ArrayList* Footprint::getFootprintArrayList(int n_X, int n_Y)
{
	return this->m_Footprint[n_X, n_Y];
}

//Returns the objectID with the maximal importance.
short Footprint::getObjectIDWithMaxImportance(void)
{
	return this->m_ObjectIDWithMaxImportance;
}

//Returns the distance array to the object with the objectID with the maximal importance.
double Footprint::getDistanceToObjectWithMaxImportance(void) __gc[,]
{
	return this->m_DistanceToObjectWithMaxImportance;
}

//Returns the distance to the object with the objectID with the maximal importance at the spacified location.
double Footprint::getDistanceToObjectWithMaxImportance(int n_X, int n_Y)
{
	return this->m_DistanceToObjectWithMaxImportance[n_X, n_Y];
}

//Sets the countersink slope.
void Footprint::setSlope(double n_Slope)
{
	this->m_Slope = n_Slope;
}

//Sets the gradient scalefactor.
void Footprint::setGradientScaleFactor(double n_GradientScaleFactor)
{
	this->m_GradientScaleFactor = n_GradientScaleFactor;
}

//Calculates the objectID with the maximal importance.
void Footprint::calculateObjectIDWithMaxImportance(void)
{
	//calculate the object ID with the maximal importance
	this->m_ObjectIDWithMaxImportance = Int16::MinValue;
	short maxImportance = Int16::MinValue;

	for(int index = 0; index < this->m_Data->get_ObjectCount(); index++)
	{
		if(this->m_Data->getObjectMember(index)->get_Importance() > maxImportance)
		{
			//add objectID to hashtabe with max sparseness value 1.0;
			this->m_ObjectIDWithMaxImportance = index;
			maxImportance = this->m_Data->getObjectMember(index)->get_Importance();
		}
	}
}

//Calculates the highest depth value of the object with the objectID with maximal importance.
double Footprint::calculateHighestDepthValue(void)
{
	double highestDepthValue = Double::MinValue;
	double tempDepthValue = Double::MinValue;

	bool isObject = false;

	for(int curr_Height = 0; curr_Height < this->m_Height; curr_Height++)
	{
		for(int curr_Width = 0; curr_Width < this->m_Width; curr_Width++)
		{
			if(this->m_Footprint[curr_Width, curr_Height] == NULL)
			{
				continue;
			}

			isObject = false;

			//check, whether current element contains the object with maximal importance
			for(int index = 0; index < this->m_Footprint[curr_Width, curr_Height]->get_Count(); index++)
			{	
				if(__try_cast<FootprintElement*>(this->m_Footprint[curr_Width, curr_Height]->get_Item(index))->getObjectID() == this->m_ObjectIDWithMaxImportance)
				{
					tempDepthValue = __try_cast<FootprintElement*>(this->m_Footprint[curr_Width, curr_Height]->get_Item(index))->getDistance();
					isObject = true;
				}
			}

			if(isObject)
			{
				highestDepthValue = Math::Max(tempDepthValue, highestDepthValue);
			}
		}
	}

	return highestDepthValue;
}

//Calculates the distance of the ray's new starting point at the specified location.
double Footprint::calculateDistanceOfNewStartingPoint(int n_X, int n_Y)
{
	if(this->m_DistanceToObjectWithMaxImportance[n_X, n_Y] != Double::MinValue)
	{
		return this->m_HighestDepthValue - (this->m_DistanceToObjectWithMaxImportance[n_X, n_Y] * this->m_Slope);
	}
	else
	{
		return Double::MinValue;
	}
}

//Returns the maximal depth of the object with the maximal importance along one Footprint list item at the specified location.
double Footprint::calculateDepthOfObjectWithMaxImpAlongFootprintAL(int n_X, int n_Y)
{
	if(this->m_Footprint[n_X, n_Y] == NULL || this->m_Footprint[n_X, n_Y]->get_Count() == 0)
	{
		return Double::MinValue;
	}
	else
	{
		double temp_Depth = Double::MinValue;
		short temp_Importance = Int16::MinValue;

		for(int index = 0; index < this->m_Footprint[n_X, n_Y]->get_Count(); index++)
		{	
			FootprintElement *temp_FootprintElement = __try_cast<FootprintElement*>(this->m_Footprint[n_X, n_Y]->get_Item(index));
			if(temp_FootprintElement->getImportance() > temp_Importance)
			{
				temp_Importance = temp_FootprintElement->getImportance();
				temp_Depth = temp_FootprintElement->getDistance();
			}
		}
		return temp_Depth;
	}
}

//Calculates the distance array.
void Footprint::calculateDistanceArray(void)
{
	this->m_DistanceToObjectWithMaxImportance = new double __gc[this->m_Width, this->m_Height];
	double depthBuffer __gc[] = new double __gc[this->m_Width * this->m_Height];

	for(int curr_Height = 0; curr_Height < this->m_Height; curr_Height++)
	{
		for(int curr_Width = 0; curr_Width < this->m_Width; curr_Width++)
		{
			if(this->m_Footprint[curr_Width, curr_Height] == NULL || this->m_Footprint[curr_Width, curr_Height]->get_Count() == 0)
			{
				//depthBuffer[curr_Width * curr_Height] = Double::MinValue;
				this->m_DistanceToObjectWithMaxImportance[curr_Width, curr_Height] = Double::MaxValue;
			}
			else
			{
				for(int index = 0; index < this->m_Footprint[curr_Width, curr_Height]->get_Count(); index++)
				{	
					FootprintElement *temp_FootprintElement = __try_cast<FootprintElement*>(this->m_Footprint[curr_Width, curr_Height]->get_Item(index));
					if(temp_FootprintElement->getObjectID() == this->m_ObjectIDWithMaxImportance)
					{
						//depthBuffer[curr_Width * curr_Height] = 0.0;
						this->m_DistanceToObjectWithMaxImportance[curr_Width, curr_Height] = 0.0;
					}
					else
					{
						//depthBuffer[curr_Width * curr_Height] = Double::MinValue;
						this->m_DistanceToObjectWithMaxImportance[curr_Width, curr_Height] = Double::MaxValue;
					}
				}
			}
		}
	}

	double d1 = 1.0;
	double d2 = Math::Sqrt(2.0);

	for(int y = 1; y < this->m_Height-1; y++)
	{
		for(int x = 1; x < this->m_Width-1; x++) 
		{
			if(this->m_DistanceToObjectWithMaxImportance[x, y] != 0.0)
			{
				double wert1 = Double::MaxValue;
				double wert2 = Double::MaxValue;
				double wert3 = Double::MaxValue;
				double wert4 = Double::MaxValue;

				double wertMin = Double::MaxValue;

				if(this->m_DistanceToObjectWithMaxImportance[x - 1, y - 1] != Double::MaxValue)
				{
					wert1 = this->m_DistanceToObjectWithMaxImportance[x - 1, y - 1] + d2;
				}

				if(this->m_DistanceToObjectWithMaxImportance[x    , y - 1] != Double::MaxValue)
				{
					wert2 = this->m_DistanceToObjectWithMaxImportance[x    , y - 1] + d1;
				}

				if(this->m_DistanceToObjectWithMaxImportance[x + 1, y - 1] != Double::MaxValue)
				{
					wert3 = this->m_DistanceToObjectWithMaxImportance[x + 1, y - 1] + d2;
				}

				if(this->m_DistanceToObjectWithMaxImportance[x - 1, y    ] != Double::MaxValue)
				{
					wert4 = this->m_DistanceToObjectWithMaxImportance[x - 1, y    ] + d1;
				}

				wertMin = Math::Min(wertMin, wert1);
				wertMin = Math::Min(wertMin, wert2);
				wertMin = Math::Min(wertMin, wert3);
				wertMin = Math::Min(wertMin, wert4);

				if(wertMin != Double::MaxValue)
				{
					this->m_DistanceToObjectWithMaxImportance[x, y] = wertMin;
				}
			}
		}
	}

	for(int y = this->m_Height - 2; y > 0; y--) 
	{
		for(int x = this->m_Width - 2; x > 0; x--) 
		{
			if(this->m_DistanceToObjectWithMaxImportance[x, y] != 0.0)
			{
				double wert1 = Double::MaxValue;
				double wert2 = Double::MaxValue;
				double wert3 = Double::MaxValue;
				double wert4 = Double::MaxValue;

				double wertMin = Double::MaxValue;

				if(this->m_DistanceToObjectWithMaxImportance[x + 1, y    ] != Double::MaxValue)
				{
					wert1 = this->m_DistanceToObjectWithMaxImportance[x + 1, y    ] + d1;
				}

				if(this->m_DistanceToObjectWithMaxImportance[x - 1, y + 1] != Double::MaxValue)
				{
					wert2 = this->m_DistanceToObjectWithMaxImportance[x - 1, y + 1] + d2;
				}

				if(this->m_DistanceToObjectWithMaxImportance[x    , y + 1] != Double::MaxValue)
				{
					wert3 = this->m_DistanceToObjectWithMaxImportance[x    , y + 1] + d1;
				}

				if(this->m_DistanceToObjectWithMaxImportance[x + 1, y + 1] != Double::MaxValue)
				{
					wert4 = this->m_DistanceToObjectWithMaxImportance[x + 1, y + 1] + d2;
				}

				wertMin = Math::Min(wertMin, wert1);
				wertMin = Math::Min(wertMin, wert2);
				wertMin = Math::Min(wertMin, wert3);
				wertMin = Math::Min(wertMin, wert4);
				wertMin = Math::Min(wertMin, this->m_DistanceToObjectWithMaxImportance[x, y]);

				if(wertMin != Double::MaxValue)
				{
					this->m_DistanceToObjectWithMaxImportance[x, y] = wertMin;
				}
			}
		}
	}

	int gsgddfjk = 0;

	for(int curr_Height = 0; curr_Height < this->m_Height; curr_Height++)
	{
		for(int curr_Width = 0; curr_Width < this->m_Width; curr_Width++)
		{
			if(this->m_DistanceToObjectWithMaxImportance[curr_Width, curr_Height] == 0.0 ||
			   this->m_DistanceToObjectWithMaxImportance[curr_Width, curr_Height] == Double::MaxValue)
			{
				//depthBuffer[curr_Width * curr_Height] = 0.0;
				this->m_DistanceToObjectWithMaxImportance[curr_Width, curr_Height] = Double::MinValue;
			}
		}
	}

	this->m_HighestDepthValue = calculateHighestDepthValue();
}

//Modifies the gradient.
Gradient* Footprint::modifyGradient(int n_X, int n_Y, double n_ScaleFactor, Matrix4 *n_RotMatrix)
{
	Gradient *modifiedGradient;

	int steps = 2;
	int k = 1;

	double minDist = Double::MaxValue;

	bool found = false;
				
	while(!found)
	{
		//obere reihe
		for(int i = 0; i < steps; i++)
		{
			int newX = n_X - k + i;
			int newY = n_Y - k;

			if(newX < 0 || newY < 0 || newX >= this->m_Width || newY >= this->m_Height)
			{
				continue;
			}
							
			if(this->m_Footprint[newX, newY] == NULL)
			{
				continue;
			}
			else
			{
				//check, whether current element contains the object with maximal importance
				for(int index = 0; index < this->m_Footprint[newX, newY]->get_Count(); index++)
				{	
					if(__try_cast<FootprintElement*>(this->m_Footprint[newX, newY]->get_Item(index))->getObjectID() == this->m_ObjectIDWithMaxImportance)
					{
						Vector2 *v1 = new Vector2(n_X, n_Y);
						Vector2 *v2 = new Vector2(newX, newY);
						Vector2 *direction = Vector2::Subtraction(v1, v2);

						double dist = direction->magnitude();

						if(dist <= minDist)
						{
							minDist = dist;

							Vector3 *tempGradient = new Vector3(direction->X, direction->Y, 1.0 / this->m_Slope);

							//save calculated vector in the gradient array
							modifiedGradient = new Gradient(Vector3::CreateNormalizedVector(tempGradient));
								
							found = true;
						}
					}
				}
			}
		}

		//rechte reihe
		for(int i = 1; i < steps; i++)
		{
			int newX = n_X + k;
			int newY = n_Y - k + i;

			if(newX < 0 || newY < 0 || newX >= this->m_Width || newY >= this->m_Height)
			{
				continue;
			}

			if(this->m_Footprint[newX, newY] == NULL)
			{
				continue;
			}
			else
			{
				//check, whether current element contains the object with maximal importance
				for(int index = 0; index < this->m_Footprint[newX, newY]->get_Count(); index++)
				{	
					if(__try_cast<FootprintElement*>(this->m_Footprint[newX, newY]->get_Item(index))->getObjectID() == this->m_ObjectIDWithMaxImportance)
					{
						Vector2 *v1 = new Vector2(n_X, n_Y);
						Vector2 *v2 = new Vector2(newX, newY);
						Vector2 *direction = Vector2::Subtraction(v1, v2);
						
						double dist = direction->magnitude();

						if(dist <= minDist)
						{
							minDist = dist;

							Vector3 *tempGradient = new Vector3(direction->X, direction->Y, 1.0 / this->m_Slope);

							//save calculated vector in the gradient array
							modifiedGradient = new Gradient(Vector3::CreateNormalizedVector(tempGradient));
								
							found = true;
						}
					}
				}
			}
		}
		//untere reihe
		for(int i = 1; i < steps; i++)
		{
			int newX = n_X + k - i;
			int newY = n_Y + k;

			if(newX < 0 || newY < 0 || newX >= this->m_Width || newY >= this->m_Height)
			{
				continue;
			}

			if(this->m_Footprint[newX, newY] == NULL)
			{
				continue;
			}
			else
			{
				//check, whether current element contains the object with maximal importance
				for(int index = 0; index < this->m_Footprint[newX, newY]->get_Count(); index++)
				{	
					if(__try_cast<FootprintElement*>(this->m_Footprint[newX, newY]->get_Item(index))->getObjectID() == this->m_ObjectIDWithMaxImportance)
					{
						Vector2 *v1 = new Vector2(n_X, n_Y);
						Vector2 *v2 = new Vector2(newX, newY);
						Vector2 *direction = Vector2::Subtraction(v1, v2);
						
						double dist = direction->magnitude();

						if(dist <= minDist)
						{
							minDist = dist;

							Vector3 *tempGradient = new Vector3(direction->X, direction->Y, 1.0 / this->m_Slope);

							//save calculated vector in the gradient array
							modifiedGradient = new Gradient(Vector3::CreateNormalizedVector(tempGradient));
								
							found = true;
						}
					}
				}
			}
		}
		//linke reihe
		for(int i = 1; i < steps - 1; i++)
		{
			int newX = n_X - k;
			int newY = n_Y + k - i;

			if(newX < 0 || newY < 0 || newX >= this->m_Width || newY >= this->m_Height)
			{
				continue;
			}

			if(this->m_Footprint[newX, newY] == NULL)
			{
				continue;
			}
			else
			{
				//check, whether current element contains the object with maximal importance
				for(int index = 0; index < this->m_Footprint[newX, newY]->get_Count(); index++)
				{	
					if(__try_cast<FootprintElement*>(this->m_Footprint[newX, newY]->get_Item(index))->getObjectID() == this->m_ObjectIDWithMaxImportance)
					{
						Vector2 *v1 = new Vector2(n_X, n_Y);
						Vector2 *v2 = new Vector2(newX, newY);
						Vector2 *direction = Vector2::Subtraction(v1, v2);
							
						double dist = direction->magnitude();

						if(dist <= minDist)
						{
							minDist = dist;

							Vector3 *tempGradient = new Vector3(direction->X, direction->Y, 1.0 / this->m_Slope);

							//save calculated vector in the gradient array
							modifiedGradient = new Gradient(Vector3::CreateNormalizedVector(tempGradient));
								
							found = true;
						}
					}
				}
			}
		}

		k++;
		steps = steps + 2;
	}

	if(modifiedGradient->get_GradientVector()->multSkalar((double)this->m_GradientScaleFactor)->magnitude() <= n_ScaleFactor)
	{
		Vector3 *scaledGradient = new Vector3(modifiedGradient->get_GradientVector()->get_X(), modifiedGradient->get_GradientVector()->get_Y(), modifiedGradient->get_GradientVector()->get_Z());
		scaledGradient = scaledGradient->multSkalar((double)this->m_GradientScaleFactor);
		modifiedGradient = new Gradient(scaledGradient);
	}
	modifiedGradient->set_ScaleFactor(n_ScaleFactor);
	modifiedGradient->scale_Length();

	modifiedGradient->set_GradientVector(Vector4::GetVector3(n_RotMatrix->multiply_WithPoint(new Vector4(modifiedGradient->get_GradientVector(), 1.0))));

	return modifiedGradient;
}