#pragma once

#include "stdafx.h"
#include "VolumeData.h"
#include "ImagePlane.h"
#include "Vector2.h"

namespace IDVR {
namespace VolumeRenderer {
namespace HWVolumeRenderer {

	using namespace IDVR::VolumeRenderer::RenderPrimitives;
	using namespace IDVR::Volume::DataStructures;
	using namespace System::Drawing;

public __gc class GLSLShaderDataManager
{
private:
	
	GLuint m_Density_tex, m_EntryPoint_tex, m_TransferFunction_tex, m_ObjectID_tex;

	unsigned int m_MaxNumOfSamples;

	__value enum indicesXYZ {X = 0,Y,Z};
	__value enum indicesRGBA {R = 0,G,B,A};

	//calculates the entry point coordinates for a given start position along the view direction
	Vector3* calculate_IntersectionPoints(Vector3* n_StartPoint, Vector3 *n_ViewDirection, unsigned short n_VolumeDimension __gc[]) __gc[];

public:
	//Constructor to initalize the instace variables
	GLSLShaderDataManager(void);

	//creates a ready-to-use 3D texture object with all density values of the current data set
	void createDensityTextureObject(VolumeData *n_Dataset, unsigned char n_InterpolationMode);

	//creates a ready-to-use 3D texture object with all ObjectID values of the current data set
	void createObjectIDTextureObject(VolumeData *n_Dataset);
	
	//creates a ready-to-use 2D texture object containing the current transfer function from data set
	void createTransferFunctionTextureObjectFromDataset(Transferfunction *n_Tranferfunction, unsigned short n_ObjectID);
	
	//creates a ready-to-use 2D texture object containing the the standard linar transfer function
	void createTransferFunctionTextureObjectFromInterpolation(Color n_Color, unsigned short n_TransferfunctionLength);

	//creates a ready-to-use 2D texture object containing the entry point coords depending on the current view direction
	void createEntryPointTextureObject(VolumeData *n_Dataset, ImagePlane *n_ImagePlane, float n_RaySampleDistance);

	//sets a specific uniform variable of the current activated fragment shader (program object)
	void setShaderUniformVariable(GLhandleARB n_ProgramObject, const GLcharARB *n_VariableName, int n_VariableData);

	//sets a specific uniform variable of the current activated fragment shader (program object)
	void setShaderUniformVariable(GLhandleARB n_ProgramObject, const GLcharARB *n_VariableName, float n_VariableData);

	//sets a specific uniform variable of the current activated fragment shader (program object)
	void setShaderUniformVariable(GLhandleARB n_ProgramObject, const GLcharARB *n_VariableName, bool n_VariableData);

	//sets a specific uniform variable of the current activated fragment shader (program object)
	void setShaderUniformVariable(GLhandleARB n_ProgramObject, const GLcharARB *n_VariableName, Vector2 *n_VariableData);

	//sets a specific uniform variable of the current activated fragment shader (program object)
	void setShaderUniformVariable(GLhandleARB n_ProgramObject, const GLcharARB *n_VariableName, Vector3 *n_VariableData);

	//sets a specific uniform variable of the current activated fragment shader (program object)
	void setShaderUniformVariable(GLhandleARB n_ProgramObject, const GLcharARB *n_VariableName, GLuint n_TextureName, GLint n_TextureUnit, GLuint n_Texturetyp);

	//sets the given fragment object to OpenGL state
	void activateProgramObject(GLhandleARB n_ProgramObject);

	//resets the OpenGL state to foxed functionality
	void deactivateProgramObject(GLhandleARB n_ProgramObject);

	__property GLuint get_DensityTexture(void);
	__property GLuint get_ObjectIDTexture(void);
	__property GLuint get_TransferFunctionTexture(void);
	__property GLuint get_EntryPointTexture(void);

	static const unsigned char NEARESTNEIGHBOUR = 0, TRILINEAR = 1;

	

};

}
}
}