#include "StdAfx.h"
#include ".\GLSLShaderLoader.h"
#using <mscorlib.dll>

using namespace IDVR::VolumeRenderer::HWVolumeRenderer;
using namespace System::IO;

GLSLShaderLoader::GLSLShaderLoader(String *n_LogFilename)
{
	this->m_MipFragObject = 0;
	this->m_MipProgrammObject = 0;

	this->m_PhongDVRFragObject = 0;
	this->m_PhongProgrammObject = 0;
	
	this->m_ContourFragObject = 0;
	this->m_ContourProgrammObject = 0;

	this->m_ToneFragObject = 0;
	this->m_ToneProgrammObject = 0;

	this->m_LogFilename = n_LogFilename;
}

void GLSLShaderLoader::printInfoLog(GLhandleARB n_Object)
{
	int infologLength = 0,
		charsWritten = 0;

	GLcharARB *infoLog;

	if(printOpenGLError(__FILE__,__LINE__))
		throw new Exception(S"An OpenGL eror has occured. For further details see the log file please.");
	glGetObjectParameterivARB(n_Object, GL_OBJECT_INFO_LOG_LENGTH_ARB, &infologLength);
	if(printOpenGLError(__FILE__,__LINE__))
		throw new Exception(S"An OpenGL eror has occured. For further details see the log file please.");
	if(infologLength > 0)
	{
		infoLog = (GLcharARB*)malloc(infologLength);
		
		if(infoLog != NULL)
		{
			glGetInfoLogARB(n_Object, infologLength, &charsWritten, infoLog);

			StreamWriter *log = File::AppendText(this->m_LogFilename);
			log->WriteLine(String::Concat(S"Loginfo from glGetInfoLogARB: ", infoLog));
			log->Close();
			free(infoLog);
		}
		else
			throw new Exception(S"Could not create info log text.");
	}

	if(printOpenGLError(__FILE__,__LINE__))
		throw new Exception(S"An OpenGL eror has occured. For further details see the log file please.");
}

bool GLSLShaderLoader::printOpenGLError(char *n_Filename, int n_Line)
{
    GLenum glErr;
    bool error = false;

	StreamWriter *log = File::AppendText(this->m_LogFilename);

    glErr = glGetError();
    while (glErr != GL_NO_ERROR)
    {
		
		log->WriteLine(String::Concat(S"OpenGL error has occured in \"", new String(n_Filename), S"\" @ line ", n_Line.ToString(), S" : ", gluErrorString(glErr)));
        //printf("glError 0x%x file %s @ %d: %s\n", glErr, file, line, gluErrorString(glErr));
        error = true;
        glErr = glGetError();
    }

	log->Close();

    return error;
}

GLint GLSLShaderLoader::getUniformLocation(GLhandleARB n_ProgramObject, const GLcharARB *n_ParameterName)
{
	GLint location = 0;

	location = glGetUniformLocationARB(n_ProgramObject, n_ParameterName);
	if(printOpenGLError(__FILE__,__LINE__))
		throw new Exception(S"An OpenGL eror has occured. For further details see the log file please.");

	return location;
}

char* GLSLShaderLoader::readShaderSource(String *n_Filename)
{
	char *shaderCode = 0;

	if(!n_Filename->EndsWith(S".frag"))
	{
		throw new Exception(S"The choosen fragment source file does not end with \".frag\"");
		return NULL;
	}

	try
	{
		StreamReader *textReader = File::OpenText(n_Filename);
		String *sourceText = textReader->ReadToEnd();

		if(sourceText == NULL)
		{
			throw new Exception(String::Concat(S"The source code of \"", n_Filename, S"\" seems to be empty."));
			return NULL;
		}

		sourceText = String::Concat(sourceText, '\0');

		shaderCode = new char[sourceText->Length];
		int index = 0;

		CharEnumerator *nextChar = sourceText->GetEnumerator();
		while(nextChar->MoveNext())
			shaderCode[index++] = (char)nextChar->Current;
	}
	catch(Exception*)
	{
		throw new Exception(String::Concat(S"An error during reading the fragment shader code has occurd at \"", n_Filename, S"\"."));
		return NULL;
	}

	return shaderCode;
}

void GLSLShaderLoader::loadGLSLShaderModel(unsigned char n_ShaderType, String *n_Filename)
{
	
	
	GLint fragCompiled, fragLinked;

	char *sourceCode = 0;
	bool initalizeError = false, compileError = false;

	try {
		sourceCode = readShaderSource(n_Filename);
	}
	catch(Exception *e)
	{
		throw new Exception(e->Message,e);
		return;
	}

	const GLcharARB *fragmentCode = sourceCode;

	const GLubyte *extensions = glGetString(GL_EXTENSIONS);
	const GLubyte *version = glGetString(GL_VERSION);

	/*String *extensions = glGetString(GL_EXTENSIONS);
	String *version = glGetString(GL_VERSION);*/

	switch(n_ShaderType)
	{
	case MIPSHADER: 
		//Initialize the specific shader object
		this->m_MipFragObject = glCreateShaderObjectARB(GL_FRAGMENT_SHADER_ARB);
		//Load the sourcecode into the shader object
		glShaderSourceARB(this->m_MipFragObject, 1, &fragmentCode, NULL);
		//Compile the new shader object and output the log into the standard log file
		glCompileShaderARB(this->m_MipFragObject);
		if(printOpenGLError(__FILE__,__LINE__))
			break;
		
		glGetObjectParameterivARB(this->m_MipFragObject,GL_OBJECT_COMPILE_STATUS_ARB, &fragCompiled);
		printInfoLog(this->m_MipFragObject);

		//Attach the compiled shader object to a new program object
		this->m_MipProgrammObject = glCreateProgramObjectARB();
		glAttachObjectARB(this->m_MipProgrammObject, this->m_MipFragObject);
		glDeleteObjectARB(this->m_MipFragObject);
		break;

	case PHONG_DVR_SHADER: 
		//Initialize the specific shader object
		this->m_PhongDVRFragObject = glCreateShaderObjectARB(GL_FRAGMENT_SHADER_ARB);
		//Load the sourcecode into the shader object
		glShaderSourceARB(this->m_PhongDVRFragObject, 1, &fragmentCode, NULL);
		//Compile the new shader object and output the log into the standard log file
		glCompileShaderARB(this->m_PhongDVRFragObject);
		if(printOpenGLError(__FILE__,__LINE__))
			break;
		
		glGetObjectParameterivARB(this->m_PhongDVRFragObject,GL_OBJECT_COMPILE_STATUS_ARB, &fragCompiled);
		printInfoLog(this->m_PhongDVRFragObject);

		//Attach the compiled shader object to a new program object
		this->m_PhongProgrammObject = glCreateProgramObjectARB();
		glAttachObjectARB(this->m_PhongProgrammObject, this->m_PhongDVRFragObject);
		glDeleteObjectARB(this->m_PhongDVRFragObject);
		break;

	case CONTOURSHADER: 
		//Initialize the specific shader object
		this->m_ContourFragObject = glCreateShaderObjectARB(GL_FRAGMENT_SHADER_ARB);
		//Load the sourcecode into the shader object
		glShaderSourceARB(this->m_ContourFragObject, 1, &fragmentCode, NULL);
		//Compile the new shader object and output the log into the standard log file
		glCompileShaderARB(this->m_ContourFragObject);
		if(printOpenGLError(__FILE__,__LINE__))
			break;
		
		glGetObjectParameterivARB(this->m_ContourFragObject,GL_OBJECT_COMPILE_STATUS_ARB, &fragCompiled);
		printInfoLog(this->m_ContourFragObject);

		//Attach the compiled shader object to a new program object
		this->m_ContourProgrammObject = glCreateProgramObjectARB();
		glAttachObjectARB(this->m_ContourProgrammObject, this->m_ContourFragObject);
		glDeleteObjectARB(this->m_ContourFragObject);
		break;

	case TONESHADER: 
		//Initialize the specific shader object
		this->m_ToneFragObject = glCreateShaderObjectARB(GL_FRAGMENT_SHADER_ARB);
		//Load the sourcecode into the shader object
		glShaderSourceARB(this->m_ToneFragObject, 1, &fragmentCode, NULL);
		//Compile the new shader object and output the log into the standard log file
		glCompileShaderARB(this->m_ToneFragObject);
		if(printOpenGLError(__FILE__,__LINE__))
			break;

		glGetObjectParameterivARB(this->m_ToneFragObject,GL_OBJECT_COMPILE_STATUS_ARB, &fragCompiled);
		printInfoLog(this->m_ToneFragObject);

		//Attach the compiled shader object to a new program object
		this->m_ToneProgrammObject = glCreateProgramObjectARB();
		glAttachObjectARB(this->m_ToneProgrammObject, this->m_ToneFragObject);
		glDeleteObjectARB(this->m_ToneFragObject);
		break;

	default:
		initalizeError = true;
		break;

	}

	if(initalizeError)
	{
		throw new Exception(S"The given fragment shader type does not exsist!");
		return;
	}

	if(!fragCompiled)
	{
		throw new Exception(S"An error during compiling has occured!\nSee the log file for further information");
		return;
	}

	switch(n_ShaderType)
	{
	case MIPSHADER: 
		//Link the programm object and output the log into the standard log file
		glLinkProgramARB(this->m_MipProgrammObject);
		glGetObjectParameterivARB(this->m_MipProgrammObject, GL_OBJECT_LINK_STATUS_ARB, &fragLinked);
		printInfoLog(this->m_MipProgrammObject);
		printOpenGLError(__FILE__,__LINE__);
		break;
	
	case PHONG_DVR_SHADER: 
		//Link the programm object and output the log into the standard log file
		glLinkProgramARB(this->m_PhongProgrammObject);
		glGetObjectParameterivARB(this->m_PhongProgrammObject, GL_OBJECT_LINK_STATUS_ARB, &fragLinked);
		printInfoLog(this->m_PhongProgrammObject);
		printOpenGLError(__FILE__,__LINE__);
		break;

	case CONTOURSHADER: 
		//Link the programm object and output the log into the standard log file
		glLinkProgramARB(this->m_ContourProgrammObject);
		glGetObjectParameterivARB(this->m_ContourProgrammObject, GL_OBJECT_LINK_STATUS_ARB, &fragLinked);
		printInfoLog(this->m_ContourProgrammObject);
		printOpenGLError(__FILE__,__LINE__);
		break;

	case TONESHADER: 
		//Link the programm object and output the log into the standard log file
		glLinkProgramARB(this->m_ToneProgrammObject);
		glGetObjectParameterivARB(this->m_ToneProgrammObject, GL_OBJECT_LINK_STATUS_ARB, &fragLinked);
		printInfoLog(this->m_ToneProgrammObject);
		printOpenGLError(__FILE__,__LINE__);
		break;

	default: break;
	}

	if(!fragLinked)
		throw new Exception(S"An error during linking has occured!\nSee the log file for further information");

	delete sourceCode;

	return;
}