#include "StdAfx.h"
#include ".\GradientEstimationSW.h"
#include ".\DataLoader.h"
#using <mscorlib.dll>

using namespace System;
using namespace IDVR::VolumeRenderer::SWVolumeRenderer::RenderCalculations;
using namespace IDVR::VolumeRenderer::RenderPrimitives;
using namespace IDVR::Volume;
using namespace IDVR::Volume::DataStructures;

//Standard constructor.
GradientEstimationSW::GradientEstimationSW(void)
{
	//maximum possible value of the gradient by central differences
	this->m_scaleFactorDifferences = Math::Sqrt(Math::Pow((DataLoader::DensityScope / 2.0), 2.0) * 3.0);
	//maximum possible value of the gradient by 4d linear regression
	this->m_scaleFactorRegression = Math::Sqrt(3.0 * Math::Pow(4 * DataLoader::DensityScope * 1.0 / Math::Sqrt(2.0) +
															   4 * DataLoader::DensityScope * 1.0 / Math::Sqrt(3.0) +
															   DataLoader::DensityScope, 2.0));	
}

//Calcultaes the gradient with central differences.
float GradientEstimationSW::calculate_NormalEstimation(unsigned short n_Column, unsigned short n_Row, unsigned short n_Depth, VolumeData* n_DataSet) __gc[]
{
	float newGradientValues __gc[] = new float __gc[3];

	try
	{
		newGradientValues[X] = 0.5f * ((float)n_DataSet->VoxelDataset->getVoxelDensity(n_Column + 1, n_Row, n_Depth)
					- (float)n_DataSet->VoxelDataset->getVoxelDensity(n_Column - 1, n_Row, n_Depth));
	}
	catch(Exception*) 
	{
		newGradientValues[X] = 0;
	}
	try
	{
		newGradientValues[Y] = 0.5f * ((float)n_DataSet->VoxelDataset->getVoxelDensity(n_Column, n_Row + 1, n_Depth)
					- (float)n_DataSet->VoxelDataset->getVoxelDensity(n_Column, n_Row - 1, n_Depth));
	}
	catch(Exception*) 
	{
		newGradientValues[Y] = 0;
	}
	try
	{
		newGradientValues[Z] = 0.5f * ((float)n_DataSet->VoxelDataset->getVoxelDensity(n_Column, n_Row, n_Depth + 1)
					- (float)n_DataSet->VoxelDataset->getVoxelDensity(n_Column, n_Row, n_Depth - 1));
	}
	catch(Exception*) 
	{
		newGradientValues[Z] = 0;
	}

	return newGradientValues;
}

//Calcultaes the gradient with 4d linear regression.
float GradientEstimationSW::calculate_4DLinerRegressionEstimation(unsigned short n_Column, unsigned short n_Row, unsigned short n_Depth, VolumeData* n_DataSet) __gc[]
{
	double a = 0, b = 0, c = 0;
	const float weightedEuclidanDistanceInSlice = 1.0f / (float)Math::Sqrt(2.0),
		        weightedEuclidanDistanceInSpace = 1.0f / (float)Math::Sqrt(3.0);	

	float weightedVoxelDensities __gc[] = new float __gc[27];

	try {
		weightedVoxelDensities[0]  = n_DataSet->VoxelDataset->getVoxelDensity(n_Column - 1, n_Row - 1, n_Depth - 1) * weightedEuclidanDistanceInSpace;
		weightedVoxelDensities[1]  = n_DataSet->VoxelDataset->getVoxelDensity(n_Column,     n_Row - 1, n_Depth - 1) * weightedEuclidanDistanceInSlice;
		weightedVoxelDensities[2]  = n_DataSet->VoxelDataset->getVoxelDensity(n_Column + 1, n_Row - 1, n_Depth - 1) * weightedEuclidanDistanceInSpace;
		weightedVoxelDensities[3]  = n_DataSet->VoxelDataset->getVoxelDensity(n_Column - 1, n_Row,	   n_Depth - 1) * weightedEuclidanDistanceInSlice;
		weightedVoxelDensities[4]  = n_DataSet->VoxelDataset->getVoxelDensity(n_Column,	    n_Row,	   n_Depth - 1);
		weightedVoxelDensities[5]  = n_DataSet->VoxelDataset->getVoxelDensity(n_Column + 1, n_Row,	   n_Depth - 1) * weightedEuclidanDistanceInSlice; 
		weightedVoxelDensities[6]  = n_DataSet->VoxelDataset->getVoxelDensity(n_Column - 1, n_Row + 1, n_Depth - 1) * weightedEuclidanDistanceInSpace;
		weightedVoxelDensities[7]  = n_DataSet->VoxelDataset->getVoxelDensity(n_Column,     n_Row + 1, n_Depth - 1) * weightedEuclidanDistanceInSlice;
		weightedVoxelDensities[8]  = n_DataSet->VoxelDataset->getVoxelDensity(n_Column + 1, n_Row + 1, n_Depth - 1) * weightedEuclidanDistanceInSpace;
		weightedVoxelDensities[9]  = n_DataSet->VoxelDataset->getVoxelDensity(n_Column - 1, n_Row - 1, n_Depth) * weightedEuclidanDistanceInSlice;
		weightedVoxelDensities[10] = n_DataSet->VoxelDataset->getVoxelDensity(n_Column,     n_Row - 1, n_Depth); 
		weightedVoxelDensities[11] = n_DataSet->VoxelDataset->getVoxelDensity(n_Column + 1, n_Row - 1, n_Depth) * weightedEuclidanDistanceInSlice;
		weightedVoxelDensities[12] = n_DataSet->VoxelDataset->getVoxelDensity(n_Column - 1, n_Row,	   n_Depth);
		weightedVoxelDensities[13] = 0.0f;
		weightedVoxelDensities[14] = n_DataSet->VoxelDataset->getVoxelDensity(n_Column + 1, n_Row,	   n_Depth);
		weightedVoxelDensities[15] = n_DataSet->VoxelDataset->getVoxelDensity(n_Column - 1, n_Row + 1, n_Depth) * weightedEuclidanDistanceInSlice;
		weightedVoxelDensities[16] = n_DataSet->VoxelDataset->getVoxelDensity(n_Column,     n_Row + 1, n_Depth);
		weightedVoxelDensities[17] = n_DataSet->VoxelDataset->getVoxelDensity(n_Column + 1, n_Row + 1, n_Depth) * weightedEuclidanDistanceInSlice;
		weightedVoxelDensities[18] = n_DataSet->VoxelDataset->getVoxelDensity(n_Column - 1, n_Row - 1, n_Depth + 1) * weightedEuclidanDistanceInSpace;
		weightedVoxelDensities[19] = n_DataSet->VoxelDataset->getVoxelDensity(n_Column,     n_Row - 1, n_Depth + 1) * weightedEuclidanDistanceInSlice; 
		weightedVoxelDensities[20] = n_DataSet->VoxelDataset->getVoxelDensity(n_Column + 1, n_Row - 1, n_Depth + 1) * weightedEuclidanDistanceInSpace; 
		weightedVoxelDensities[21] = n_DataSet->VoxelDataset->getVoxelDensity(n_Column - 1, n_Row,	   n_Depth + 1) * weightedEuclidanDistanceInSlice;
		weightedVoxelDensities[22] = n_DataSet->VoxelDataset->getVoxelDensity(n_Column,	    n_Row,     n_Depth + 1);
		weightedVoxelDensities[23] = n_DataSet->VoxelDataset->getVoxelDensity(n_Column + 1, n_Row,	   n_Depth + 1) * weightedEuclidanDistanceInSlice; 
		weightedVoxelDensities[24] = n_DataSet->VoxelDataset->getVoxelDensity(n_Column - 1, n_Row + 1, n_Depth + 1) * weightedEuclidanDistanceInSpace;
		weightedVoxelDensities[25] = n_DataSet->VoxelDataset->getVoxelDensity(n_Column,     n_Row + 1, n_Depth + 1) * weightedEuclidanDistanceInSlice;
		weightedVoxelDensities[26] = n_DataSet->VoxelDataset->getVoxelDensity(n_Column + 1, n_Row + 1, n_Depth + 1) * weightedEuclidanDistanceInSpace;

		a = //Slice with depth z -> voxel nummers 9,11,12,14,15,17 - 10, 13 and 16 fell out because of xk = 0
			/*#9*/ -weightedVoxelDensities[9] + /*#11*/ weightedVoxelDensities[11] - /*#12*/weightedVoxelDensities[12] +
			/*#14*/weightedVoxelDensities[14] - /*#15*/weightedVoxelDensities[15] +  /*#17*/weightedVoxelDensities[17] -

			//Slice with depth z-1 -> voxel nummers 0,2,3,5,6,8 - 1, 4 and 7 fell out because of xk = 0
			/*#0*/weightedVoxelDensities[0] + /*#2*/weightedVoxelDensities[2] - /*#3*/weightedVoxelDensities[3] +
			/*#5*/weightedVoxelDensities[5] - /*#6*/weightedVoxelDensities[6] + /*#8*/weightedVoxelDensities[8] -
			
			//Slice with depth z+1 -> voxel nummers 18,20,21,23,24,26 - 19, 22 and 25 fell out because of xk = 0
			/*#18*/weightedVoxelDensities[18] + /*#20*/weightedVoxelDensities[20] - /*#21*/weightedVoxelDensities[21] + 
			/*#23*/weightedVoxelDensities[23] - /*#24*/weightedVoxelDensities[24] + /*#26*/weightedVoxelDensities[26];

		b = //Slice with depth z -> voxel nummers 9,10,11,15,16,17 - 12, 13 and 14 fell out because of yk = 0
			/*#9*/ -weightedVoxelDensities[9] - /*#10*/weightedVoxelDensities[10] - /*#11*/weightedVoxelDensities[11] +
			/*#15*/weightedVoxelDensities[15] + /*#16*/weightedVoxelDensities[16] + /*#17*/weightedVoxelDensities[17] -

			//Slice with depth z-1 -> voxel nummers 0,1,2,6,7,8 - 3, 4 and 5 fell out because of yk = 0
			/*#0*/weightedVoxelDensities[0] - /*#1*/weightedVoxelDensities[1] - /*#2*/weightedVoxelDensities[2] +
			/*#6*/weightedVoxelDensities[6] + /*#7*/weightedVoxelDensities[7] + /*#8*/weightedVoxelDensities[8] -
			
			//Slice with depth z+1 -> voxel nummers 18,19,20,24,25,26 - 21, 22 and 23 fell out because of yk = 0
			/*#18*/weightedVoxelDensities[18] - /*#19*/weightedVoxelDensities[19] - /*#20*/weightedVoxelDensities[20] +
			/*#24*/weightedVoxelDensities[24] + /*#25*/weightedVoxelDensities[25] + /*#26*/weightedVoxelDensities[26];

		c = //Slice with depth z-1 -> voxel nummers 0,1,2,3,4,5,6,7,8
			/*#0*/ -weightedVoxelDensities[0] - /*#1*/weightedVoxelDensities[1] - /*#2*/weightedVoxelDensities[2] -
			/*#3*/ weightedVoxelDensities[3] -  /*#4*/weightedVoxelDensities[4] - /*#5*/weightedVoxelDensities[5] -
			/*#6*/ weightedVoxelDensities[6] -  /*#7*/weightedVoxelDensities[7] - /*#8*/weightedVoxelDensities[8] +
			
			//Slice with depth z+1 -> voxel nummers 18,19,20,21,22,23,24,25,26
			/*#18*/weightedVoxelDensities[18] + /*#19*/weightedVoxelDensities[19] + /*#20*/weightedVoxelDensities[20] +
			/*#21*/weightedVoxelDensities[21] + /*#22*/weightedVoxelDensities[22] + /*#23*/weightedVoxelDensities[23] +
			/*#24*/weightedVoxelDensities[24] + /*#25*/weightedVoxelDensities[25] + /*#26*/weightedVoxelDensities[26];
	}
	catch(Exception*) 
	{
		//The calculated xoxel coordinates are otside of the valid voxel dataset, the gradient will be set to 0 by default
		a = b = c = 0;
	}

	float newGradientValues __gc[] = {(float)a, (float)b, (float)c};
	return newGradientValues;
}

//Returns the scale factor of central differences.
double GradientEstimationSW::get_scaleFactorDifferences(void)
{
	return this->m_scaleFactorDifferences;
}

//Returns the scale factor of 4d linear regression.
double GradientEstimationSW::get_scaleFactorRegression(void)
{
	return this->m_scaleFactorRegression;
}