#pragma once
#include "IGradientInterpolation.h"
#include "GradientEstimationSW.h"

namespace IDVR {
namespace VolumeRenderer {
namespace SWVolumeRenderer {
namespace RenderCalculations {

using namespace IDVR::Interfaces;
using namespace IDVR::Tools;

/**
<B>Class for the gradient interpolation with precalculated gradients..</B>

namespace: IDVR::VolumeRenderer::SWVolumeRenderer::RenderCalculations

Baseclass: IDVR::Interfaces::IGradientInterpolation

This class provides methods to interpolate the gradient of a given raysample if the gradients have already
been estimated before.

@see GradientInterpolationWithoutPreCalcSW
@author <i>Christopher Thurnher</i>
@version 1.0
@date 02-19-2005
*/

public __gc class GradientInterpolationWithPreCalcSW : public IGradientInterpolation
{
public:

	/**
	Standard constructor.
	*/
	GradientInterpolationWithPreCalcSW(void);

	/**
	Interpolates the gradient of a given raysample.
	@param intMode the interpolation mode.
	@param grMode the gradient estimation mode.
	@param n_RaySample the given raysample.
	@param n_DataSet the dataset.
	@param x_down x-value round down.
	@param x_up x-value round up.
	@param y_down y-value round down.
	@param y_up y-value round up.
	@param z_down z-value round down.
	@param z_up z-value round up.
	@return The interpolated gradient.
	*/
	virtual Gradient* interpolateGradient(unsigned char intMode, unsigned char grMode, RaySample* n_RaySample, VolumeData* n_DataSet, 
											double x_down, double x_up, double y_down, double y_up, double z_down, double z_up);
	
	__value enum Indices{X = 0, Y, Z};

	static const unsigned char BY_NEARESTNEIGHBOUR = 0; /**< Interpolation by nearest neighbour. */
	static const unsigned char BY_TRILINIEAR = 1; /** Trilinear interpolation.*/
};
}
}
}
}