#pragma once

#include ".\RaySample.h"

#using <mscorlib.dll>

namespace IDVR {
namespace Interfaces {

using namespace IDVR::VolumeRenderer::RenderPrimitives;

/*
Interface for the classification model.

namespace: IDVR::Interfaces

This interface provides methods to calculate an opacity value to a given raysample. This is the first rendering
step which consists of three steps: classification - shading - compositing
*/

public __gc __interface IClassificationModel
{
	/*
	Calculates the opacity value for a given raysample.
	*/
	RaySample* doClassification(RaySample* raySamp) = 0;
};
}
}

/*! \interface IDVR::Interfaces::IClassificationModel
    \brief <B>Interface for the classification model.</B>

	namespace: IDVR::Interfaces
	<br><br>
	This interface provides methods to calculate an opacity value to a given raysample. This is the first rendering
	step which consists of three steps: classification - shading - compositing

	@see IDVR::VolumeRenderer::SWVolumeRenderer::ClassificationModels::LinearClassificationModelSW - the linear classification model.
	@see IDVR::VolumeRenderer::SWVolumeRenderer::ClassificationModels::TFClassificationModelSW - the transfer function classification model.
	@author <i>Christopher Thurnher</i>
	@version 1.0
	@date 02-18-2005

	<hr>
	<b>Public member function:</b><br><br>
	<b>RaySample *IClassificationModel::doClassification(RaySample *raySamp)</b><br><br>
    Calculates the opacity value for a given raysample.
    \param raySamp the raysample with the current density value.
    \return The classificated raysample which now contains a basic opacity and color value.

 */