#pragma once

#include ".\IIDVRModulationModel.h"

namespace IDVR {
namespace VolumeRenderer {
namespace SWVolumeRenderer {
namespace IDVRModulationModels {

using namespace IDVR::Interfaces;
using namespace System::Drawing;

/**
<B>Class for the color and opacity modulator.</B>

Namespace: IDVR::VolumeRenderer::SWVolumeRenderer::IDVRModulationModels

Baseclass: IDVR::Interfaces::IIDVRModulationModel

This class does color and opacity modulation. It multiplies the calculated color and opacity values of a given
raysample with the level of sparsness calculated in the IDVRCompositing model.

@see IDVR::VolumeRenderer::SWVolumeRenderer::IDVRCompositingModels::IDVRMImPCompositingModelSW - the maximum importance driven volume rendering model
@author <i>Christopher Thurnher</i>
@version 1.0
@date 02-18-2005
*/
public __gc class IDVRColAndOpModulatorSW : public IIDVRModulationModel
{
public:

	/**
	Standard Constructor.
	*/
	IDVRColAndOpModulatorSW(void);

	/**
	Multiplies the calculated color and opacity values of a given raysample with the level of sparsness calculated in the IDVRCompositing model.
	@param n_RaySample the raysample that should be modulated.
	@param n_LevelsOfSparseness level of sparseness hashtable.
	@return The moulated rasample.
	*/
	virtual RaySample* doModulation(RaySample *n_RaySample, Hashtable *n_LevelsOfSparseness);
};
}
}
}
}