#pragma once

#include ".\RaySample.h"

#using <mscorlib.dll>

namespace IDVR {
namespace Interfaces {

using namespace IDVR::VolumeRenderer::RenderPrimitives;

/*
Interface for the shading model.

namespace: IDVR::Interfaces

This interface provides methods to calculate a color value to a given raysample. This is the second rendering
step which consists of three steps: classification - shading - compositing
*/

public __gc __interface IShadingModel
{
	/*
	Calculates the color value for a given raysample.
	@param n_RaySample the specified raysample.
	@return The modified raysample.
	*/
	RaySample* doShading(RaySample* n_RaySample) = 0;
};
}
}

/*! \interface IDVR::Interfaces::IShadingModel
	\brief <B>Interface for the shading model.</B>

	namespace: IDVR::Interfaces

	This interface provides methods to calculate a color value to a given raysample. This is the second rendering
	step which consists of three steps: classification - shading - compositing

	@see IDVR::VolumeRenderer::SWVolumeRenderer::ShadingModels::ContourShadingModelSW - the contour sahding model
	@see IDVR::VolumeRenderer::SWVolumeRenderer::ShadingModels::MipShadingModelSW - the mip shading model
	@see IDVR::VolumeRenderer::SWVolumeRenderer::ShadingModels::PhongShadingModelSW - the phong shading model
	@see IDVR::VolumeRenderer::SWVolumeRenderer::ShadingModels::ToneShadingModelSW - the tone shading model
	@author <i>Christopher Thurnher</i>
	@version 1.0
	@date 02-18-2005

	<hr>
	<b>Public member function:</b><br><br><ul>
	<li><b>RaySample* doShading(RaySample* n_RaySample)</b><br><br>
	 Calculates the color value for a given raysample.<br><br>
	 \param n_RaySample the specified raysample.<br><br>
	 \return The modified raysample.</li><br><br>
   </ul>
 */
