#include "StdAfx.h"
#include ".\LMipCompositingModelSW.h"
#using <mscorlib.dll>

using namespace IDVR::VolumeRenderer::SWVolumeRenderer::CompositingModels;

//Constructor.
LMipCompositingModelSW::LMipCompositingModelSW(float n_Threshold, char n_ProjectionMode)
{
	this->m_Threshold = n_Threshold;
	this->m_Intensity = new float __gc[3];
	this->m_Opacity = 0.0f;
	this->m_LocalMaximumFound = false;
	this->m_Density = 0;
	this->m_ProjectionMode = n_ProjectionMode;
}

//Standard destructor.
LMipCompositingModelSW::~LMipCompositingModelSW(void)
{
	this->m_Intensity = NULL;
}

//Blends the final intensity and opacity values of a given raysample.
void LMipCompositingModelSW::doCompositing(RaySample* n_RaySample)
{
	if(this->m_ProjectionMode == INTENSITY_PROJECTION)
	{
		if(!m_LocalMaximumFound)
		{
			float greyValue_CurrentSample = (n_RaySample->get_Color()[RED] + n_RaySample->get_Color()[GREEN] + n_RaySample->get_Color()[BLUE]) / 3;
			float greyValue_Calculated = (this->m_Intensity[RED] + this->m_Intensity[GREEN] + this->m_Intensity[BLUE]) / 3;

			if(greyValue_CurrentSample >= greyValue_Calculated)
			{
				//no local maximum found; go on
				this->m_Intensity = n_RaySample->get_Color();
				this->m_Opacity = n_RaySample->get_Opacity();
			}
			else if(greyValue_CurrentSample < greyValue_Calculated && greyValue_Calculated > this->m_Threshold)
			{
				//local maximum found; set value
				this->m_LocalMaximumFound = true;
			}	
		}
	}

	if(this->m_ProjectionMode == DENSITY_PROJECTION)
	{
		if(!m_LocalMaximumFound)
		{
			short density_CurrentSample = n_RaySample->get_Density();

			if(density_CurrentSample >= this->m_Density)
			{
				//no local maximum found; go on
				this->m_Intensity = n_RaySample->get_Color();
				this->m_Opacity = n_RaySample->get_Opacity();
				this->m_Density = density_CurrentSample;
			}
			else if(density_CurrentSample < this->m_Density && this->m_Density > this->m_Threshold)
			{
				//local maximum found; set value
				this->m_LocalMaximumFound = true;
			}	
		}
	}
}

//Blends the final pixelcolor with the given background color.
void LMipCompositingModelSW::doCompositingBackground(float n_BackColor __gc[])
{
/*
	//blends according to: calculated color : background color = calculated opacity : 1 - calculated opacity
	this->m_Intensity[RED] = this->m_Opacity * this->m_Intensity[RED] + (1 - this->m_Opacity) * backColor[RED];
	this->m_Intensity[GREEN] = this->m_Opacity * this->m_Intensity[GREEN] + (1 - this->m_Opacity) * backColor[GREEN];
	this->m_Intensity[BLUE] = this->m_Opacity * this->m_Intensity[BLUE] + (1 - this->m_Opacity) * backColor[BLUE];*/
	if (this->Opacity == 0.0)
	{
		this->m_Intensity[RED] = n_BackColor[RED];
		this->m_Intensity[GREEN] = n_BackColor[GREEN];
		this->m_Intensity[BLUE] = n_BackColor[BLUE];
	}
}

//Resets the values of the composition model.
void LMipCompositingModelSW::resetValues(void)
{
	this->m_Intensity = new float __gc[3];
	this->m_Opacity = 0.0f;
	this->m_Density = 0;
}

//Sets the intensity array.
void LMipCompositingModelSW::set_Intensity(float n_Intensity __gc[])
{
	this->m_Intensity = n_Intensity;
}

//Sets the opacity value.
void LMipCompositingModelSW::set_Opacity(float n_Opacity)
{
	this->m_Opacity = n_Opacity;
}

//Returns the intensity array.
float LMipCompositingModelSW::get_Intensity(void) __gc[]
{
	return this->m_Intensity;
}

//Returns the opacity value.
float LMipCompositingModelSW::get_Opacity(void)
{
	return this->m_Opacity;
}

//Returns true if a local maximum is found.
bool LMipCompositingModelSW::get_LocalMaximumFound(void)
{
	return this->m_LocalMaximumFound;
}

//Returns the threshold value.
float LMipCompositingModelSW::get_Threshold(void)
{
	return this->m_Threshold;
}

//Sets the localMaximumFound value.
void LMipCompositingModelSW::set_LocalMaximumFound(bool n_LocalMaximumFound)
{
	this->m_LocalMaximumFound = n_LocalMaximumFound;
}