#include "StdAfx.h"
#include ".\LightObjectGL.h"
#using <mscorlib.dll>

using namespace IDVR::Tools;
using namespace IDVR::Tools::OpenGLTools;
using namespace IDVR::VolumeRenderer::RenderPrimitives;

LightObjectGL::LightObjectGL(void)
{
	this->m_CanvasLength = 0;
	this->m_OffsetRotation = this->m_BaseRotation = this->m_DefaultRotation = NULL;
	this->m_CanvasBackColor = new float __gc[3];
	this->m_IsProxyDrawn = false;

	this->m_LightViewMode = ABSOLUTELIGHTVIEW;
}
LightObjectGL::LightObjectGL(int n_CanvasLength, Light *n_LightModel)
{
	this->m_CanvasLength = n_CanvasLength;
	this->m_LightModel = n_LightModel;
	this->m_CanvasBackColor = new float __gc[3];
	this->m_IsProxyDrawn = false;

	this->m_OffsetRotation =  Matrix4::CreateIdentityMatrix();
	this->m_BaseRotation   =  Matrix4::CreateIdentityMatrix();
	this->m_DefaultRotation = Matrix4::CreateIdentityMatrix();
	
	this->m_LightViewMode = ABSOLUTELIGHTVIEW;
}

LightObjectGL::LightObjectGL(int n_CanvasLength, Light *n_LightModel, Matrix4 *n_DefaultRotation)
{
	this->m_CanvasLength = n_CanvasLength;
	this->m_LightModel = n_LightModel;
	this->m_DefaultRotation = new Matrix4(n_DefaultRotation);

	this->m_CanvasBackColor = new float __gc[3];
	this->m_IsProxyDrawn = false;
	this->m_OffsetRotation = this->m_BaseRotation = Matrix4::CreateIdentityMatrix();
	
	this->m_LightViewMode = ABSOLUTELIGHTVIEW;
}

void LightObjectGL::Init(void)
{
	glClearColor(this->m_CanvasBackColor[0], this->m_CanvasBackColor[1], this->m_CanvasBackColor[2], 0.0);
	glViewport(0,0,this->m_CanvasLength,this->m_CanvasLength);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	glOrtho(0.0, this->m_CanvasLength, 0.0, this->m_CanvasLength, -this->m_CanvasLength, this->m_CanvasLength);		

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	//Enable shading features
	glEnable(GL_DEPTH_TEST);
    glEnable(GL_LINE_SMOOTH);
    glEnable(GL_POLYGON_SMOOTH);
	
	glFrontFace(GL_CW);
	glShadeModel(GL_SMOOTH);
	glPolygonMode(GL_FRONT, GL_FILL);

	this->m_ArrowObject = glGenLists(1);
	this->m_OrientationInterface = glGenLists(1);
	glNewList(this->m_ArrowObject, GL_COMPILE);
		defineArrowObject();
	glEndList();
	glNewList(this->m_OrientationInterface, GL_COMPILE);
		defineOrientationInterface();
	glEndList();

	//Enable color materials
	glColorMaterial(GL_FRONT_AND_BACK, GL_DIFFUSE);
	glEnable(GL_COLOR_MATERIAL);

	//Enable other features
	glEnable(GL_AUTO_NORMAL);
    glEnable(GL_NORMALIZE);
}

void LightObjectGL::Draw(void)
{
	if (!this->m_IsProxyDrawn)
	{
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	}
	glPushMatrix();
	glLoadIdentity();
	
	//Do transformation with taking the object center into account
	glTranslatef(this->m_CanvasLength * 0.85f, this->m_CanvasLength * 0.15f, 0.0f);
	glMultMatrixd(this->m_OffsetRotation->getOneDimArray());
	if (this->m_LightViewMode == ABSOLUTELIGHTVIEW)
		glMultTransposeMatrixdARB(this->m_BaseRotation->getOneDimArray());
	glMultTransposeMatrixdARB(this->m_DefaultRotation->getOneDimArray());
	glTranslatef(-(ArrowWidth * this->m_CanvasLength) / 2.0f, -(ArrowHeight * this->m_CanvasLength) / 2.0f, -(ArrowDepth * this->m_CanvasLength) / 2.0f);
	glCallList(this->m_ArrowObject);
	glPopMatrix();

	glPushMatrix();
	glLoadIdentity();
	glTranslatef(this->m_CanvasLength * 0.85f, this->m_CanvasLength * 0.15f, 0.0f);
	glCallList(this->m_OrientationInterface);
	glPopMatrix();

	glFlush();
}

void LightObjectGL::defineArrowObject(void)
{
	//Draw outlines of the arrow object
	glColor3d(0.0,0.0,0.0);
	glBegin(GL_LINE_LOOP);
		glVertex3f(0.0f, ArrowStepHeight * this->m_CanvasLength, ArrowDepth * this->m_CanvasLength);
		glVertex3f(0.0f, ArrowStepHeight * this->m_CanvasLength + ShaftHeight * this->m_CanvasLength, ArrowDepth * this->m_CanvasLength);
		glVertex3f(ShaftWidth * this->m_CanvasLength, ArrowStepHeight * this->m_CanvasLength + ShaftHeight * this->m_CanvasLength, ArrowDepth * this->m_CanvasLength);
		glVertex3f(ShaftWidth * this->m_CanvasLength, ArrowHeight * this->m_CanvasLength, ArrowDepth * this->m_CanvasLength);
		glVertex3f(ArrowWidth * this->m_CanvasLength, ArrowMiddleHeight * this->m_CanvasLength, ArrowDepth * this->m_CanvasLength);
		glVertex3f(ShaftWidth * this->m_CanvasLength, 0.0f, ArrowDepth * this->m_CanvasLength);
		glVertex3f(ShaftWidth * this->m_CanvasLength, ArrowStepHeight * this->m_CanvasLength, ArrowDepth * this->m_CanvasLength);
	glEnd();
	glBegin(GL_LINE_LOOP);
		glVertex3f(0.0f, ArrowStepHeight * this->m_CanvasLength, 0.0f);
		glVertex3f(0.0f, ArrowStepHeight * this->m_CanvasLength + ShaftHeight * this->m_CanvasLength, 0.0f);
		glVertex3f(ShaftWidth * this->m_CanvasLength, ArrowStepHeight * this->m_CanvasLength + ShaftHeight * this->m_CanvasLength, 0.0f); 
		glVertex3f(ShaftWidth * this->m_CanvasLength, ArrowHeight * this->m_CanvasLength, 0.0f);
		glVertex3f(ArrowWidth * this->m_CanvasLength, ArrowMiddleHeight * this->m_CanvasLength, 0.0f);
		glVertex3f(ShaftWidth * this->m_CanvasLength, 0.0f, 0.0f);
		glVertex3f(ShaftWidth * this->m_CanvasLength, ArrowStepHeight * this->m_CanvasLength, 0.0f);
	glEnd();
	glBegin(GL_LINES);
		glVertex3f(0.0f, ArrowStepHeight * this->m_CanvasLength, ArrowDepth * this->m_CanvasLength);
		glVertex3f(0.0f, ArrowStepHeight * this->m_CanvasLength, 0.0f);
		glVertex3f(0.0f, ArrowStepHeight * this->m_CanvasLength + ShaftHeight * this->m_CanvasLength, ArrowDepth * this->m_CanvasLength);
		glVertex3f(0.0f, ArrowStepHeight * this->m_CanvasLength + ShaftHeight * this->m_CanvasLength, 0.0f);
		glVertex3f(ShaftWidth * this->m_CanvasLength, ArrowStepHeight * this->m_CanvasLength + ShaftHeight * this->m_CanvasLength, ArrowDepth * this->m_CanvasLength);
		glVertex3f(ShaftWidth * this->m_CanvasLength, ArrowStepHeight * this->m_CanvasLength + ShaftHeight * this->m_CanvasLength, 0.0f); 
		glVertex3f(ShaftWidth * this->m_CanvasLength, ArrowHeight * this->m_CanvasLength, ArrowDepth * this->m_CanvasLength);
		glVertex3f(ShaftWidth * this->m_CanvasLength, ArrowHeight * this->m_CanvasLength, 0.0f);
		glVertex3f(ArrowWidth * this->m_CanvasLength, ArrowMiddleHeight * this->m_CanvasLength, ArrowDepth * this->m_CanvasLength);
		glVertex3f(ArrowWidth * this->m_CanvasLength, ArrowMiddleHeight * this->m_CanvasLength, 0.0f);
		glVertex3f(ShaftWidth * this->m_CanvasLength, 0.0f, ArrowDepth * this->m_CanvasLength);
		glVertex3f(ShaftWidth * this->m_CanvasLength, 0.0f, 0.0f);
		glVertex3f(ShaftWidth * this->m_CanvasLength, ArrowStepHeight * this->m_CanvasLength, ArrowDepth * this->m_CanvasLength);
		glVertex3f(ShaftWidth * this->m_CanvasLength, ArrowStepHeight * this->m_CanvasLength, 0.0f);
	glEnd();

	Color currentLigthColor;
	if (this->m_LightModel)
		 currentLigthColor = Color::FromArgb(this->m_LightModel->get_LightColor());
	else 
		currentLigthColor = Color::White;

	glColor4f((float)currentLigthColor.get_R() / 255.0f, (float)currentLigthColor.get_G() / 255.0f, (float)currentLigthColor.get_B() / 255.0f, 1.0f);
	//Define all faces of a 3-dim arrow which represent current light direction
	glBegin(GL_TRIANGLES);
		//front arrow peak face
		glVertex3d(ShaftWidth * this->m_CanvasLength,0.0,ArrowDepth * this->m_CanvasLength);
		glVertex3d(ShaftWidth * this->m_CanvasLength,ArrowHeight * this->m_CanvasLength,ArrowDepth * this->m_CanvasLength);
		glVertex3d(ArrowWidth * this->m_CanvasLength,ArrowMiddleHeight * this->m_CanvasLength,ArrowDepth * this->m_CanvasLength);

		//back arrow peak face
		glVertex3d(ShaftWidth * this->m_CanvasLength,0.0,0.0);
		glVertex3d(ShaftWidth * this->m_CanvasLength,ArrowHeight * this->m_CanvasLength,0.0);
		glVertex3d(ArrowWidth * this->m_CanvasLength,ArrowMiddleHeight * this->m_CanvasLength,0.0);
	glEnd();

	//back arrow face
	glBegin(GL_QUADS);
		glVertex3d(0.0,ArrowStepHeight * this->m_CanvasLength,0.0);
		glVertex3d(0.0,ArrowStepHeight * this->m_CanvasLength + ShaftHeight * this->m_CanvasLength,0.0);
		glVertex3d(ShaftWidth * this->m_CanvasLength,ArrowStepHeight * this->m_CanvasLength + ShaftHeight * this->m_CanvasLength,0.0);
		glVertex3d(ShaftWidth * this->m_CanvasLength,ArrowStepHeight * this->m_CanvasLength,0.0);
	glEnd();
	
	glBegin(GL_QUADS);
		//front arrow face
		glVertex3d(0.0,ArrowStepHeight * this->m_CanvasLength,ArrowDepth * this->m_CanvasLength);
		glVertex3d(0.0,ArrowStepHeight * this->m_CanvasLength + ShaftHeight * this->m_CanvasLength,ArrowDepth * this->m_CanvasLength);
		glVertex3d(ShaftWidth * this->m_CanvasLength,ArrowStepHeight * this->m_CanvasLength + ShaftHeight * this->m_CanvasLength,ArrowDepth * this->m_CanvasLength);
		glVertex3d(ShaftWidth * this->m_CanvasLength,ArrowStepHeight * this->m_CanvasLength,ArrowDepth * this->m_CanvasLength);
		
		//left shaft face
		glVertex3d(0.0,ArrowStepHeight * this->m_CanvasLength,ArrowDepth * this->m_CanvasLength);
		glVertex3d(0.0,ArrowStepHeight * this->m_CanvasLength, 0.0);
		glVertex3d(0.0,ArrowStepHeight * this->m_CanvasLength + ShaftHeight * this->m_CanvasLength,0.0);
		glVertex3d(0.0,ArrowStepHeight * this->m_CanvasLength + ShaftHeight * this->m_CanvasLength,ArrowDepth * this->m_CanvasLength);

		//top shaft face
		glVertex3d(0.0,ArrowStepHeight * this->m_CanvasLength + ShaftHeight * this->m_CanvasLength,ArrowDepth * this->m_CanvasLength);
		glVertex3d(0.0,ArrowStepHeight * this->m_CanvasLength + ShaftHeight * this->m_CanvasLength,0.0);
		glVertex3d(ShaftWidth * this->m_CanvasLength,ArrowStepHeight * this->m_CanvasLength + ShaftHeight * this->m_CanvasLength,0.0);
		glVertex3d(ShaftWidth * this->m_CanvasLength,ArrowStepHeight * this->m_CanvasLength + ShaftHeight * this->m_CanvasLength,ArrowDepth * this->m_CanvasLength);
	
		//bottom shaft face
		glVertex3d(0.0, ArrowStepHeight * this->m_CanvasLength, ArrowDepth * this->m_CanvasLength);
		glVertex3d(0.0, ArrowStepHeight * this->m_CanvasLength, 0.0);
		glVertex3d(ShaftWidth * this->m_CanvasLength, ArrowStepHeight * this->m_CanvasLength, 0.0);
		glVertex3d(ShaftWidth * this->m_CanvasLength, ArrowStepHeight * this->m_CanvasLength, ArrowDepth * this->m_CanvasLength);
	
		//top left arrow peak face
		glVertex3d(ShaftWidth * this->m_CanvasLength, ArrowStepHeight * this->m_CanvasLength + ShaftHeight * this->m_CanvasLength, ArrowDepth * this->m_CanvasLength);
		glVertex3d(ShaftWidth * this->m_CanvasLength, ArrowStepHeight * this->m_CanvasLength + ShaftHeight * this->m_CanvasLength , 0.0);
		glVertex3d(ShaftWidth * this->m_CanvasLength, ArrowHeight * this->m_CanvasLength, 0.0);
		glVertex3d(ShaftWidth * this->m_CanvasLength, ArrowHeight * this->m_CanvasLength, ArrowDepth * this->m_CanvasLength);
	
		//top right arrow peak face
		glVertex3d(ShaftWidth * this->m_CanvasLength, ArrowHeight * this->m_CanvasLength, ArrowDepth * this->m_CanvasLength);
		glVertex3d(ShaftWidth * this->m_CanvasLength, ArrowHeight * this->m_CanvasLength , 0.0);
		glVertex3d(ArrowWidth * this->m_CanvasLength, ArrowMiddleHeight * this->m_CanvasLength, 0.0);
		glVertex3d(ArrowWidth * this->m_CanvasLength, ArrowMiddleHeight * this->m_CanvasLength, ArrowDepth * this->m_CanvasLength);

		//bottom left arrow peak face
		glVertex3d(ShaftWidth * this->m_CanvasLength, 0.0 , ArrowDepth * this->m_CanvasLength);
		glVertex3d(ShaftWidth * this->m_CanvasLength, 0.0 , 0.0);
		glVertex3d(ShaftWidth * this->m_CanvasLength, ArrowStepHeight * this->m_CanvasLength, 0.0);
		glVertex3d(ShaftWidth * this->m_CanvasLength, ArrowStepHeight * this->m_CanvasLength, ArrowDepth * this->m_CanvasLength);
	
		//bottom right arrow peak face
		glVertex3d(ShaftWidth * this->m_CanvasLength, 0.0, ArrowDepth * this->m_CanvasLength);
		glVertex3d(ShaftWidth * this->m_CanvasLength, 0.0 , 0.0);
		glVertex3d(ArrowWidth * this->m_CanvasLength, ArrowMiddleHeight * this->m_CanvasLength, 0.0);
		glVertex3d(ArrowWidth * this->m_CanvasLength, ArrowMiddleHeight * this->m_CanvasLength, ArrowDepth * this->m_CanvasLength);
	glEnd();
}

void LightObjectGL::defineOrientationInterface(void)
{
	Color currentLigthColor;
	if (this->m_LightModel)
		 currentLigthColor = Color::FromArgb(this->m_LightModel->get_LightColor());
	else 
		currentLigthColor = Color::White;

	glColor4f((float)currentLigthColor.get_R() / 255.0f, (float)currentLigthColor.get_G() / 255.0f, (float)currentLigthColor.get_B() / 255.0f, 1.0f);
	//Draw Orientation Interface
	glBegin(GL_LINES);
		glVertex3f(0.0f, (ArrowWidth * this->m_CanvasLength) / 2.0f  + (Gap * this->m_CanvasLength), 0.0f);
		glVertex3f(0.0f, (ArrowWidth * this->m_CanvasLength) / 2.0f  + (Gap * this->m_CanvasLength) + (LineLength * this->m_CanvasLength), 0.0f);
		glVertex3f(0.0f, -(ArrowWidth * this->m_CanvasLength) / 2.0f - (Gap * this->m_CanvasLength), 0.0f);
		glVertex3f(0.0f, -(ArrowWidth * this->m_CanvasLength) / 2.0f - (Gap * this->m_CanvasLength) - (LineLength * this->m_CanvasLength), 0.0f);

		glVertex3f((ArrowWidth * this->m_CanvasLength) / 2.0f  + (Gap * this->m_CanvasLength), 0.0f, 0.0f);
		glVertex3f((ArrowWidth * this->m_CanvasLength) / 2.0f  + (Gap * this->m_CanvasLength) + (LineLength * this->m_CanvasLength), 0.0f, 0.0f);
		glVertex3f(-(ArrowWidth * this->m_CanvasLength) / 2.0f - (Gap * this->m_CanvasLength), 0.0f, 0.0f);
		glVertex3f(-(ArrowWidth * this->m_CanvasLength) / 2.0f - (Gap * this->m_CanvasLength) - (LineLength * this->m_CanvasLength), 0.0f, 0.0f);
	glEnd();

	CsGL::OpenGL::GLUquadric diskInterface = gluNewQuadric();
	gluDisk(diskInterface, (ArrowWidth * this->m_CanvasLength) / 2.0f  + (Gap * this->m_CanvasLength), 
						   (ArrowWidth * this->m_CanvasLength) / 2.0f  + (Gap * this->m_CanvasLength) + 2.0f, 
						    40, 4);
	gluDisk(diskInterface, (ArrowWidth * this->m_CanvasLength) / 2.0f  + (Gap * this->m_CanvasLength) + (LineLength * this->m_CanvasLength), 
						   (ArrowWidth * this->m_CanvasLength) / 2.0f  + (Gap * this->m_CanvasLength) + (LineLength * this->m_CanvasLength) + 2.0f, 
						    40, 4);

	gluDeleteQuadric(diskInterface);

	glColor4f(1.0f,0.0f,0.0f,1.0f);
	glPointSize(4.0);
	glBegin(GL_POINTS);
		glVertex3f(0.0f, (ArrowWidth * this->m_CanvasLength) / 2.0f  + (Gap * this->m_CanvasLength) + 1.0f, 1.0f);
		glVertex3f(0.0f, (ArrowWidth * this->m_CanvasLength) / 2.0f  + (Gap * this->m_CanvasLength) + (LineLength * this->m_CanvasLength) + 1.0f, 1.0f);
		glVertex3f(0.0f, -(ArrowWidth * this->m_CanvasLength) / 2.0f - (Gap * this->m_CanvasLength) - 1.0f, 1.0f);
		glVertex3f(0.0f, -(ArrowWidth * this->m_CanvasLength) / 2.0f - (Gap * this->m_CanvasLength) - (LineLength * this->m_CanvasLength) - 1.0f, 1.0f);

		glVertex3f((ArrowWidth * this->m_CanvasLength) / 2.0f  + (Gap * this->m_CanvasLength) + 1.0f, 0.0f, 1.0f);
		glVertex3f((ArrowWidth * this->m_CanvasLength) / 2.0f  + (Gap * this->m_CanvasLength) + (LineLength * this->m_CanvasLength) + 1.0f, 0.0f, 1.0f);
		glVertex3f(-(ArrowWidth * this->m_CanvasLength) / 2.0f - (Gap * this->m_CanvasLength) - 1.0f, 0.0f, 1.0f);
		glVertex3f(-(ArrowWidth * this->m_CanvasLength) / 2.0f - (Gap * this->m_CanvasLength) - (LineLength * this->m_CanvasLength) - 1.0f, 0.0f, 1.0f);
	glEnd();
	
}

void LightObjectGL::set_BaseRotation(Matrix4 *n_BaseRotation)
{
	this->m_BaseRotation = n_BaseRotation;
}

void LightObjectGL::set_OffsetRotation(Matrix4 *n_OffsetRotation)
{
	this->m_OffsetRotation = n_OffsetRotation;
}

Matrix4* LightObjectGL::get_BaseRotation(void)
{
	return new Matrix4(this->m_BaseRotation);
}

Matrix4* LightObjectGL::get_OffsetRotation(void)
{
	return new Matrix4(this->m_OffsetRotation);
}

void LightObjectGL::set_NewCanvasSize(int n_CanvasLength)
{
	this->m_CanvasLength = n_CanvasLength;
}

void LightObjectGL::set_CanvasBackColor(Color n_CanvasBackColor)
{
	this->m_CanvasBackColor[0] = (float)n_CanvasBackColor.get_R() / 255.0f;
	this->m_CanvasBackColor[1] = (float)n_CanvasBackColor.get_G() / 255.0f;
	this->m_CanvasBackColor[2] = (float)n_CanvasBackColor.get_B() / 255.0f;
}

void LightObjectGL::set_LightViewMode(unsigned char n_LightViewMode)
{
	this->m_LightViewMode = n_LightViewMode;
}

void LightObjectGL::setLightModel(Light *n_LightModel)
{
	this->m_LightModel = n_LightModel;
}

void LightObjectGL::set_ProxyDrawnMode(bool n_IsProxyDrawn)
{
	this->m_IsProxyDrawn = n_IsProxyDrawn;
}

void LightObjectGL::modifyOffsetRotation(Matrix4 *n_OffsetRotation)
{
	this->m_OffsetRotation = Matrix4::MultiplyMatices(n_OffsetRotation, this->m_OffsetRotation);
}

void LightObjectGL::modifyBaseRotation(Matrix4 *n_BaseRotation)
{
	this->m_BaseRotation = Matrix4::MultiplyMatices(n_BaseRotation, this->m_BaseRotation);
}


void LightObjectGL::resetToBaseLightDirection(void)
{
	this->m_OffsetRotation = Matrix4::CreateIdentityMatrix();
}