#pragma once
#include ".\IClassificationModel.h"

namespace IDVR {
namespace VolumeRenderer {
namespace SWVolumeRenderer {
namespace ClassificationModels {

using namespace IDVR::Interfaces;

/**
<B>Class for the linear classification model.</B>

Namespace: IDVR::VolumeRenderer::SWVolumeRenderer::ClassificationModels

Baseclass: IDVR::Interfaces::IClassificationModel

This class calculates the opacity value of a given raysample. It maps the opacity linear according to its density value.

@see TFClassificationModelSW - the transfer function classification model.
@author <i>Christopher Thurnher</i>
@version 1.0
@date 02-18-2005
*/

public __gc class LinearClassificationModelSW : public IClassificationModel
{

private:

	unsigned short m_DensityScope; /**< The scope of the density values of the dataset. */

public:

	/**
	Constructor.
	@param n_DensityScope the scope of the density values of the dataset.
	*/
	LinearClassificationModelSW(unsigned short n_DensityScope);

	/**
	Calculates the opacity for a given raysample linear according to its density value.
	@param n_RaySample the specified raysample.
	@return The modified raysample.
	*/
	virtual RaySample* doClassification(RaySample* n_RaySample);

};
}
}
}
}