#include "StdAfx.h"
#include "Neu.h"

#using <mscorlib.dll>

using namespace IDVR;

void Neu::initialize_Elements(void)
{
		
	//Set all rendercontrol GUI elements to default values given by the configuration
	reinitialize_GUIComponents();

	this->set_Text(this->MAINTITLE);
	this->trb_CanvasSize->set_Value(DataLoader::CanvasSize / this->trb_CanvasSize->get_Maximum());
	this->trb_ImageRes->set_Value(DataLoader::CanvasSize / this->trb_ImageRes->get_Maximum());

	this->stepLength = (float)(this->trb_CanvasSize->get_Value() * this->trb_CanvasSize->get_Maximum()) / 100.0f;

	//Initialize OpenGLCanvas object
	this->m_Canvas = new OpenGLCanvas(226, 80, 786, 799, DataLoader::CanvasSize, 
														 DataLoader::CanvasSize,
														 this->colDia_BackColor->get_Color());
	this->m_Canvas->set_MdiParent(this);
	this->m_Canvas->Show();

	//Initialize OpenGLPreviewCanvas object
	this->m_PreviewCanvas = new OpenGLPreviewCanvas(PREVIEWCANVAS_DEFAULTPOSX,
												    PREVIEWCANVAS_DEFAULTPOSY, 
													this->colDia_BackColor->get_Color(),
													Convert::ToSingle(this->nud_OpacityThreshold->Value),
													Convert::ToSingle(this->nud_GradientThreshold->Value),
													this);
	this->set_Location(Point(PREVIEWCANVAS_DEFAULTPOSX + this->m_PreviewCanvas->Size.Width, PREVIEWCANVAS_DEFAULTPOSY));
	this->m_PreviewCanvas->setControlSubItem(this->subItem_PreviewWindow);
	this->m_PreviewCanvas->setControlTbSubItem(this->tbSubItem_PreviewWindow);
	this->m_PreviewCanvas->set_ShowInTaskbar(true);
	this->m_PreviewCanvas->set_Location(Point(0,0));
	this->m_PreviewCanvas->Show();

	//display current image size
	String* temp  = String::Concat("Image Resolution: ", (trb_ImageRes->Value*trb_ImageRes->get_Maximum()).ToString(), " x ", (trb_ImageRes->Value*trb_ImageRes->get_Maximum()).ToString());
	lab_ImageRes->Text = temp;

	//display current canvas size
	temp  = String::Concat("Canvas Size: ", (trb_CanvasSize->Value*trb_CanvasSize->get_Maximum()).ToString(), " x ", (trb_CanvasSize->Value*trb_CanvasSize->get_Maximum()).ToString());
	lab_CanvasSize->Text = temp;

	//Set control GUI elements to status NODATALOADED
	setGUIElementsToNoDataLoadedStatus();

	//Initialize toolbar comboboxes
	this->tbc_RenderLevel->get_ComboBox()->set_SelectedItem(this->tbc_RenderLevel->get_ComboBox()->get_Items()->get_Item(1));
	this->tbc_ViewDirection->get_ComboBox()->set_SelectedItem(this->tbc_ViewDirection->get_ComboBox()->get_Items()->get_Item(2));
	this->tbc_VPRotationAxis->get_ComboBox()->set_SelectedItem(this->tbc_VPRotationAxis->get_ComboBox()->get_Items()->get_Item(0));
	this->tbc_LightRotationAxis->get_ComboBox()->set_SelectedItem(this->tbc_LightRotationAxis->get_ComboBox()->get_Items()->get_Item(0));

	//Initailize default rendering quality
	this->cob_SamplingFactor->set_SelectedIndex(NORMALSAMPLING);
	this->m_RenderType = SOFTWARE;
	this->m_GradientEstimationMode = GradientEstimationSW::BY_DIFFERENCES;
	this->m_LightViewMode = LightObjectGL::RELATIVLIGHTVIEW;

	this->m_DensityInterpolationMode   = InterpolationSW::BY_TRILINIEAR;
	this->m_GradientEstimationMode = InterpolationSW::BY_TRILINIEAR;
	this->m_VoxelIDInterpolationMode   = InterpolationSW::BY_NEARESTNEIGHBOUR;
	this->subItem_DensityNN->set_Checked(false);
	this->subItem_DensityTL->set_Checked(true);
	this->subItem_GradientNN->set_Checked(false);
	this->subItem_GradientTL->set_Checked(true);
	this->subItem_VoxelIDNN->set_Checked(true);
	this->subItem_VoxelIDTL->set_Checked(false);
	this->subItem_AllNN->set_Checked(false);
	this->subItem_AllTL->set_Checked(false);

	this->subItem_Central->set_Checked(false);
	this->subItem_Software->set_Checked(true);
	this->subItem_Regression->set_Checked(true);
	this->subItem_Hardware->set_Checked(false);

	this->subItem_PreviewWindow->set_Checked(true);

	//Initailize status bar display
	this->sbp_RenderMode->set_Text(S"SW");
	this->sbp_RenderMode->ToolTipText = S"Software renderer is activated";
	this->sbp_Gradient->set_Text(S"4DLinReg");
	this->sbp_Gradient->ToolTipText = S"Current GradientEstimation by 4D Reggression";
	this->sbp_DenityMode->set_Text(S"Density: TL");
	this->sbp_DenityMode->ToolTipText = S"Current DensityMode is TriLinear";
	this->sbp_GradientMode->set_Text(S"Gradient: TL");
	this->sbp_GradientMode->ToolTipText = S"Current GradientMode is TriLinear";
	this->sbp_VoxelIDMode->set_Text(S"VoxelID: NN");
	this->sbp_VoxelIDMode->ToolTipText = S"Current VoxelIDMode is Nearest Neighbour";

	//Set ActionListener for toolbar comboboxes
	this->tbc_FilePath->get_ComboBox()->SelectedIndexChanged += new System::EventHandler(this, perform_ActionTbComboboxSelectedIndexChanged);
	this->tbc_RenderLevel->get_ComboBox()->SelectedIndexChanged += new System::EventHandler(this, perform_ActionTbComboboxSelectedIndexChanged);
	this->tbc_ViewDirection->get_ComboBox()->SelectedIndexChanged += new System::EventHandler(this, perform_ActionTbComboboxSelectedIndexChanged);
	this->tbc_VPRotationAxis->get_ComboBox()->SelectedIndexChanged += new System::EventHandler(this, perform_ActionTbComboboxSelectedIndexChanged);
} 

void Neu::reinitialize_GUIComponents(void)
{
	//Load the default directories for the main file dialogs
	this->openDia_LoadDataFile->set_InitialDirectory(DataLoader::Default_DataFileDirectory);
	this->openDia_LoadMaskFile->set_InitialDirectory(DataLoader::Default_MaskFileDirectory);
	this->openDia_LoadTransFunc->set_InitialDirectory(DataLoader::Default_TransFuncFileDirectory);
	this->openDia_LoadConfigFile->set_InitialDirectory(DataLoader::Default_ConfigFileDirectory);
	this->saveDia_SaveTransFunc->set_InitialDirectory(DataLoader::Default_TransFuncFileDirectory);
	this->saveDia_SaveConfigFile->set_InitialDirectory(DataLoader::Default_ConfigFileDirectory);
		
	//display current ambient reflection
	this->nud_Ambient->set_Value(Convert::ToDecimal(DataLoader::Default_Ambient));
	//display current diffuse reflection
	this->nud_Diffuse->set_Value(Convert::ToDecimal(DataLoader::Default_Diffuse));
	//display current specular reflection	
	this->nud_Specular->set_Value(Convert::ToDecimal(DataLoader::Default_Specular));
	//display current specular exponent
	this->nud_SpExponent->set_Value(Convert::ToDecimal(DataLoader::Default_SpExponent));	

	//PhongShading default values
	this->nud_DepthK1->set_Value(Convert::ToDecimal(DataLoader::Default_K1DepthCueing));
	this->nud_DepthK2->set_Value(Convert::ToDecimal(DataLoader::Default_K2DepthCueing));

	//Optimation default values
	this->trb_SamplingDist->set_Value(Convert::ToInt32(DataLoader::Default_SamplingDistance * this->trb_SamplingDist->get_Maximum() * 0.5));
	this->nud_GradientThreshold->set_Value(Convert::ToDecimal(DataLoader::Default_GradientThreshold));
	this->nud_OpacityThreshold->set_Value(Convert::ToDecimal(DataLoader::Default_OpacityThreshold));
	this->tex_SamplingDist->set_Text(((float)trb_SamplingDist->Value / 10).ToString());	

	//Mip default settings
	this->trb_MipThreshold->set_Value(DataLoader::Default_MipThreshold);
	this->tex_MipThreshold->set_Text(trb_MipThreshold->Value.ToString());
	this->colDia_MipColor->set_Color(DataLoader::Default_MipColor);
	this->lab_MipColor->set_BackColor(this->colDia_MipColor->get_Color());
	this->rab_MipLinClassification->set_Checked(true);

	//Contour default settings
	this->trb_ContourThreshold->set_Value(Convert::ToInt32(Math::Round(DataLoader::Default_ContourThreshold * 100.0f)));
	this->tex_ContourThreshold->set_Text(Convert::ToString((float)trb_ContourThreshold->get_Value() / 100.0f));
	this->colDia_ContourColor->set_Color(DataLoader::Default_ContourColor);
	this->lab_ContourColor->set_BackColor(this->colDia_ContourColor->get_Color());
	this->nud_ContourGradient->set_Value(Convert::ToDecimal(DataLoader::Default_ContourGradientThreshold));
	this->nud_ContourExponent->set_Value(Convert::ToDecimal(DataLoader::Default_ContourExponent));
	this->nud_ContourKTF->set_Value(Convert::ToDecimal(DataLoader::Default_ContourKTF));
	this->rab_ContourLMip->set_Checked(true);

	//Tone default settings
	this->trb_ToneThreshold->set_Value(Convert::ToInt32(Math::Round(DataLoader::Default_ToneThreshold * 100.0f)));
	this->tex_ToneThreshold->set_Text(Convert::ToString((float)trb_ToneThreshold->get_Value() / 100.0f));
	this->colDia_ToneColorWarm->set_Color(DataLoader::Default_ToneWarmColor);
	this->colDia_ToneColorCold->set_Color(DataLoader::Default_ToneColdColor);
	this->lab_ToneColWarm->set_BackColor(this->colDia_ToneColorWarm->get_Color());
	this->lab_ToneColCold->set_BackColor(this->colDia_ToneColorCold->get_Color());
	this->nud_ToneGradient->set_Value(Convert::ToDecimal(DataLoader::Default_ToneMagnitudeThreshold));
	this->nud_ToneKW->set_Value(Convert::ToDecimal(DataLoader::Default_ToneKWarm));
	this->nud_ToneKC->set_Value(Convert::ToDecimal(DataLoader::Default_ToneKCold));
	this->nud_ToneKTF->set_Value(Convert::ToDecimal(DataLoader::Default_ToneKTF));
	this->rab_ToneLMip->set_Checked(true);
}

void Neu::setGUIElementsToReadyStatus(void)
{
	//Initialize image plane with default position
	unsigned short dim __gc[] = this->m_VolumeData->m_Dataset->get_VolumeDimension(),
				   width = dim[X],
		           heigth = dim[Y];

	if(width > heigth)
	{
		Vector3 *leftBottom = new Vector3(0.0, -Math::Round((width - heigth) / 2.0), -Math::Max(Math::Max(dim[X], dim[Y]), dim[Z]));

		this->m_ImagePlane = new ImagePlane(new Vector3(leftBottom->X, 
														   leftBottom->Y + heigth,
														   leftBottom->Z),
											new Vector3(leftBottom->X + heigth, 
														   leftBottom->Y + heigth,
														   leftBottom->Z),
											leftBottom,
											new Vector3(leftBottom->X + heigth, 
														   leftBottom->Y,
														   leftBottom->Z),
											this->trb_ImageRes->get_Value() * this->trb_ImageRes->get_Maximum(),
											dim);	
	}
	else if(width < heigth)
	{
		Vector3 *leftBottom = new Vector3(-Math::Round(heigth - width) / 2.0, 0.0, -Math::Max(Math::Max(dim[X], dim[Y]), dim[Z]));

		this->m_ImagePlane = new ImagePlane(new Vector3(leftBottom->X, 
														   leftBottom->Y + heigth,
														   leftBottom->Z),
											new Vector3(leftBottom->X + heigth, 
														   leftBottom->Y + heigth,
														   leftBottom->Z),
											leftBottom,
											new Vector3(leftBottom->X + heigth, 
														   leftBottom->Y,
														   leftBottom->Z),
											this->trb_ImageRes->get_Value() * this->trb_ImageRes->get_Maximum(),
											dim);	
	}
	else
	{
		Vector3 *leftBottom = new Vector3(0.0, 0.0, -Math::Max(Math::Max(dim[X], dim[Y]), dim[Z]));

		this->m_ImagePlane = new ImagePlane(new Vector3(leftBottom->X, 
														leftBottom->Y + heigth,
														leftBottom->Z),
											new Vector3(leftBottom->X + heigth, 
														leftBottom->Y + heigth,
														leftBottom->Z),
											leftBottom,
											new Vector3(leftBottom->X + heigth, 
														leftBottom->Y,
														leftBottom->Z),
											this->trb_ImageRes->get_Value() * this->trb_ImageRes->get_Maximum(),
											dim);		

	}
	

	//Initialize light position
	//this->m_Light = new Light(this->colDia_LightColor->get_Color().ToArgb(), new Vector3(0.0, 0.0, 1.0));
	this->m_Light->set_LightColor(this->colDia_LightColor->get_Color().ToArgb());
	this->m_Light->set_DefaultLightDirection(new Vector3(this->m_VolumeData->Default_Lightdirection_X, 
												         this->m_VolumeData->Default_Lightdirection_Y, 
												         this->m_VolumeData->Default_Lightdirection_Z));

	//Initialize the standard object with a default DVR render model
	Transferfunction *generalTF = Transferfunction::CreateGeneralTransfer(DataLoader::DensityScope);
	this->m_VolumeData->m_Dataset->getObjectMember((unsigned short)0)->
					set_RenderModel(new RendererSW(RendererSW::TF_PHONG_DVR, 
									new TFClassificationModelSW(generalTF),
									new PhongShadingModelSW(Convert::ToDouble(this->nud_Ambient->get_Value()),
															Convert::ToDouble(this->nud_Diffuse->get_Value()),
															Convert::ToDouble(this->nud_Specular->get_Value()),
															Convert::ToDouble(this->nud_SpExponent->get_Value()),
															Convert::ToDouble(this->nud_DepthK1->get_Value()),
															Convert::ToDouble(this->nud_DepthK2->get_Value()),
															generalTF),
									new DvrCompositingModelSW()));

	//Set title addition with the name of current loaded file
	FileInfo *info = new FileInfo(openDia_LoadDataFile->FileName);
	this->set_Text(String::Concat(this->MAINTITLE, S" - [loaded dataset] ", info->Name));
	
	//Initialize the filepath combobox
	if (!this->tbc_FilePath->get_ComboBox()->get_Items()->Contains(openDia_LoadDataFile->FileName))
	{
		this->tbc_FilePath->get_ComboBox()->get_Items()->Add(openDia_LoadDataFile->FileName);
		ComboBox::ObjectCollection* pathCollection = this->tbc_FilePath->get_ComboBox()->get_Items();
		this->tbc_FilePath->get_ComboBox()->set_SelectedItem(pathCollection->get_Item(pathCollection->get_Count() - 1));
	}

	//Deactivate all components which are used for Two-Level rendering and IDVR rendering
	setGUIElementsToNonTwoLevelRenderingStatus();

	//Initialize the IDVR comboboxes with default items
	//Add items to IDVR compositing and modulation comboboxes
	this->cob_IDVRCompositing->get_Items()->Clear();
	this->cob_IDVRModulation->get_Items()->Clear();
	this->cob_IDVRCompositing->get_Items()->Add(S"Maximum Importance");
	this->cob_IDVRModulation->get_Items()->Add(S"Color and Opacity");

	/*//Initailize the object selection combobox
	this->cob_TransObj->get_Items()->Clear();
	this->cob_TransObj->get_Items()->Add(this->m_VolumeData->m_Dataset->getGeneralDataObject()->get_ObjectDescription());
	this->cob_TransObj->set_SelectedIndex(0);*/
			
	//Activate all GUI elements which need the loaded Dataset to perform correctly
	//Activate menu items
	this->subItem_LoadMaskFile->Enabled = true;
	this->subItem_LoadTransFunc->Enabled = true;
	this->subItem_SaveTransFunc->Enabled = true;
	this->subItem_EditTransferFunction->Enabled = true;
	this->subItem_SliceViewer->Enabled = true;
	this->subItem_AnimationEditor->Enabled = true;

	//Activate toolbar items
	this->tbb_LoadMask->Enabled = true;
	this->tbMenuItem_Rendering->Enabled = true;
	this->tbSubItem_StartRendering->Enabled = true;
	this->tbSubItem_StopRendering->Enabled = false;
	this->tbc_ViewDirection->Enabled = true;
	
	//Activate rendering controls
	this->but_Render->Enabled = true;
	this->tab_RenderControl->Enabled = true;
	this->but_LoadMaskFile->Enabled = true;
	
	/*//Deactivate all components which are relevant for two level rendering
	this->chb_TwoLevelVR->Enabled = false;
	this->cob_TransObj->Enabled = false;*/

	/*//Deactivate all  Importance controls
	this->chb_IDVR->Enabled = false;

	//Set IDVR compositing and modulation comboboxes
	this->cob_IDVRCompositing->set_SelectedIndex(0);
	this->cob_IDVRModulation->set_SelectedIndex(0);

	//Initailize the IDVR object selection combobox and numericUpDown
	this->cob_IDVRObject->get_Items()->Clear();
	this->cob_IDVRObject->get_Items()->Add(this->m_VolumeData->m_Dataset->getGeneralDataObject()->get_ObjectDescription());
	this->cob_IDVRObject->set_SelectedIndex(0);
	this->nud_IDVRObjectImportance->set_Value(Convert::ToInt32(this->m_VolumeData->m_Dataset->getGeneralDataObject()->get_Importance()));
*/
	//Initailize status bar display
	//View
	this->sbp_VolumeDimension->set_Text(String::Concat(S"Current Volume Dimension: ", Convert::ToString(dim[0]),  S" x ", Convert::ToString(dim[1]), S" x ", Convert::ToString(dim[2]))); 
	this->sbp_ViewDirection->set_Text(String::Concat(S"View:   ", 
			Decimal::Round(Convert::ToDecimal(m_ImagePlane->get_ViewDirection()->X), 1).ToString(), S"  |  ", 
			Decimal::Round(Convert::ToDecimal(m_ImagePlane->get_ViewDirection()->Y), 1).ToString(), S"  |  ", 
			Decimal::Round(Convert::ToDecimal(m_ImagePlane->get_ViewDirection()->Z), 1).ToString()));

	//Light
	this->sbp_LightDirection->set_Text(String::Concat(S"Light:   ", 
		Decimal::Round(Convert::ToDecimal(m_Light->OffsetLightDirection->X), 1).ToString(), S"  |  ", 
		Decimal::Round(Convert::ToDecimal(m_Light->OffsetLightDirection->Y), 1).ToString(), S"  |  ", 
		Decimal::Round(Convert::ToDecimal(m_Light->OffsetLightDirection->Z), 1).ToString()));

	try
	{
		this->sbp_RayDistance->set_Text(String::Concat(S"Ray Distance: ", Convert::ToString(this->m_ImagePlane->get_RayDistanceWidth())->Substring(0, 6)));
	}
	catch (ArgumentOutOfRangeException*) 
	{
		this->sbp_RayDistance->set_Text(String::Concat(S"Ray Distance: ", Convert::ToString(this->m_ImagePlane->get_RayDistanceWidth())));
	}

	//At last set the flag for data loaded status and redraw the preview panel
	this->m_SystemStatus = READY;
	this->sbp_Status->set_Text(S"Ready");
	this->pan_TFPreview->Invalidate();

	if (this->m_VolumeData->m_Dataset->get_VolumeDimension()[0] >= this->m_VolumeData->m_Dataset->get_VolumeDimension()[1]
	           && this->m_VolumeData->m_Dataset->get_VolumeDimension()[0] >= this->m_VolumeData->m_Dataset->get_VolumeDimension()[2])
	{
		float temp __gc[] = {1.0f, 
							 (float)this->m_VolumeData->m_Dataset->get_VolumeDimension()[1] / this->m_VolumeData->m_Dataset->get_VolumeDimension()[0], 
							 (float)this->m_VolumeData->m_Dataset->get_VolumeDimension()[2] / this->m_VolumeData->m_Dataset->get_VolumeDimension()[0]};
		     this->m_Canvas->createProxyObjectGL(this->trb_CanvasSize->get_Value() * this->trb_CanvasSize->get_Maximum(),temp);
	}
	else if (this->m_VolumeData->m_Dataset->get_VolumeDimension()[1] > this->m_VolumeData->m_Dataset->get_VolumeDimension()[0]
		          && this->m_VolumeData->m_Dataset->get_VolumeDimension()[1] >= this->m_VolumeData->m_Dataset->get_VolumeDimension()[2])
	{
		float temp __gc[] = {(float)this->m_VolumeData->m_Dataset->get_VolumeDimension()[0] / this->m_VolumeData->m_Dataset->get_VolumeDimension()[1], 
							 1.0f, 
							 (float)this->m_VolumeData->m_Dataset->get_VolumeDimension()[2] / this->m_VolumeData->m_Dataset->get_VolumeDimension()[1]};
		    this->m_Canvas->createProxyObjectGL(this->trb_CanvasSize->get_Value() * this->trb_CanvasSize->get_Maximum(),temp);
	}
	else
	{
		float temp __gc[] = {(float)this->m_VolumeData->m_Dataset->get_VolumeDimension()[0] / this->m_VolumeData->m_Dataset->get_VolumeDimension()[2], 
							 (float)this->m_VolumeData->m_Dataset->get_VolumeDimension()[1] / this->m_VolumeData->m_Dataset->get_VolumeDimension()[2],
							 1.0f};
	    this->m_Canvas->createProxyObjectGL(this->trb_CanvasSize->get_Value() * this->trb_CanvasSize->get_Maximum(),temp);
	}

	Matrix4 *lightRotMatrix = Matrix4::MultiplyMatices(Matrix4::CreateRotZMatrix(Math::PI / 2.0), Matrix4::CreateRotYMatrix(Math::PI / 2.0));
	this->m_Canvas->createLightObjectGL(this->trb_CanvasSize->get_Value() * this->trb_CanvasSize->get_Maximum(),this->m_Light,lightRotMatrix);

	this->m_Canvas->set_SystemStatus(READY);
	this->m_Canvas->set_ImagePlaneModel(this->m_ImagePlane);
	this->m_Canvas->set_CanvasBackColor(this->colDia_BackColor->get_Color());
	this->m_Canvas->set_LightModel(this->m_Light);
	this->m_Canvas->setLightDirectionPanel(this->sbp_LightDirection);
	this->m_Canvas->setViewDirectionPanel(this->sbp_ViewDirection);
	this->m_Canvas->setLightViewMode(this->m_LightViewMode);

	this->m_PreviewCanvas->set_Dataset(this->m_VolumeData->getDataset());
	this->m_PreviewCanvas->set_SystemStatus(READY);
	this->m_PreviewCanvas->set_OriginalImagePlane(this->m_ImagePlane);
	this->m_PreviewCanvas->set_CanvasBackColor(this->colDia_BackColor->get_Color());
	this->m_PreviewCanvas->set_LightModel(this->m_Light);

	this->set_Cursor(System::Windows::Forms::Cursors::Default);

	//Delete thread and ProgressBar object in m_VolumeData
	this->m_VolumeData->setProgressbarHandler(NULL);
	this->m_VolumeData->setGUIHandler1(NULL);
	this->m_VolumeData->setGUIHandler2(NULL);
}

	void Neu::setGUIElementsToRenderedStatus(void)
	{	
		this->m_SystemStatus = READY;
		this->m_Canvas->set_SystemStatus(READY);
		this->m_PreviewCanvas->set_SystemStatus(READY);
		this->sbp_Status->set_Text(S"Ready");
		this->subItem_SaveRenderedPicture->Enabled = true;
		this->tbSubItem_StopRendering->Enabled = false;
		this->set_Cursor(System::Windows::Forms::Cursors::Default);

		//Call the garbage collector to clean up any memory leaks
		GC::Collect();
		GC::WaitForPendingFinalizers();
	}

	void Neu::setGUIElementsToNoDataLoadedStatus(void)
	{
		this->m_SystemStatus = NODATALOADED;
		this->m_Canvas->set_SystemStatus(NODATALOADED);

		//Delete currently shown image
		if (this->m_Canvas)
			this->m_Canvas->clearCanvas();

		if (this->m_PreviewCanvas)
		{
			this->m_PreviewCanvas->set_TwoLevelRendering(false);
			this->m_PreviewCanvas->set_IDVR(false);
		}

		this->subItem_EditTransferFunction->Enabled = false;
		this->subItem_SliceViewer->Enabled = false;
		this->subItem_AnimationEditor->Enabled = false;
		
		//Deactivate all GUI elements which need the loaded Dataset to perform correctly
		//Deactivate menu items
		this->subItem_LoadMaskFile->Enabled = false;
		this->subItem_LoadTransFunc->Enabled = false;
		this->subItem_SaveTransFunc->Enabled = false;
		this->subItem_SaveRenderedPicture->Enabled = false;

		//Deactivate toolbar items
		this->tbb_LoadMask->Enabled = false;
		this->tbMenuItem_Rendering->Enabled = false;
		this->tbSubItem_StartRendering->Enabled = false;
		this->tbSubItem_StopRendering->Enabled = false;
		this->tbc_ViewDirection->Enabled = false;
	
		//Deactivate rendering controls
		this->but_Render->Enabled = false;
		this->tab_RenderControl->Enabled = false;
		this->lab_TransObj->Enabled = false;
		this->cob_TransObj->Enabled = false;
		this->chb_TwoLevelVR->Enabled = false;
		this->but_LoadMaskFile->Enabled = false;
		this->but_DeleteVolumeObject->Enabled = false;

		//Deactivate Importance controls
		setGUIElementsToNonTwoLevelRenderingStatus();

		//Initalize a dataloader object that will contain all volume data voxels
		this->m_VolumeData = new DataLoader();
		this->m_ImagePlane = new ImagePlane();
		this->m_Light = new Light();
		this->m_TransFuncInput = new Transferfunction();

		//Initailize status bar display
		this->sbp_VolumeDimension->set_Text(S"Dataset Dimension:");
		this->sbp_ViewDirection->set_Text(S"View:");
		this->sbp_LightDirection->set_Text(S"Light:");
		this->sbp_RayDistance->set_Text(S"Ray Distance:");
		this->sbp_Status->set_Text(S"No data is loaded");

		//Delete thread and ProgressBar object in m_VolumeData
		this->m_VolumeData->setProgressbarHandler(NULL);
		this->m_VolumeData->setGUIHandler1(NULL);
		this->m_VolumeData->setGUIHandler2(NULL);
	}

	void Neu::setGUIElementsToTwoLevelRenderingStatus(void)
	{
		//Activate all components which are relevant for two level rendering
		this->lab_TransObj->Enabled = true;
		this->chb_TwoLevelVR->Enabled = true;
		if (!this->chb_TwoLevelVR->Checked)
			this->chb_TwoLevelVR->set_Checked(true);
		this->cob_TransObj->Enabled = true;

		//Activate Importance controls
		this->chb_IDVR->Enabled = true;
		this->m_VolumeData->m_Dataset->createGeneralIDVRObjects();

		//Initialize all object descriptions
		Array *objectIDs = this->m_VolumeData->m_Dataset->getObjectIDs();
		this->cob_TransObj->get_Items()->Clear();
		this->cob_IDVRObject->get_Items()->Clear();
		for (int i = 0; i <  objectIDs->Length; i++)
		{
			String *newItem = this->m_VolumeData->m_Dataset->getObjectMember(Convert::ToUInt16(objectIDs->get_Item(i), NumberFormatInfo::get_InvariantInfo()))->ObjectDescription;
			this->cob_TransObj->get_Items()->Add(newItem);
			this->cob_IDVRObject->get_Items()->Add(newItem);
		}

		this->cob_TransObj->set_SelectedIndex(0);
		this->cob_IDVRObject->set_SelectedIndex(0);
		this->cob_IDVRCompositing->set_SelectedIndex(0);
		this->cob_IDVRModulation->set_SelectedIndex(0);

		if (this->m_VolumeData->m_Dataset->ObjectCount == 2)
			this->but_DeleteVolumeObject->Enabled = true;

		this->set_Cursor(System::Windows::Forms::Cursors::Default);
	}

	void Neu::setGUIElementsToNonTwoLevelRenderingStatus(void)
	{
		//Deactivate all components which are relevant for two level rendering
		this->lab_TransObj->Enabled = false;
		this->chb_TwoLevelVR->Enabled = false;
		this->chb_TwoLevelVR->set_Checked(false);
		this->cob_TransObj->Enabled = false;

		//Deactivate Importance controls
		this->chb_IDVR->Enabled = false;
		this->chb_IDVR->set_Checked(false);
		this->lab_IDVRObject->Enabled = false;
		this->cob_IDVRObject->Enabled = false;
		this->nud_IDVRObjectImportance->Enabled = false;
		this->lab_IDVRCompositing->Enabled = false;
		this->cob_IDVRCompositing->Enabled = false;
		this->lab_IDVRModulation->Enabled = false;
		this->cob_IDVRModulation->Enabled = false;
		this->but_IDVRAssign->Enabled = false;
		this->but_IDVRReset->Enabled = false;
		this->lab_IDVRCountersinkSlope->Enabled = false;
		this->nud_IDVRCountersinkSlope->Enabled = false;
		this->lab_IDVRGScale->Enabled = false;
		this->nud_IDVRGScale->Enabled = false;

		this->set_Cursor(System::Windows::Forms::Cursors::Default);

		//Initialize all object descriptions
		if (this->m_VolumeData)
			if (this->m_VolumeData->m_Dataset)
			{
				this->cob_TransObj->get_Items()->Clear();
				this->cob_IDVRObject->get_Items()->Clear();
				this->cob_TransObj->get_Items()->Add(this->m_VolumeData->m_Dataset->getGeneralDataObject()->get_ObjectDescription());
				this->cob_IDVRObject->get_Items()->Add(this->m_VolumeData->m_Dataset->getGeneralDataObject()->get_ObjectDescription());
				this->cob_TransObj->set_SelectedIndex(0);
				this->cob_IDVRObject->set_SelectedIndex(0);
				this->nud_IDVRObjectImportance->set_Value(Convert::ToInt32(this->m_VolumeData->m_Dataset->getGeneralDataObject()->get_Importance()));
			}
	}

	void Neu::setCanvasImageElements(void)
	{
		//display current image size
		String* imageRes  = String::Concat("Image Resolution: ", (trb_ImageRes->Value*this->trb_ImageRes->get_Maximum()).ToString(), " x ", (trb_ImageRes->Value*this->trb_ImageRes->get_Maximum()).ToString());
		lab_ImageRes->Text = imageRes;

		String* canvasSize  = String::Concat("Canvas Size: ", (trb_CanvasSize->Value*this->trb_CanvasSize->get_Maximum()).ToString(), " x " ,(trb_CanvasSize->Value*this->trb_CanvasSize->get_Maximum()).ToString());
		lab_CanvasSize->Text = canvasSize;

		//resize canvas
		this->m_Canvas->set_OpenGLCanvasSize(this->trb_CanvasSize->get_Value() * this->trb_CanvasSize->get_Maximum(),
											 this->trb_CanvasSize->get_Value() * this->trb_CanvasSize->get_Maximum());

		//Calculate new ray distance and set new resolution to image plane object
		this->m_ImagePlane->set_Resolution(this->trb_ImageRes->get_Value() * this->trb_ImageRes->get_Maximum());

		if (this->m_SystemStatus != NODATALOADED)
		{
			this->m_ImagePlane->calculateRayDistance();
			try	
			{
				this->sbp_RayDistance->set_Text(String::Concat(S"Ray Distance: ", Convert::ToString(this->m_ImagePlane->get_RayDistanceWidth())->Substring(0, 6)));
			}
			catch (ArgumentOutOfRangeException*) 
			{
				this->sbp_RayDistance->set_Text(String::Concat(S"Ray Distance: ", Convert::ToString(this->m_ImagePlane->get_RayDistanceWidth())));
			}
		}

		this->stepLength = (float)(this->trb_CanvasSize->get_Value() * this->trb_CanvasSize->get_Maximum()) / 100.0f;
	}

void Neu::startRenderingProcess(void)
	{	
		this->m_SystemStatus = RENDERING;
		this->m_Canvas->set_SystemStatus(RENDERING);
		this->tbSubItem_StopRendering->Enabled = true;

		if (this->m_RenderType == SOFTWARE)
		{
			this->m_ImagePlane->set_Resolution(this->trb_ImageRes->get_Value()*this->trb_ImageRes->get_Maximum());

			RenderMachineSW* rayCaster = new RenderMachineSW(this->m_VolumeData->m_Dataset, this->m_ImagePlane, this->colDia_BackColor->get_Color(), __try_cast<IOpenGLCanvas*>(this->m_Canvas));
			rayCaster->set_DensityInterpolationMode(this->m_DensityInterpolationMode);
			rayCaster->set_GradientEstimationMode(this->m_GradientEstimationMode);
			rayCaster->set_VoxelIDInterpolationMode(this->m_VoxelIDInterpolationMode);
			rayCaster->set_GradientEstimationMode(this->m_GradientEstimationMode);
			rayCaster->set_SampingDistance(double(this->trb_SamplingDist->get_Value() / ((double)this->trb_SamplingDist->get_Maximum() / 2.0)));
			rayCaster->set_TwoLevelRendering(this->chb_TwoLevelVR->Checked);
			rayCaster->set_ImportanceDrivenRendering(this->chb_IDVR->Checked);
			rayCaster->set_OpacityThreshold(Convert::ToDouble(this->nud_OpacityThreshold->get_Value()));
			rayCaster->set_GradientThreshold(Convert::ToDouble(this->nud_GradientThreshold->get_Value()));

			Array* objectIDs = this->m_VolumeData->m_Dataset->getObjectIDs();

			for (int index = 0; index < objectIDs->get_Count(); index++)
			{
				IRenderer *renderer = this->m_VolumeData->m_Dataset->getObjectMember(index)->get_RenderModel();

				if (renderer->getShader()->GetType() == __typeof(PhongShadingModelSW))
				{
					PhongShadingModelSW *shader = __try_cast<PhongShadingModelSW*>(renderer->getShader());
					shader->setViewLightParameters(this->m_ImagePlane->get_ViewDirection(), 
												   this->m_ImagePlane->getCurrentLeftUpperPoint(), 
												   this->m_Light);
				}
				else if (renderer->getShader()->GetType() == __typeof(ContourShadingModelSW))
				{
					ContourShadingModelSW *shader = __try_cast<ContourShadingModelSW*>(renderer->getShader());
					shader->set_ViewDirection(this->m_ImagePlane->get_ViewDirection());
				}
				else if (renderer->getShader()->GetType() == __typeof(ToneShadingModelSW))
				{
					ToneShadingModelSW *shader = __try_cast<ToneShadingModelSW*>(renderer->getShader());
					shader->set_LightDirection(this->m_Light->OffsetLightDirection);
				}
			}
			this->set_Cursor(System::Windows::Forms::Cursors::WaitCursor);

			ProgressWin *progressWin;

			if (this->chb_IDVR->Checked)
				progressWin  = new ProgressWin(S"Rendering Process Control", S"The footprint is being generated now");
			else
				progressWin = new ProgressWin(S"Rendering Process Control", S"The data is being rendered now");
	
			System::Drawing::Point newLocation = System::Drawing::Point(
				this->get_Size().Width / 2 - (progressWin->get_Size().Width ),
				this->get_Size().Height / 4 - (progressWin->get_Size().Height / 2));

			progressWin->set_Location(System::Drawing::Point(newLocation));
			progressWin->set_MdiParent(this);
			progressWin->Show();

			//Initialize a thread for setting all Gui-elements to rendered status which will be started in the rayCasterHandler
			Thread *initializeHandler = new Thread(new ThreadStart(this, Neu::setGUIElementsToRenderedStatus));
			initializeHandler->set_IsBackground(true);

			rayCaster->setProgressWin(progressWin);
			rayCaster->setGUIHandler(initializeHandler);

			Thread *rayCasterHandler = new Thread(new ThreadStart(rayCaster, RenderMachineSW::start_Raycasting));

			rayCasterHandler->set_IsBackground(true);
			rayCasterHandler->set_Name(RENDERINGTHREAD);
			rayCasterHandler->Start();	
		 }
	}

void Neu::loadDatasetFile(String* n_FileName)
{
	try
	{
		//Prepare a progressbar window object for laoding the data
		ProgressWin *progressbarDisplay = new ProgressWin(S"Loading Process Control", S"The data is being loaded into the datastructure now.");
		progressbarDisplay->set_MdiParent(this);

		//Initialize a thread for setting all Gui-elements to load status which will be started in the dataReaderHandler
		Thread *initializeHandler1 = new Thread(new ThreadStart(this, Neu::setGUIElementsToReadyStatus));
		initializeHandler1->set_IsBackground(true);

		Thread *initializeHandler2 = new Thread(new ThreadStart(this, Neu::setGUIElementsToNoDataLoadedStatus));
		initializeHandler2->set_IsBackground(true);

		this->m_VolumeData->set_FileName(n_FileName);
		this->m_VolumeData->setProgressbarHandler(progressbarDisplay);
		this->m_VolumeData->setGUIHandler1(initializeHandler1);
		this->m_VolumeData->setGUIHandler2(initializeHandler2);
		this->m_VolumeData->set_GradientEstimationMode(this->m_GradientEstimationMode);

		this->set_Cursor(System::Windows::Forms::Cursors::WaitCursor);
				
		System::Drawing::Point newLocation = System::Drawing::Point(
		this->get_Size().Width / 2 - (progressbarDisplay->get_Size().Width ),
		this->get_Size().Height / 4 - (progressbarDisplay->get_Size().Height / 2));
				
		progressbarDisplay->set_Location(System::Drawing::Point(newLocation));
		progressbarDisplay->Show();

		//Initialize a thread for handling the loading of the data into the datastructure
		Thread *dataReaderHandler = new Thread(new ThreadStart(this->m_VolumeData, DataLoader::readDataFile));
		dataReaderHandler->set_IsBackground(true);
		dataReaderHandler->Start();
	}
	catch (Exception* e)
	{
		throw new Exception(e->Message, e);
	}
}

	void Neu::loadMaskDataFile(String* n_MaskFileName)
	{
		try
		{
			FileInfo *maskFileInfo = new FileInfo(this->openDia_LoadMaskFile->get_FileName());
			int voxelCount =  this->m_VolumeData->m_Dataset->VolumeDimension[0] * 
							this->m_VolumeData->m_Dataset->VolumeDimension[1] *
							this->m_VolumeData->m_Dataset->VolumeDimension[2];
			if (maskFileInfo->Length != Convert::ToInt64(Math::Ceiling((double)voxelCount / 8.0)))
			{
				MessageBox::Show(this, S"The choosen file' size is appearenty not equal to the base volume dataset. Please check whether a correct mask file is chosen.", S"Error by reading the .MASK-File", MessageBoxButtons::OK, MessageBoxIcon::Error);
				maskFileInfo = NULL;
				return;
			}
			else
				maskFileInfo = NULL;

			//Initialize default render model for new volume object
			Transferfunction *generalTF = Transferfunction::CreateGeneralTransfer(DataLoader::DensityScope);
			RendererSW *renderer =	new RendererSW(RendererSW::TF_PHONG_DVR, 
									new TFClassificationModelSW(generalTF),
									new PhongShadingModelSW(Convert::ToDouble(this->nud_Ambient->get_Value()),
														Convert::ToDouble(this->nud_Diffuse->get_Value()),
														Convert::ToDouble(this->nud_Specular->get_Value()),
														Convert::ToDouble(this->nud_SpExponent->get_Value()),
														Convert::ToDouble(this->nud_DepthK1->get_Value()),
														Convert::ToDouble(this->nud_DepthK2->get_Value()),
														generalTF),
									new DvrCompositingModelSW());	

			this->m_VolumeData->set_FileName(this->openDia_LoadMaskFile->get_FileName());
			this->set_Cursor(System::Windows::Forms::Cursors::WaitCursor);

			//Prepare a progressbar window object for laoding the mask data
			ProgressWin *progressbarDisplay = new ProgressWin(S"Loading Process Control", S"The mask data is being loaded into the datastructure now");
			progressbarDisplay->set_MdiParent(this);
			System::Drawing::Point newLocation = System::Drawing::Point(
					this->get_Size().Width / 2 - (progressbarDisplay->get_Size().Width ),
					this->get_Size().Height / 4 - (progressbarDisplay->get_Size().Height / 2));
			progressbarDisplay->set_Location(System::Drawing::Point(newLocation));

			Thread *guiElementHandler = new Thread(new ThreadStart(this, Neu::setGUIElementsToTwoLevelRenderingStatus));
			guiElementHandler->set_IsBackground(true);

			ObjectDescriptionInput* inputDialog = new ObjectDescriptionInput(this->m_VolumeData , guiElementHandler, renderer, this);
			inputDialog->setProgressBarDisplay(progressbarDisplay);
			inputDialog->Show();
		}
		catch (Exception* e)
		{
			throw new Exception(e->Message, e);
		}
	}

void Neu::setLightDirectionDisplay(void)
{
	this->sbp_LightDirection->set_Text(String::Concat(S"Light:   ", 
			Decimal::Round(Convert::ToDecimal(this->m_Light->OffsetLightDirection->X), 1).ToString(), S"  |  ", 
			Decimal::Round(Convert::ToDecimal(this->m_Light->OffsetLightDirection->Y), 1).ToString(), S"  |  ", 
			Decimal::Round(Convert::ToDecimal(this->m_Light->OffsetLightDirection->Z), 1).ToString()));
}

void Neu::setViewDirectionDisplay(void)
{
	this->sbp_ViewDirection->set_Text(String::Concat(S"View:   ", 
			Decimal::Round(Convert::ToDecimal(this->m_ImagePlane->ViewDirection->X), 1).ToString(), S"  |  ", 
			Decimal::Round(Convert::ToDecimal(this->m_ImagePlane->ViewDirection->Y), 1).ToString(), S"  |  ", 
			Decimal::Round(Convert::ToDecimal(this->m_ImagePlane->ViewDirection->Z), 1).ToString()));
}