#include "StdAfx.h"
#include "OpenGLCanvas.h"

#using <mscorlib.dll>

using namespace IDVR;

//Sets the size of the openGL canvas.
void OpenGLCanvas::set_Size(int n_Width, int n_Height)
{
	this->set_Width(n_Width);
	this->set_Height(n_Height);
}

//Sets the size of the openGL panel.
void OpenGLCanvas::set_OpenGLCanvasSize(int n_Width, int n_Height)
{
	this->m_OpenGLPanel->set_Width(n_Width);
	this->m_OpenGLPanel->set_Height(n_Height);

	if (this->m_ArcBallProxy)
		this->m_ArcBallProxy->setBounds((float)this->m_OpenGLPanel->get_Width(), (float)this->m_OpenGLPanel->get_Height());

	if (this->m_ArcBallWorld)
		this->m_ArcBallWorld->setBounds((float)this->m_OpenGLPanel->get_Width(), (float)this->m_OpenGLPanel->get_Height());

	if (this->m_ProxyObject)
		this->m_ProxyObject->set_NewCanvasSize(this->m_OpenGLPanel->get_Width());

	if (this->m_LightObject)
		this->m_LightObject->set_NewCanvasSize(this->m_OpenGLPanel->get_Width());

	if(this->m_ProxyObject && this->m_IsProxyObjectDrawn)
	{
		drawProxy();
		if(this->m_IsLightObjectDrawn)
			drawLightArrow();
		this->m_OpenGLPanel->Refresh();
	}
	else if(this->m_DrawModel && this->m_IsResultDrawn)
	{
		this->m_DrawModel->CanvasWidth  = n_Width;
		this->m_DrawModel->CanvasHeight = n_Height;
		this->m_OpenGLPanel->set_Code(this->m_DrawModel);
		this->m_OpenGLPanel->Refresh();
	}
	else
		clearCanvas();
}

//Sets the image plane.
void OpenGLCanvas::set_ImagePlaneModel(ImagePlane *n_ImagePlane)
{
	this->m_ImagePlane = n_ImagePlane;
	if (this->m_DrawModel)
	{
		this->m_DrawModel->TextureWidth = Convert::ToInt16(Math::Round(this->m_ImagePlane->get_Resolution()));
		this->m_DrawModel->TextureHeight = Convert::ToInt16(Math::Round(this->m_ImagePlane->get_Resolution()));
	}
}

//Sets the draw model.
void OpenGLCanvas::set_DrawModel(DrawModelGLSW *n_DrawModelGLSW)
{
	this->m_DrawModel = n_DrawModelGLSW;
}

//Sets the light model.
void OpenGLCanvas::set_LightModel(Light *n_LightModel)
{
	this->m_MainLight = n_LightModel;
	this->m_ProxyLight = new Light(n_LightModel);

	if (this->m_ProxyObject)
		this->m_ProxyObject->setLightModel(this->m_ProxyLight);
	if (this->m_LightObject)
		this->m_LightObject->setLightModel(this->m_ProxyLight);

	if (this->m_OpenGLPanel->Code->Equals( this->m_ProxyObject ))
		drawProxy();
	else if (this->m_OpenGLPanel->Code->Equals( this->m_LightObject ))
		drawLightArrow();
}

//Sets the proxy object.
void OpenGLCanvas::set_ProxyObject(ProxyObjectGL *n_ProxyObject)
{
	this->m_ProxyObject = n_ProxyObject;
}

//Sets the status.
void OpenGLCanvas::set_SystemStatus(unsigned char n_Status)
{
	this->m_Status = n_Status;
}

//Sets the background color of the openGL canvas.
void OpenGLCanvas::set_CanvasBackColor(Color n_CanvasBackColor)
{
	this->m_CanvasBackColor = n_CanvasBackColor;
	if (this->m_ProxyObject)
		this->m_ProxyObject->set_CanvasBackColor(this->m_CanvasBackColor);
	if (this->m_DrawModel)
		this->m_DrawModel->set_CanvasBackColor(this->m_CanvasBackColor);
	this->m_ClearCanvasGL->set_CanvasBackColor(this->m_CanvasBackColor);
}

//Sets the light direction panel.
void OpenGLCanvas::setLightDirectionPanel(StatusBarPanel *n_LightDirectionPanel)
{
	this->m_LightDirectionPanel = n_LightDirectionPanel;
}

//Sets the view direction panel.
void OpenGLCanvas::setViewDirectionPanel(StatusBarPanel *n_ViewDirectionPanel)
{
	this->m_ViewDirectionPanel = n_ViewDirectionPanel;
}

//Sets the light view mode.
void OpenGLCanvas::setLightViewMode(unsigned char n_LightViewMode)
{
	this->m_LightViewMode = n_LightViewMode;

	if (this->m_ProxyLight)
	{
		this->m_ProxyLight->LightViewMode = this->m_LightViewMode;
		this->m_ProxyLight->calculateLightDirection();
	}

	if (m_LightObject)
		this->m_LightObject->LightViewMode = this->m_LightViewMode;
		
	drawProxy();
	drawLightArrow();
}

//Returns the image plane.
ImagePlane* OpenGLCanvas::get_ImagePlaneModel(void)
{
	return this->m_ImagePlane;
}

//Returns the draw model.
DrawModelGLSW* OpenGLCanvas::get_DrawModel(void)
{
	return this->m_DrawModel;
}

//Returns the proxy object.
ProxyObjectGL* OpenGLCanvas::get_ProxyObject(void)
{
	return this->m_ProxyObject;
}

//Returns the openGL canvas.
OpenGLPanel::OpenGLPanel* OpenGLCanvas::getCanvas(void)
{
	return this->m_OpenGLPanel;
}

//Creates the proxy object.
void OpenGLCanvas::createProxyObjectGL(int n_CanvasLength, float n_RelLength __gc[], Matrix4 *n_TransMatrix)
{
	this->m_ProxyObject = NULL;
	this->m_ProxyObject = new ProxyObjectGL(n_CanvasLength, n_RelLength, n_TransMatrix);
	this->m_ProxyObject->set_CanvasBackColor(this->m_CanvasBackColor);
}

//Creates the proxy object.
void OpenGLCanvas::createProxyObjectGL(int n_CanvasLength, float n_RelLength __gc[])
{
	this->m_ProxyObject = NULL;
	this->m_ProxyObject = new ProxyObjectGL(n_CanvasLength, n_RelLength);
	this->m_ProxyObject->set_CanvasBackColor(this->m_CanvasBackColor);
}

//Creates the draw object.
void OpenGLCanvas::createDrawModelGLSW(int n_Width, int n_Height, short n_Resolution)
{
	this->m_DrawModel = NULL;
	this->m_DrawModel = new DrawModelGLSW(n_Width, n_Height, n_Resolution, n_Resolution);
	this->m_DrawModel->set_CanvasBackColor(this->m_CanvasBackColor);
}

//Creates the light object.
void OpenGLCanvas::createLightObjectGL(int n_CanvasLength, Light *n_LightModel)
{
	this->m_LightObject = NULL;
	this->m_LightObject = new LightObjectGL(n_CanvasLength, n_LightModel);
	this->m_LightObject->set_CanvasBackColor(this->m_CanvasBackColor);
	this->m_LightObject->LightViewMode = this->m_LightViewMode;
}

//Creates the light object.
void OpenGLCanvas::createLightObjectGL(int n_CanvasLength, Light *n_LightModel, Matrix4 *n_TransMatrix)
{
	this->m_LightObject = NULL;
	this->m_LightObject = new LightObjectGL(n_CanvasLength, n_LightModel, n_TransMatrix);
	this->m_LightObject->set_CanvasBackColor(this->m_CanvasBackColor);
	this->m_LightObject->LightViewMode = this->m_LightViewMode;
}

//Sets the transform matrix of the proxy object.
void OpenGLCanvas::setProxyObjectTransMatrix(Matrix4 *n_TransMatrix)
{
	this->m_ProxyObject->set_TransformationMatrix(n_TransMatrix);
}

//Modifies the transform matrix of the proxy object.
void OpenGLCanvas::modifyProxyObjectTransMatrix(Matrix4 *n_TransMatrix)
{
	this->m_ProxyObject->set_TransformationMatrix(Matrix4::MultiplyMatices(this->m_ProxyObject->TransformationMatrix, n_TransMatrix));
}

//Modifies the offset rotation matrix of the light object.
void OpenGLCanvas::modifyLightObjectGLOffsetRotation(Matrix4 *n_OffsetRotation)
{
	if (this->m_ProxyLight)
	{
		this->m_ProxyLight->modifyOffsetRotation(n_OffsetRotation);
		this->m_ProxyLight->calculateLightDirection();
	}

	if (this->m_LightObject)
		this->m_LightObject->modifyOffsetRotation(n_OffsetRotation);
}

//Modifies the base rotation matrix of the light object.
void OpenGLCanvas::modifyLightObjectGLBaseRotation(Matrix4 *n_BaseRotation)
{
	if (this->m_ProxyLight)
	{
		this->m_ProxyLight->modifyBaseRotation(n_BaseRotation);
		this->m_ProxyLight->calculateLightDirection();
	}

	if (this->m_LightObject)
		this->m_LightObject->modifyBaseRotation(Matrix4::InvertMatrix(n_BaseRotation));
}

//Draws the framebuffer to the openGL canvas.
void OpenGLCanvas::drawImage(unsigned char n_FrameBuffer __gc[])
{
	if(this->m_DrawModel)
	{
		this->m_DrawModel->CanvasWidth = this->m_OpenGLPanel->Width;
		this->m_DrawModel->CanvasHeight = this->m_OpenGLPanel->Width;
		this->m_DrawModel->TextureWidth = Convert::ToInt16(Math::Round(this->m_ImagePlane->get_Resolution()));
		this->m_DrawModel->TextureHeight = Convert::ToInt16(Math::Round(this->m_ImagePlane->get_Resolution()));
		this->m_DrawModel->setFrameBuffer(n_FrameBuffer);
		this->m_DrawModel->CanvasBackColor = this->m_CanvasBackColor;
	}
	else
	{
		this->m_DrawModel = NULL;
		this->m_DrawModel = new DrawModelGLSW(this->m_OpenGLPanel->get_Width(),
											this->m_OpenGLPanel->get_Height(),
											Convert::ToInt16(Math::Round(this->m_ImagePlane->get_Resolution())),
											Convert::ToInt16(Math::Round(this->m_ImagePlane->get_Resolution())),
											n_FrameBuffer,
											this->m_CanvasBackColor);
	}

	this->m_IsProxyObjectDrawn = this->m_IsLightObjectDrawn = false;
	this->m_IsResultDrawn = true;
	if (this->m_LightObject)
		this->m_LightObject->set_ProxyDrawnMode(false);

	this->m_OpenGLPanel->set_Code(this->m_DrawModel);
	this->m_OpenGLPanel->Refresh();
}

//Draws the proxy object.
void OpenGLCanvas::drawProxy(void)
{
	this->m_IsProxyObjectDrawn = true;
	this->m_IsResultDrawn = false;
	if (this->m_LightObject)
		this->m_LightObject->set_ProxyDrawnMode(true);

	if(this->m_ProxyObject)
	{
		this->m_OpenGLPanel->set_Code(this->m_ProxyObject);
		this->m_OpenGLPanel->Refresh();
	}
}

//Clears the openGL canvas.
void OpenGLCanvas::clearCanvas(void)
{
	this->m_IsProxyObjectDrawn = this->m_IsLightObjectDrawn = this->m_IsResultDrawn = false;

	if (this->m_LightObject)
		this->m_LightObject->set_ProxyDrawnMode(false);

	this->m_OpenGLPanel->set_Code(this->m_ClearCanvasGL);
	this->m_OpenGLPanel->Refresh();
}

//Draws the light arrow.
void OpenGLCanvas::drawLightArrow(void)
{
	if (this->m_ProxyObject && this->m_IsProxyObjectDrawn)
		drawProxy();

	this->m_IsLightObjectDrawn = true;
	this->m_IsResultDrawn = false;
	if(this->m_LightObject)
	{
		this->m_OpenGLPanel->set_Code(this->m_LightObject);
		this->m_OpenGLPanel->Refresh();
	}
}

//Resets the view parameters to default values.
void OpenGLCanvas::resetViewPointPosition(void)
{
	if (this->m_Status == READY)
	{
		this->m_ThisRotProxy = Matrix4::CreateIdentityMatrix();
		this->m_LastRotProxy = Matrix4::CreateIdentityMatrix();
		this->m_TransformMatrixProxy = Matrix4::CreateIdentityMatrix();

		this->m_ThisRotWorld = Matrix4::CreateIdentityMatrix();
		this->m_LastRotWorld = Matrix4::CreateIdentityMatrix();
		this->m_TransformMatrixWorld = Matrix4::CreateIdentityMatrix();

		this->m_CurrentTransformWorld = Matrix4::CreateIdentityMatrix();
		
		if (this->m_ImagePlane)
		{
			this->m_ImagePlane->resetMatrices();
			this->m_ImagePlane->resetPoints();
			this->m_ImagePlane->set_ViewDirection(new Vector3(DataLoader::Default_Viewdirection_X, 
																DataLoader::Default_Viewdirection_Y, 
																DataLoader::Default_Viewdirection_Z));
			this->m_ViewDirectionPanel->set_Text(String::Concat(S"View:   ", 
				Decimal::Round(Convert::ToDecimal(this->m_ImagePlane->ViewDirection->X), 1).ToString(), S"  |  ", 
				Decimal::Round(Convert::ToDecimal(this->m_ImagePlane->ViewDirection->Y), 1).ToString(), S"  |  ", 
				Decimal::Round(Convert::ToDecimal(this->m_ImagePlane->ViewDirection->Z), 1).ToString()));	
		}

		Vector3 *defaultDirection = new Vector3(DataLoader::Default_Lightdirection_X, DataLoader::Default_Lightdirection_Y, DataLoader::Default_Lightdirection_Z);

		if (this->m_MainLight)
		{	
			this->m_MainLight->resetBaseLightDirection();

			this->m_LightDirectionPanel->set_Text(String::Concat(S"Light:   ", 
				Decimal::Round(Convert::ToDecimal(this->m_MainLight->OffsetLightDirection->X), 1).ToString(), S"  |  ", 
				Decimal::Round(Convert::ToDecimal(this->m_MainLight->OffsetLightDirection->Y), 1).ToString(), S"  |  ", 
				Decimal::Round(Convert::ToDecimal(this->m_MainLight->OffsetLightDirection->Z), 1).ToString()));
		}
		if (this->m_ProxyLight)
		{
			this->m_ProxyLight->resetBaseLightDirection();
		}

		if (m_LightObject)
		{
			this->m_LightObject->set_BaseRotation(Matrix4::CreateIdentityMatrix());
			drawLightArrow();
		}

		if (m_ProxyObject)
		{
			this->m_ProxyObject->set_TransformationMatrix(Matrix4::CreateIdentityMatrix());
			this->drawProxy();
		}

		this->m_IsLeftMouseButton = false;
	}
}

//Resets the light direction to default values.
void OpenGLCanvas::resetLightDirection(void)
{
	if (this->m_Status == READY)
	{
		Vector3 *defaultDirection = new Vector3(DataLoader::Default_Lightdirection_X, DataLoader::Default_Lightdirection_Y, DataLoader::Default_Lightdirection_Z);

		if (this->m_MainLight)
		{	
			this->m_MainLight->resetOffsetLightDirection();

			this->m_LightDirectionPanel->set_Text(String::Concat(S"Light:   ", 
				Decimal::Round(Convert::ToDecimal(this->m_MainLight->OffsetLightDirection->X), 1).ToString(), S"  |  ", 
				Decimal::Round(Convert::ToDecimal(this->m_MainLight->OffsetLightDirection->Y), 1).ToString(), S"  |  ", 
				Decimal::Round(Convert::ToDecimal(this->m_MainLight->OffsetLightDirection->Z), 1).ToString()));
		}
		if (this->m_ProxyLight)
			this->m_ProxyLight->resetOffsetLightDirection();

		if (this->m_LightObject) 
		{
			this->m_LightObject->set_OffsetRotation(Matrix4::CreateIdentityMatrix());
			drawLightArrow();
		}

		this->m_IsRightMouseButton = false;
	}
}
