#include "StdAfx.h"
#include "OpenGLPreviewCanvas.h"

#using <mscorlib.dll>

using namespace IDVR;

void OpenGLPreviewCanvas::set_OpacityThreshold(float n_OpacityThreshold)
{
	this->m_OpacityThreshold = n_OpacityThreshold;
}

void OpenGLPreviewCanvas::set_GradientThreshold(float n_GradientThreshold)
{
	this->m_GradientThreshold = n_GradientThreshold;
}

void OpenGLPreviewCanvas::setControlSubItem(TD::SandBar::MenuButtonItem *n_SubItem_PreviewCanvas)
{
	this->m_SubItem_PreviewCanvas = n_SubItem_PreviewCanvas;
}

void OpenGLPreviewCanvas::setControlTbSubItem(TD::SandBar::MenuButtonItem *n_TbSubItem_PreviewCanvas)
{
	this->m_TbSubItem_PreviewCanvas = n_TbSubItem_PreviewCanvas;
}

void OpenGLPreviewCanvas::setReadyStatus(void)
{
	this->m_Status = READY;
	this->m_PreviewImagePlane = NULL;
	this->set_Cursor(System::Windows::Forms::Cursors::Default);
	this->Activate();
	if (this->m_ParentForm)
		this->m_ParentForm->set_Cursor(System::Windows::Forms::Cursors::Default);
}

void OpenGLPreviewCanvas::set_OriginalImagePlane(ImagePlane *n_OriginalImagePlane)
{
	this->m_OriginalImagePlane = n_OriginalImagePlane;
}

void OpenGLPreviewCanvas::set_LightModel(Light *n_LightModel)
{
	this->m_LightModel = n_LightModel;
}

void OpenGLPreviewCanvas::set_CanvasBackColor(Color n_CanvasBackColor)
{
	this->m_CanvasBackColor = n_CanvasBackColor;
	if (this->m_ClearCanvas)
		this->m_ClearCanvas->set_CanvasBackColor(this->m_CanvasBackColor);
	if (this->m_DrawModel)
		this->m_DrawModel->set_CanvasBackColor(this->m_CanvasBackColor);
}

void OpenGLPreviewCanvas::set_SystemStatus(unsigned char n_Status)
{
	this->m_Status = n_Status;
}

void OpenGLPreviewCanvas::set_TwoLevelRendering(bool n_TwoLevelRendering)
{
		this->m_TwoLevelRendering = n_TwoLevelRendering;
}

void OpenGLPreviewCanvas::set_IDVR(bool n_IDVR)
{
	this->m_IDVR = n_IDVR;
}

void OpenGLPreviewCanvas::clearCanvas(void)
{
	if (this->m_ClearCanvas)
	{
		this->panGL_Preview->set_Code(this->m_ClearCanvas);
		this->panGL_Preview->Refresh();
	}
}

void OpenGLPreviewCanvas::drawImage(unsigned char n_FrameBuffer __gc[])
{
	if (this->m_PreviewImagePlane)
	{
		this->m_DrawModel = new DrawModelGLSW(this->panGL_Preview->get_Width(),
												this->panGL_Preview->get_Height(),
												Convert::ToInt16(Math::Round(this->m_PreviewImagePlane->get_Resolution())),
												Convert::ToInt16(Math::Round(this->m_PreviewImagePlane->get_Resolution())),
												n_FrameBuffer,
												this->m_CanvasBackColor);

		this->panGL_Preview->set_Code(this->m_DrawModel);
		this->panGL_Preview->Refresh();
	}
}

void OpenGLPreviewCanvas::set_Dataset(VolumeData *n_Dataset)
{
	this->m_Dataset = n_Dataset;
}

void OpenGLPreviewCanvas::startRenderingProcess(void)
{
	this->m_Status = RENDERING;

	//Initialize preview imageplane with subsampling factor 2.0
	this->m_PreviewImagePlane = new ImagePlane(this->m_OriginalImagePlane);
	//Set the resolution of the preview imageplane to subsampling factor 2.0
	this->m_PreviewImagePlane->Resolution = Convert::ToInt16(this->panGL_Preview->get_Size().Width / 2.0);
	//Calculate the new imageplane position
	this->m_PreviewImagePlane->transformImagePlane();

	RenderMachineSW* rayCaster = new RenderMachineSW(this->m_Dataset, this->m_PreviewImagePlane, this->m_CanvasBackColor, this);
	rayCaster->set_Data(this->m_Dataset);	
	rayCaster->set_DensityInterpolationMode(InterpolationSW::BY_NEARESTNEIGHBOUR);
	rayCaster->set_GradientEstimationMode(InterpolationSW::BY_NEARESTNEIGHBOUR);
	rayCaster->set_VoxelIDInterpolationMode(InterpolationSW::BY_NEARESTNEIGHBOUR);
	rayCaster->set_GradientEstimationMode(GradientEstimationSW::BY_DIFFERENCES);
	rayCaster->set_SampingDistance( 1.0 );
	rayCaster->set_TwoLevelRendering(this->m_TwoLevelRendering);
	rayCaster->set_ImportanceDrivenRendering(this->m_IDVR);
	rayCaster->set_OpacityThreshold( this->m_OpacityThreshold );
	rayCaster->set_GradientThreshold( this->m_GradientThreshold );	

	Array* objectIDs = this->m_Dataset->getObjectIDs();

	for (int index = 0; index < objectIDs->get_Count(); index++)
	{
		IRenderer *renderer = this->m_Dataset->getObjectMember(index)->get_RenderModel();

		if (renderer->getShader()->GetType() == __typeof(PhongShadingModelSW))
		{
			PhongShadingModelSW *shader = __try_cast<PhongShadingModelSW*>(renderer->getShader());
			shader->setViewLightParameters(this->m_OriginalImagePlane->get_ViewDirection(), 
										   this->m_OriginalImagePlane->getCurrentLeftUpperPoint(), 
										   this->m_LightModel);
		}
		else if (renderer->getShader()->GetType() == __typeof(ContourShadingModelSW))
		{
			ContourShadingModelSW *shader = __try_cast<ContourShadingModelSW*>(renderer->getShader());
			shader->set_ViewDirection(this->m_OriginalImagePlane->get_ViewDirection());
		}
		else if (renderer->getShader()->GetType() == __typeof(ToneShadingModelSW))
		{
			ToneShadingModelSW *shader = __try_cast<ToneShadingModelSW*>(renderer->getShader());
			shader->set_LightDirection(this->m_LightModel->OffsetLightDirection);
		}
	}

	ProgressWin *progressWin;

	if (this->m_IDVR)
		progressWin  = new ProgressWin(S"Rendering Process Control", S"The footprint is being generated now");
	else
		progressWin = new ProgressWin(S"Rendering Process Control", S"The data is being rendered now");

	if (this->m_ParentForm)
	{
		System::Drawing::Point newLocation = System::Drawing::Point(
			this->m_ParentForm->get_Size().Width / 2 - (progressWin->get_Size().Width ),
			this->m_ParentForm->get_Size().Height / 4 - (progressWin->get_Size().Height / 2));

		progressWin->set_Location(System::Drawing::Point(newLocation));
		progressWin->set_MdiParent(this->m_ParentForm);
	}

	progressWin->Show();

	Thread *initializeHandler = new Thread(new ThreadStart(this, OpenGLPreviewCanvas::setReadyStatus));
	initializeHandler->set_IsBackground(true);

	rayCaster->setProgressWin(progressWin);
	rayCaster->setGUIHandler(initializeHandler);

	Thread *rayCasterHandler = new Thread(new ThreadStart(rayCaster, RenderMachineSW::start_Raycasting));

	rayCasterHandler->set_IsBackground(true);
	rayCasterHandler->Start();	

	this->set_Cursor(System::Windows::Forms::Cursors::WaitCursor);
	if (this->m_ParentForm)
		this->m_ParentForm->set_Cursor(System::Windows::Forms::Cursors::WaitCursor);
}