#pragma once

using namespace System;
using namespace System::ComponentModel;
using namespace System::Collections;
using namespace System::Windows::Forms;
using namespace System::Data;
using namespace System::Drawing;


namespace IDVR
{
	/**
	<B>GUI class for creating the <i>Progressbar</i> dialog window</B>.

	This class is part of the main application namespace IDVR and contains the definition of the <i>Progressbar</i> dialog window.
	<br><br>
	During several possible calculation processes like the loading of the volume data/mask data, the actual rendering or the video creation
	a progress bar should be shown to visualze the current status of the specific process. An instance of this class will be always created in 
	the specific class which starts the choosen calculation process (e.g. the class Neu for loading volume/mask data).
	<br><br>
	Thus, this class defines the progressbar dialog window and provides an interface which is needed to update the current process status.
	Note that the actual progressbar is performed by a particular thread to garantuee correct functionality.
	<br><br>
	The API documentation for this class shows only the most important member variables and function. Event handling functions and GUI
	components (e.g. labels, textboxes,...) will not be included but all essential parts of the class are surely documented.

	@see Neu - the main GUI window class of this application.
	@author <i>Bernhard Pflugfelder</i>
	@version 1.0
	@date 02-21-2005
	*/
	public __gc class ProgressWin : public System::Windows::Forms::Form
	{
	public: 

		/**
		This is the unique constructor of this class.
		It implements the standard initialization of the GUI window and actually start a 
		thread to create and control a specific dialog window.
		@param n_Title the new dialog title which will be shown on the top of the dialog window.
		@param n_Description the new description which will be shown inside the dialog window.
		*/
		ProgressWin(String *n_Title, String *n_Description)
		{
			InitializeComponent();

			this->Text = n_Title;
			this->lab_ProgessbarDescritpion->Text = n_Description;
			this->m_IsActive = true;
		}

		/**
		This is the standard constructor.
		It implements the standard initialization of the GUI window and actually start a 
		thread to create and control a specific dialog window.
		*/
		ProgressWin(void)
		{
			InitializeComponent();
			this->m_IsActive = true;
		}
        
        
	protected: 

		/**
		Destroys all GUI components before this current instance will be actually discarded.
		@param disposing is a flag variable which is true if the instance have to be discarded.
		*/
		void Dispose(Boolean disposing)
		{
			if (disposing && components)
			{
				components->Dispose();
			}
			__super::Dispose(disposing);
		}

	private: System::Windows::Forms::Label *  lab_ProgessbarDescritpion;
	private: System::Windows::Forms::ProgressBar *  prb_Progress;
	private: System::Windows::Forms::Label *  lab_ProgressDisplay;
	private: System::Windows::Forms::Label *  lab_ProgressDescription2;
	private: bool m_IsActive;
			 

	private:
		
		/// Erforderliche Designervariable.
		System::ComponentModel::Container* components;

		/**
		The main function to create the all needed GUI components.
		Furthermore, the linking of those created GUI components (e.g. labels, textboxes,...) to the current container
		is implemented. After the correct linking all GUI items will be shown on the screen.
		@note This function must not be changed in a code editor because otherwise the GUI builder could be working incorrectly.
		*/
		void InitializeComponent(void)
		{
			System::Resources::ResourceManager *  resources = new System::Resources::ResourceManager(__typeof(IDVR::ProgressWin));
			this->lab_ProgessbarDescritpion = new System::Windows::Forms::Label();
			this->prb_Progress = new System::Windows::Forms::ProgressBar();
			this->lab_ProgressDisplay = new System::Windows::Forms::Label();
			this->lab_ProgressDescription2 = new System::Windows::Forms::Label();
			this->SuspendLayout();
			// 
			// lab_ProgessbarDescritpion
			// 
			this->lab_ProgessbarDescritpion->AccessibleDescription = resources->GetString(S"lab_ProgessbarDescritpion.AccessibleDescription");
			this->lab_ProgessbarDescritpion->AccessibleName = resources->GetString(S"lab_ProgessbarDescritpion.AccessibleName");
			this->lab_ProgessbarDescritpion->Anchor = (*__try_cast<__box System::Windows::Forms::AnchorStyles *  >(resources->GetObject(S"lab_ProgessbarDescritpion.Anchor")));
			this->lab_ProgessbarDescritpion->AutoSize = (*__try_cast<__box System::Boolean *  >(resources->GetObject(S"lab_ProgessbarDescritpion.AutoSize")));
			this->lab_ProgessbarDescritpion->Dock = (*__try_cast<__box System::Windows::Forms::DockStyle *  >(resources->GetObject(S"lab_ProgessbarDescritpion.Dock")));
			this->lab_ProgessbarDescritpion->Enabled = (*__try_cast<__box System::Boolean *  >(resources->GetObject(S"lab_ProgessbarDescritpion.Enabled")));
			this->lab_ProgessbarDescritpion->Font = (__try_cast<System::Drawing::Font *  >(resources->GetObject(S"lab_ProgessbarDescritpion.Font")));
			this->lab_ProgessbarDescritpion->Image = (__try_cast<System::Drawing::Image *  >(resources->GetObject(S"lab_ProgessbarDescritpion.Image")));
			this->lab_ProgessbarDescritpion->ImageAlign = (*__try_cast<__box System::Drawing::ContentAlignment *  >(resources->GetObject(S"lab_ProgessbarDescritpion.ImageAlign")));
			this->lab_ProgessbarDescritpion->ImageIndex = (*__try_cast<__box System::Int32 *  >(resources->GetObject(S"lab_ProgessbarDescritpion.ImageIndex")));
			this->lab_ProgessbarDescritpion->ImeMode = (*__try_cast<__box System::Windows::Forms::ImeMode *  >(resources->GetObject(S"lab_ProgessbarDescritpion.ImeMode")));
			this->lab_ProgessbarDescritpion->Location = (*__try_cast<__box System::Drawing::Point *  >(resources->GetObject(S"lab_ProgessbarDescritpion.Location")));
			this->lab_ProgessbarDescritpion->Name = S"lab_ProgessbarDescritpion";
			this->lab_ProgessbarDescritpion->RightToLeft = (*__try_cast<__box System::Windows::Forms::RightToLeft *  >(resources->GetObject(S"lab_ProgessbarDescritpion.RightToLeft")));
			this->lab_ProgessbarDescritpion->Size = (*__try_cast<__box System::Drawing::Size *  >(resources->GetObject(S"lab_ProgessbarDescritpion.Size")));
			this->lab_ProgessbarDescritpion->TabIndex = (*__try_cast<__box System::Int32 *  >(resources->GetObject(S"lab_ProgessbarDescritpion.TabIndex")));
			this->lab_ProgessbarDescritpion->Text = resources->GetString(S"lab_ProgessbarDescritpion.Text");
			this->lab_ProgessbarDescritpion->TextAlign = (*__try_cast<__box System::Drawing::ContentAlignment *  >(resources->GetObject(S"lab_ProgessbarDescritpion.TextAlign")));
			this->lab_ProgessbarDescritpion->Visible = (*__try_cast<__box System::Boolean *  >(resources->GetObject(S"lab_ProgessbarDescritpion.Visible")));
			// 
			// prb_Progress
			// 
			this->prb_Progress->AccessibleDescription = resources->GetString(S"prb_Progress.AccessibleDescription");
			this->prb_Progress->AccessibleName = resources->GetString(S"prb_Progress.AccessibleName");
			this->prb_Progress->Anchor = (*__try_cast<__box System::Windows::Forms::AnchorStyles *  >(resources->GetObject(S"prb_Progress.Anchor")));
			this->prb_Progress->BackgroundImage = (__try_cast<System::Drawing::Image *  >(resources->GetObject(S"prb_Progress.BackgroundImage")));
			this->prb_Progress->Dock = (*__try_cast<__box System::Windows::Forms::DockStyle *  >(resources->GetObject(S"prb_Progress.Dock")));
			this->prb_Progress->Enabled = (*__try_cast<__box System::Boolean *  >(resources->GetObject(S"prb_Progress.Enabled")));
			this->prb_Progress->Font = (__try_cast<System::Drawing::Font *  >(resources->GetObject(S"prb_Progress.Font")));
			this->prb_Progress->ImeMode = (*__try_cast<__box System::Windows::Forms::ImeMode *  >(resources->GetObject(S"prb_Progress.ImeMode")));
			this->prb_Progress->Location = (*__try_cast<__box System::Drawing::Point *  >(resources->GetObject(S"prb_Progress.Location")));
			this->prb_Progress->Name = S"prb_Progress";
			this->prb_Progress->RightToLeft = (*__try_cast<__box System::Windows::Forms::RightToLeft *  >(resources->GetObject(S"prb_Progress.RightToLeft")));
			this->prb_Progress->Size = (*__try_cast<__box System::Drawing::Size *  >(resources->GetObject(S"prb_Progress.Size")));
			this->prb_Progress->Step = 1;
			this->prb_Progress->TabIndex = (*__try_cast<__box System::Int32 *  >(resources->GetObject(S"prb_Progress.TabIndex")));
			this->prb_Progress->Text = resources->GetString(S"prb_Progress.Text");
			this->prb_Progress->Visible = (*__try_cast<__box System::Boolean *  >(resources->GetObject(S"prb_Progress.Visible")));
			// 
			// lab_ProgressDisplay
			// 
			this->lab_ProgressDisplay->AccessibleDescription = resources->GetString(S"lab_ProgressDisplay.AccessibleDescription");
			this->lab_ProgressDisplay->AccessibleName = resources->GetString(S"lab_ProgressDisplay.AccessibleName");
			this->lab_ProgressDisplay->Anchor = (*__try_cast<__box System::Windows::Forms::AnchorStyles *  >(resources->GetObject(S"lab_ProgressDisplay.Anchor")));
			this->lab_ProgressDisplay->AutoSize = (*__try_cast<__box System::Boolean *  >(resources->GetObject(S"lab_ProgressDisplay.AutoSize")));
			this->lab_ProgressDisplay->Dock = (*__try_cast<__box System::Windows::Forms::DockStyle *  >(resources->GetObject(S"lab_ProgressDisplay.Dock")));
			this->lab_ProgressDisplay->Enabled = (*__try_cast<__box System::Boolean *  >(resources->GetObject(S"lab_ProgressDisplay.Enabled")));
			this->lab_ProgressDisplay->Font = (__try_cast<System::Drawing::Font *  >(resources->GetObject(S"lab_ProgressDisplay.Font")));
			this->lab_ProgressDisplay->Image = (__try_cast<System::Drawing::Image *  >(resources->GetObject(S"lab_ProgressDisplay.Image")));
			this->lab_ProgressDisplay->ImageAlign = (*__try_cast<__box System::Drawing::ContentAlignment *  >(resources->GetObject(S"lab_ProgressDisplay.ImageAlign")));
			this->lab_ProgressDisplay->ImageIndex = (*__try_cast<__box System::Int32 *  >(resources->GetObject(S"lab_ProgressDisplay.ImageIndex")));
			this->lab_ProgressDisplay->ImeMode = (*__try_cast<__box System::Windows::Forms::ImeMode *  >(resources->GetObject(S"lab_ProgressDisplay.ImeMode")));
			this->lab_ProgressDisplay->Location = (*__try_cast<__box System::Drawing::Point *  >(resources->GetObject(S"lab_ProgressDisplay.Location")));
			this->lab_ProgressDisplay->Name = S"lab_ProgressDisplay";
			this->lab_ProgressDisplay->RightToLeft = (*__try_cast<__box System::Windows::Forms::RightToLeft *  >(resources->GetObject(S"lab_ProgressDisplay.RightToLeft")));
			this->lab_ProgressDisplay->Size = (*__try_cast<__box System::Drawing::Size *  >(resources->GetObject(S"lab_ProgressDisplay.Size")));
			this->lab_ProgressDisplay->TabIndex = (*__try_cast<__box System::Int32 *  >(resources->GetObject(S"lab_ProgressDisplay.TabIndex")));
			this->lab_ProgressDisplay->Text = resources->GetString(S"lab_ProgressDisplay.Text");
			this->lab_ProgressDisplay->TextAlign = (*__try_cast<__box System::Drawing::ContentAlignment *  >(resources->GetObject(S"lab_ProgressDisplay.TextAlign")));
			this->lab_ProgressDisplay->Visible = (*__try_cast<__box System::Boolean *  >(resources->GetObject(S"lab_ProgressDisplay.Visible")));
			// 
			// lab_ProgressDescription2
			// 
			this->lab_ProgressDescription2->AccessibleDescription = resources->GetString(S"lab_ProgressDescription2.AccessibleDescription");
			this->lab_ProgressDescription2->AccessibleName = resources->GetString(S"lab_ProgressDescription2.AccessibleName");
			this->lab_ProgressDescription2->Anchor = (*__try_cast<__box System::Windows::Forms::AnchorStyles *  >(resources->GetObject(S"lab_ProgressDescription2.Anchor")));
			this->lab_ProgressDescription2->AutoSize = (*__try_cast<__box System::Boolean *  >(resources->GetObject(S"lab_ProgressDescription2.AutoSize")));
			this->lab_ProgressDescription2->Dock = (*__try_cast<__box System::Windows::Forms::DockStyle *  >(resources->GetObject(S"lab_ProgressDescription2.Dock")));
			this->lab_ProgressDescription2->Enabled = (*__try_cast<__box System::Boolean *  >(resources->GetObject(S"lab_ProgressDescription2.Enabled")));
			this->lab_ProgressDescription2->Font = (__try_cast<System::Drawing::Font *  >(resources->GetObject(S"lab_ProgressDescription2.Font")));
			this->lab_ProgressDescription2->Image = (__try_cast<System::Drawing::Image *  >(resources->GetObject(S"lab_ProgressDescription2.Image")));
			this->lab_ProgressDescription2->ImageAlign = (*__try_cast<__box System::Drawing::ContentAlignment *  >(resources->GetObject(S"lab_ProgressDescription2.ImageAlign")));
			this->lab_ProgressDescription2->ImageIndex = (*__try_cast<__box System::Int32 *  >(resources->GetObject(S"lab_ProgressDescription2.ImageIndex")));
			this->lab_ProgressDescription2->ImeMode = (*__try_cast<__box System::Windows::Forms::ImeMode *  >(resources->GetObject(S"lab_ProgressDescription2.ImeMode")));
			this->lab_ProgressDescription2->Location = (*__try_cast<__box System::Drawing::Point *  >(resources->GetObject(S"lab_ProgressDescription2.Location")));
			this->lab_ProgressDescription2->Name = S"lab_ProgressDescription2";
			this->lab_ProgressDescription2->RightToLeft = (*__try_cast<__box System::Windows::Forms::RightToLeft *  >(resources->GetObject(S"lab_ProgressDescription2.RightToLeft")));
			this->lab_ProgressDescription2->Size = (*__try_cast<__box System::Drawing::Size *  >(resources->GetObject(S"lab_ProgressDescription2.Size")));
			this->lab_ProgressDescription2->TabIndex = (*__try_cast<__box System::Int32 *  >(resources->GetObject(S"lab_ProgressDescription2.TabIndex")));
			this->lab_ProgressDescription2->Text = resources->GetString(S"lab_ProgressDescription2.Text");
			this->lab_ProgressDescription2->TextAlign = (*__try_cast<__box System::Drawing::ContentAlignment *  >(resources->GetObject(S"lab_ProgressDescription2.TextAlign")));
			this->lab_ProgressDescription2->Visible = (*__try_cast<__box System::Boolean *  >(resources->GetObject(S"lab_ProgressDescription2.Visible")));
			// 
			// ProgressWin
			// 
			this->AccessibleDescription = resources->GetString(S"$this.AccessibleDescription");
			this->AccessibleName = resources->GetString(S"$this.AccessibleName");
			this->AutoScaleBaseSize = (*__try_cast<__box System::Drawing::Size *  >(resources->GetObject(S"$this.AutoScaleBaseSize")));
			this->AutoScroll = (*__try_cast<__box System::Boolean *  >(resources->GetObject(S"$this.AutoScroll")));
			this->AutoScrollMargin = (*__try_cast<__box System::Drawing::Size *  >(resources->GetObject(S"$this.AutoScrollMargin")));
			this->AutoScrollMinSize = (*__try_cast<__box System::Drawing::Size *  >(resources->GetObject(S"$this.AutoScrollMinSize")));
			this->BackgroundImage = (__try_cast<System::Drawing::Image *  >(resources->GetObject(S"$this.BackgroundImage")));
			this->ClientSize = (*__try_cast<__box System::Drawing::Size *  >(resources->GetObject(S"$this.ClientSize")));
			this->Controls->Add(this->lab_ProgressDescription2);
			this->Controls->Add(this->lab_ProgressDisplay);
			this->Controls->Add(this->prb_Progress);
			this->Controls->Add(this->lab_ProgessbarDescritpion);
			this->Cursor = System::Windows::Forms::Cursors::WaitCursor;
			this->Enabled = (*__try_cast<__box System::Boolean *  >(resources->GetObject(S"$this.Enabled")));
			this->Font = (__try_cast<System::Drawing::Font *  >(resources->GetObject(S"$this.Font")));
			this->FormBorderStyle = System::Windows::Forms::FormBorderStyle::FixedToolWindow;
			this->Icon = (__try_cast<System::Drawing::Icon *  >(resources->GetObject(S"$this.Icon")));
			this->ImeMode = (*__try_cast<__box System::Windows::Forms::ImeMode *  >(resources->GetObject(S"$this.ImeMode")));
			this->Location = (*__try_cast<__box System::Drawing::Point *  >(resources->GetObject(S"$this.Location")));
			this->MaximumSize = (*__try_cast<__box System::Drawing::Size *  >(resources->GetObject(S"$this.MaximumSize")));
			this->MinimizeBox = false;
			this->MinimumSize = (*__try_cast<__box System::Drawing::Size *  >(resources->GetObject(S"$this.MinimumSize")));
			this->Name = S"ProgressWin";
			this->RightToLeft = (*__try_cast<__box System::Windows::Forms::RightToLeft *  >(resources->GetObject(S"$this.RightToLeft")));
			this->ShowInTaskbar = false;
			this->StartPosition = (*__try_cast<__box System::Windows::Forms::FormStartPosition *  >(resources->GetObject(S"$this.StartPosition")));
			this->Text = resources->GetString(S"$this.Text");
			this->Closed += new System::EventHandler(this, ProgressWin_Closed);
			this->ResumeLayout(false);

		}		


		private: System::Void ProgressWin_Closed(System::Object *  sender, System::EventArgs *  e)
		{
			this->m_IsActive = false;
		}

		public:

			/**
			Sets a new description for the current progressbar instance.
			If the progressbar is actually active the new description will be instantly shown.
			@param n_Description the new description which will be shown inside the dialog window.
			*/
			__property void set_Description(String *n_Description);
			/**
			Sets a new title for the current progressbar instance.
			If the progressbar is actually active the new title will be instantly shown.
			@param n_Title the new dialog title which will be shown on the top of the dialog window.
			*/
			__property void set_Title(String *n_Title);

			/**
			Increments the current progressbar about one single step.
			*/
			void setProgressbarStep();
			/**
			Sets the progressbar to the maximum step count.
			The progressbar will be shown fully filled.
			*/
			void setProgressbarToMaximum();
			/**
			Sets the range of the progressbar.
			This means the maximum count of steps of the current progressbar.
			@param n_StepRange the new range of steps for the progressbar.
			*/
			void setProgressbarStepRange(int n_StepRange);
			/**
			Returns the status of the progressbar dialog window.
			*/
			bool isWindowActive(void);
			/**
			Resets all GUI components of the progressbar dialog window to the corresponding parameters.
			The actual progressbar will be set to step count zero (start status).
			@param n_Title the new dialog title which will be shown on the top of the dialog window.
			@param n_Description the new description which will be shown inside the dialog window.
			*/
			void resetGUIItems(String *n_Title, String *n_Description);
};
}
