#include "StdAfx.h"
#include ".\proxyobjectgl.h"
#using <mscorlib.dll>

using namespace IDVR::Tools;
using namespace IDVR::Tools::OpenGLTools;
using namespace IDVR::VolumeRenderer::RenderPrimitives;

ProxyObjectGL::ProxyObjectGL(void)
{
	this->m_CanvasLength = 0;
	this->m_RelLength = NULL;
	this->m_AbsLength = NULL;
	this->m_TransMatrix = NULL;
	this->m_CanvasBackColor = new float __gc[3];
}
ProxyObjectGL::ProxyObjectGL(int n_CanvasLength, float n_RelLength __gc[])
{
	this->m_CanvasLength = n_CanvasLength;
	this->m_RelLength = new float __gc[3];
	this->m_AbsLength = new float __gc[3];
	this->m_CanvasBackColor = new float __gc[3];

	this->m_RelLength[X] = n_RelLength[X];
	this->m_RelLength[Y] = n_RelLength[Y];
	this->m_RelLength[Z] = n_RelLength[Z];
	this->m_TransMatrix = Matrix4::CreateIdentityMatrix();

	this->m_AbsLength[X] = this->m_RelLength[X] * (this->m_CanvasLength * LENGTHRATIO);
	this->m_AbsLength[Y] = this->m_RelLength[Y] * (this->m_CanvasLength * LENGTHRATIO);
	this->m_AbsLength[Z] = this->m_RelLength[Z] * (this->m_CanvasLength * LENGTHRATIO);
}

ProxyObjectGL::ProxyObjectGL(int n_CanvasLength, float n_RelLength __gc[], Matrix4 *n_TransMatrix)
{
	this->m_CanvasLength = n_CanvasLength;
	this->m_RelLength = new float __gc[3];
	this->m_AbsLength = new float __gc[3];
	this->m_CanvasBackColor = new float __gc[3];

	this->m_RelLength[X] = n_RelLength[X];
	this->m_RelLength[Y] = n_RelLength[Y];
	this->m_RelLength[Z] = n_RelLength[Z];
	this->m_TransMatrix = n_TransMatrix;

	this->m_AbsLength[X] = this->m_RelLength[X] * (this->m_CanvasLength * LENGTHRATIO);
	this->m_AbsLength[Y] = this->m_RelLength[Y] * (this->m_CanvasLength * LENGTHRATIO);
	this->m_AbsLength[Z] = this->m_RelLength[Z] * (this->m_CanvasLength * LENGTHRATIO);
}

ProxyObjectGL::~ProxyObjectGL(void)
{
	this->m_TranslateImagePlaneMatrix = NULL;
	this->m_RotImagePlaneMatrix = NULL;
	this->m_ScaleImagePlaneMatrix = NULL;
	this->m_RelLength = NULL;
	this->m_AbsLength = NULL;
}

void ProxyObjectGL::Init(void)
{
	glClearColor(this->m_CanvasBackColor[0], this->m_CanvasBackColor[1], this->m_CanvasBackColor[2], 0.0);
	glViewport(0,0,this->m_CanvasLength,this->m_CanvasLength);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	glOrtho(0.0, this->m_CanvasLength, 0.0, this->m_CanvasLength, -this->m_CanvasLength, this->m_CanvasLength);		

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	//Enabel shading features
	glEnable(GL_DEPTH_TEST);
    glEnable(GL_LINE_SMOOTH);
    glEnable(GL_POLYGON_SMOOTH);
	
	glFrontFace(GL_CW);
	glShadeModel(GL_SMOOTH);
	glPolygonMode(GL_FRONT, GL_FILL);

	//Enable current light features
	if (this->m_LightModel)
	{
		float light_ambient __gc[] = {this->m_LightModel->getNormalizedLightColorArray()[0] / 2.0f,
									  this->m_LightModel->getNormalizedLightColorArray()[1] / 2.0f,
									  this->m_LightModel->getNormalizedLightColorArray()[2] / 2.0f,
									  1.0f};
		float light_diffuse __gc[] = {this->m_LightModel->getNormalizedLightColorArray()[0],
									  this->m_LightModel->getNormalizedLightColorArray()[1],
									  this->m_LightModel->getNormalizedLightColorArray()[2],
									  1.0f};
		float light_specular __gc[] = {this->m_LightModel->getNormalizedLightColorArray()[0],
									  this->m_LightModel->getNormalizedLightColorArray()[1],
									  this->m_LightModel->getNormalizedLightColorArray()[2],
									  1.0f};
		Vector3 *currentLightDirection = this->m_LightModel->OffsetLightDirection->convertToOpenGLAlignedVector();
		float light_position __gc[] = Vector4::GetVector4OfVector3(currentLightDirection)->getFloatArrayOfVector();

		glLightfv(GL_LIGHT0, GL_POSITION, light_position);
		glLightfv(GL_LIGHT0, GL_AMBIENT, light_ambient);
		glLightfv(GL_LIGHT0, GL_DIFFUSE, light_diffuse);
		glLightfv(GL_LIGHT0, GL_SPECULAR, light_specular);
	}
	else
	{
		float ambient __gc[] = {0.6f,0.6f,0.6f,1.0f};
		glLightModelfv(GL_LIGHT_MODEL_AMBIENT, ambient);
	}

	this->m_CuboidObject = glGenLists(1);
	glNewList(this->m_CuboidObject, GL_COMPILE);
		defineCuboidObject();
	glEndList();

	//Enable color materials
	glColorMaterial(GL_FRONT_AND_BACK, GL_DIFFUSE);
	glEnable(GL_COLOR_MATERIAL);

	//Enable other features
	glEnable(GL_AUTO_NORMAL);
    glEnable(GL_NORMALIZE);
}

void ProxyObjectGL::Draw(void)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	//glColor3f(1.0, 0.0, 0.0);
	//Disable lighting features
	glEnable(GL_LIGHT0);
	glEnable(GL_LIGHTING);

	glPushMatrix();
		glLoadIdentity();
		
		//Do transformation with taking the object center into account
		glTranslatef(this->m_CanvasLength / 2.0f, this->m_CanvasLength / 2.0f, 0);
		glMultTransposeMatrixdARB(this->m_TransMatrix->getOneDimArray());
		glTranslatef(-this->m_AbsLength[X] / 2.0f, -this->m_AbsLength[Y] / 2.0f, -this->m_AbsLength[Z] / 2.0f);
		
		glCallList(this->m_CuboidObject);
	glPopMatrix();

	//Disable lighting features
	glDisable(GL_LIGHT0);
	glDisable(GL_LIGHTING);

	glFlush();
}

void ProxyObjectGL::defineCuboidObject(void)
{
	//Draw a cuboid object to simulate the dataset size and orientation
	//front face
	
	glBegin(GL_QUADS);
		glColor4f(0.0,0.0,1.0,1.0);
		glNormal3d(0.0,0.0,1.0);
		glVertex3d(0.0,0.0,this->m_AbsLength[Z]);
		glNormal3d(0.0,0.0,1.0);
		glVertex3d(0.0,this->m_AbsLength[Y],this->m_AbsLength[Z]);
		glNormal3d(0.0,0.0,1.0);
		glVertex3d(this->m_AbsLength[X],this->m_AbsLength[Y],this->m_AbsLength[Z]);
		glNormal3d(0.0,0.0,1.0);
		glVertex3d(this->m_AbsLength[X],0.0,this->m_AbsLength[Z]);
	
		//back face
		glColor4f(1.0,0.0,0.0,1.0);	
		glNormal3d(0.0,0.0,-1.0);
		glVertex3d(0.0,0.0,0.0);
		glNormal3d(0.0,0.0,-1.0);
		glVertex3d(this->m_AbsLength[X],0.0,0.0);
		glNormal3d(0.0,0.0,-1.0);
		glVertex3d(this->m_AbsLength[X],this->m_AbsLength[Y],0.0);
		glNormal3d(0.0,0.0,-1.0);
		glVertex3d(0.0,this->m_AbsLength[Y],0.0);
	
		//left face
		glColor4f(0.0,1.0,0.0,1.0);
		glNormal3d(-1.0,0.0,0.0);
		glVertex3d(0.0,0.0,this->m_AbsLength[Z]);
		glNormal3d(-1.0,0.0,0.0);
		glVertex3d(0.0,0.0,0.0);
		glNormal3d(-1.0,0.0,0.0);
		glVertex3d(0.0,this->m_AbsLength[Y],0.0);
		glNormal3d(-1.0,0.0,0.0);
		glVertex3d(0.0,this->m_AbsLength[Y],this->m_AbsLength[Z]);
	
		//right face
		glColor4f(0.0,1.0,0.0,1.0);
		glNormal3d(1.0,0.0,0.0);
		glVertex3d(this->m_AbsLength[X],0.0,0.0);
		glNormal3d(1.0,0.0,0.0);
		glVertex3d(this->m_AbsLength[X],0.0,this->m_AbsLength[Z]);
		glNormal3d(1.0,0.0,0.0);
		glVertex3d(this->m_AbsLength[X],this->m_AbsLength[Y],this->m_AbsLength[Z]);
		glNormal3d(1.0,0.0,0.0);
		glVertex3d(this->m_AbsLength[X],this->m_AbsLength[Y],0.0);

		//top face
		glColor4f(1.0,1.0,0.0,1.0);
		glNormal3d(0.0,1.0,0.0);
		glVertex3d(0.0,this->m_AbsLength[Y],0.0);
		glNormal3d(0.0,1.0,0.0);
		glVertex3d(this->m_AbsLength[X],this->m_AbsLength[Y],0.0);
		glNormal3d(0.0,1.0,0.0);
		glVertex3d(this->m_AbsLength[X],this->m_AbsLength[Y],this->m_AbsLength[Z]);
		glNormal3d(0.0,1.0,0.0);
		glVertex3d(0.0,this->m_AbsLength[Y],this->m_AbsLength[Z]);
	
		//bottom face
		glColor4f(1.0,1.0,0.0,1.0);
		glNormal3d(0.0,-1.0,0.0);
		glVertex3d(this->m_AbsLength[X],0.0,0.0);
		glNormal3d(0.0,-1.0,0.0);
		glVertex3d(0.0,0.0,0.0);
		glNormal3d(0.0,-1.0,0.0);
		glVertex3d(0.0,0.0,this->m_AbsLength[Z]);
		glNormal3d(0.0,-1.0,0.0);
		glVertex3d(this->m_AbsLength[X],0.0,this->m_AbsLength[Z]);
		
	glEnd();
}

void ProxyObjectGL::set_TransformationMatrix(Matrix4 *n_TransMatrix)
{
	this->m_TransMatrix = n_TransMatrix;
}

Matrix4* ProxyObjectGL::get_TransformationMatrix(void)
{
	return this->m_TransMatrix;
}

void ProxyObjectGL::set_NewCanvasSize(int n_CanvasLength)
{
	this->m_CanvasLength = n_CanvasLength;

	this->m_AbsLength[X] = this->m_RelLength[X] * (this->m_CanvasLength * LENGTHRATIO);
	this->m_AbsLength[Y] = this->m_RelLength[Y] * (this->m_CanvasLength * LENGTHRATIO);
	this->m_AbsLength[Z] = this->m_RelLength[Z] * (this->m_CanvasLength * LENGTHRATIO);
}

void ProxyObjectGL::setNewRelLengthValues(float n_RelLength __gc[])
{
	this->m_RelLength = n_RelLength;

	this->m_AbsLength[X] = this->m_RelLength[X] * (this->m_CanvasLength * LENGTHRATIO);
	this->m_AbsLength[Y] = this->m_RelLength[Y] * (this->m_CanvasLength * LENGTHRATIO);
	this->m_AbsLength[Z] = this->m_RelLength[Z] * (this->m_CanvasLength * LENGTHRATIO);
}

void ProxyObjectGL::set_CanvasBackColor(Color n_CanvasBackColor)
{
	this->m_CanvasBackColor[0] = (float)n_CanvasBackColor.get_R() / 255.0f;
	this->m_CanvasBackColor[1] = (float)n_CanvasBackColor.get_G() / 255.0f;
	this->m_CanvasBackColor[2] = (float)n_CanvasBackColor.get_B() / 255.0f;
}

void ProxyObjectGL::setLightModel(Light *n_LightModel)
{
	this->m_LightModel = n_LightModel;
}