#include "StdAfx.h"
#include ".\ray.h"
#include "RaySample.h"

#using <mscorlib.dll>

using namespace IDVR::VolumeRenderer::RenderPrimitives;
using namespace System::Collections;

//This is the standard constructor.
Ray::Ray(void)
{	
	this->m_RaySamples = new ArrayList();
	this->m_UsedObjectIDs = new ArrayList();
}

//This is the standard destructor.
Ray::~Ray(void)
{
	this->m_RaySamples = NULL;
	this->m_UsedObjectIDs = NULL;
}

//Sets the raycapacity.
void Ray::setRayCapacity(int n_RayCapacity)
{
	try
	{
		this->m_RaySamples->set_Capacity(n_RayCapacity);
		this->m_UsedObjectIDs->set_Capacity(n_RayCapacity);
	}
	catch(ArgumentOutOfRangeException *e)
	{
		throw new Exception(e->Message, e);
	}
}

//Inserts a raysample into the ray.
void Ray::insertSample(RaySample *newSample)
{
	int index1, index2;

	try
	{
		index1 = this->m_RaySamples->Add(newSample);
		index2 = this->m_UsedObjectIDs->Add(__box(newSample->ObjectID));
	}
	catch(NotSupportedException* e) 
	{
		throw new Exception(e->get_Message(), e);
	}
}

//Returns the raysample at the given position.
RaySample* Ray::getRaySample(int rayPosition)
{
	try
	{
		return __try_cast<RaySample*>(this->m_RaySamples->get_Item(rayPosition));
	}
	catch(Exception* e) 
	{
		throw new Exception(e->get_Message(), e); 
	};
}

//Returns all raysamples of the ray.
Array* Ray::getRaySamples(void)
{
	if (this->m_RaySamples->Count == 0)
		return NULL;
	else
	{
		Array *tempValues = this->m_RaySamples->ToArray(__typeof(RaySample));
		return tempValues;
	}
}

//Returns the already used objectIDs of the raysamples that are stored in the ray.
unsigned short Ray::getUsedObjectIDs(void) __gc[]
{
	unsigned short usedObjectIDs __gc[]; 

	if (this->m_RaySamples->Count == 0)
		return NULL;
	else
	{
		usedObjectIDs = new unsigned short __gc[this->m_UsedObjectIDs->Count];

		for (int i = 0; i < this->m_UsedObjectIDs->Count; i++)
		{
			usedObjectIDs[i] = Convert::ToUInt16(this->m_UsedObjectIDs->get_Item(i));
		}
		return usedObjectIDs;
		
	}
	
}

//Returns the number of stored raysamples.
int Ray::getSampleCount()
{
	return this->m_RaySamples->Count;
}

//Cleares all raysamples of the ray.
void Ray::clearRay(void)
{
	this->m_RaySamples->Clear();
	this->m_UsedObjectIDs->Clear();
}