#pragma once

#include ".\Vector3.h"
#include ".\Gradient.h"

namespace IDVR {
namespace VolumeRenderer {
namespace RenderPrimitives {

using namespace IDVR::Tools;
using namespace IDVR::VolumeRenderer;

/**
<b>Class for the raysample data structure.</b>

This class contains a couple of methods for the raysample primitive. Each raysample has various properties,
such as color, opacity, density, gradient vector, objectID and coordinates, which wil be described later.
@author <i>Christopher Thurnher</i>
@version 1.0
@date 02-17-2005
*/
public __gc class RaySample : public System::Object
{
private:
	
	float m_Color __gc[]; /**< The color of the raysample. */
	unsigned short m_Density; /**< The density of the raysample. */
	float m_Opacity; /**< The opacity of the raysample. */
	RenderPrimitives::Gradient* m_Gradient; /**< The gradient of the raysample. */
	unsigned short m_ObjectID; /**< The objectID of the raysample. */
	Vector3* m_Coords; /**< The coordinates of the raysample. */
	
public:

	/**
	This is the standard constructor.
	This standard constructor initialzed all elements with zero.
	*/
	RaySample(void);
	/**
	This constructor initalized a raysample with the given coordinates. All other parmeters are set to zero.
	@param n_Coords the coordinates of the raysample.
	*/
	RaySample(Vector3* n_Coords);
	/**
	This constructor initalized a raysample with the given parameters.
	@param n_Color the color of the raysample.
	@param n_Opacity the opacity of the raysample.
	@param n_ObjectID the objectID of the raysample.
	*/
	RaySample(float n_Color __gc[], float n_Opacity, unsigned short n_ObjectID);
	/**
	This is the standard destructor.
	*/
	~RaySample(void);

	/**
	Sets the color of the raysample.
	@param n_Color the color of the raysample.
	*/
	__property void set_Color(float n_Color __gc[]);
	/**
	Sets the density of the raysample.
	@param n_Density the density of the raysample.
	*/
	__property void set_Density(unsigned short n_Density);
	/**
	Sets the opacity of the raysample.
	@param n_Opacity the opacity of the raysample.
	*/
	__property void set_Opacity(float n_Opacity);
	/**
	Sets the gradient of the raysample.
	@param n_Gradient the gradient of the raysample.
	*/
	__property void set_Gradient(RenderPrimitives::Gradient* n_Gradient);
	/**
	Sets the objectID of the raysample.
	@param n_ObjectID the objectID of the raysample.
	*/
	__property void set_ObjectID(unsigned short n_ObjectID);
	/**
	Sets the coordinates of the raysample.
	@param n_Coords the coordinates of the raysample.
	*/
	__property void set_Coords(Vector3 *n_Coords);

	/**
	Returns the color of the raysample.
	@return The color of the raysample.
	*/
	__property float get_Color(void) __gc[];
	/**
	Returns the density of the raysample.
	@return The density of the raysample.
	*/
	__property unsigned short get_Density(void);
	/**
	Returns the opacity of the raysample.
	@return The opacity of the raysample.
	*/
	__property float get_Opacity(void);
	/**
	Returns the gradient of the raysample.
	@return The gradient of the raysample.
	*/
	__property RenderPrimitives::Gradient* get_Gradient(void);
	/**
	Returns the objectID of the raysample.
	@return The objectID of the raysample.
	*/
	__property unsigned short get_ObjectID(void);
	/**
	Returns the coordinates of the raysample.
	@return The coordinates of the raysample.
	*/
	__property Vector3* get_Coords(void);
};
}
}
}