#include "StdAfx.h"
#include ".\Transferfunction.h"
#include ".\DataLoader.h"
#using <mscorlib.dll>

using namespace System::Collections;
using namespace System::Globalization;
using namespace IDVR::Volume::DataStructures;
using namespace System::Drawing;

/*
This is the standard constructor.
All member variables of this new instance will be set to zero/null.
*/
Transferfunction::Transferfunction(void) 
{ 
	this->m_OpacityFunction = NULL; 
	this->m_ColorFunction = NULL; 
	this->m_ColorSamples = NULL; 
	this->m_OpacitySamples = NULL;
	this->m_FunctionLength = 0;
	this->m_InterpolationFactor = 0.5f;
	this->m_BackColor = DataLoader::Default_DvrColor;
}

/*
Constructor initialization with the maximum number of valid gray (<i>density</i>) values.
All member variables will be set to zero, the Transferfunction::m_FunctionLength member will be initialized with
the given maximum number of gray values.
*/
Transferfunction::Transferfunction(int numOfDensityValues)
{ 
	this->m_FunctionLength = numOfDensityValues;
	this->m_OpacityFunction = new float __gc[m_FunctionLength];
	this->m_ColorFunction =  new float __gc[3,this->m_FunctionLength];
	this->m_ColorSamples = new Hashtable();
	this->m_OpacitySamples = new Hashtable();
	this->m_InterpolationFactor = 0.5;
	this->m_BackColor = DataLoader::Default_DvrColor;
}

/*
Constructor initialization with an user-defined transfer function.
All member variables will be set to the corresponding parameters.
*/
Transferfunction::Transferfunction(float n_OpacityFunction __gc[], float n_ColorFunction __gc[,], unsigned short n_DensityScope, Hashtable *n_ColorSamples, Hashtable *n_OpacitySamples) 
{ 
	this->m_OpacityFunction = n_OpacityFunction; 
	this->m_ColorFunction = n_ColorFunction; 
	this->m_FunctionLength = n_DensityScope; 
	this->m_ColorSamples= n_ColorSamples;
	this->m_OpacitySamples = n_OpacitySamples;
	this->m_InterpolationFactor = 0.5;
	this->m_BackColor = DataLoader::Default_DvrColor;
}

/*
This is the standard deconstructor.
Espacially the Histogram::m_OpacityFunction, Histogram::m_ColorFunction, Histogram::m_OpacitySamples and
Histogram::m_ColorSamples members will be deleted.
*/
Transferfunction::~Transferfunction(void)
{
	this->m_OpacityFunction = NULL;
	this->m_ColorFunction = NULL;
	this->m_OpacitySamples = NULL;
	this->m_ColorSamples = NULL;
}

/**********************************************************************************************************************/
/**********************************************************************************************************************/

/*
Adds a new opacity function value to a specified density value.
The remaining opacity transfer function will not be changed.
*/
void Transferfunction::addNewOpacityValue(int n_Density,  float n_Opacity)
{ 
	this->m_OpacityFunction[n_Density] = n_Opacity; 
}

/*
Adds a new color function value to a specified density value.
The remaining color transfer function will not be changed.
*/
void Transferfunction::addNewColorValue(int n_Density,  float n_Color __gc[]) 
{ 
	this->m_ColorFunction[RED, n_Density] = n_Color[RED]; 
	this->m_ColorFunction[GREEN, n_Density] = n_Color[GREEN]; 
	this->m_ColorFunction[BLUE, n_Density] = n_Color[BLUE]; 
}

//Returns the opacity function value to the specified density value.
float Transferfunction::getOpacity(int n_Density) 
{ 
	return this->m_OpacityFunction[n_Density]; 
}

//Returns the coor function value to the specified density value.
float Transferfunction::getColor(int n_Density) __gc[] 
{ 
	float tempColor __gc[] = new float __gc[3];
	tempColor[RED] = this->m_ColorFunction[RED, n_Density];
	tempColor[GREEN] = this->m_ColorFunction[GREEN, n_Density];
	tempColor[BLUE] = this->m_ColorFunction[BLUE, n_Density];

	return tempColor;
}

/*
This function interpolates the opacity function for each density value.
Based upon the sample points of the Transferfunction::m_OpacitySamples member the opacity transfer function
can be linearly interpolated.
*/
void::Transferfunction::calculateOpacityFrequency(unsigned short n_DensityScope)
{
	 this->m_OpacityFunction = new float __gc[n_DensityScope];
	 float sortedIndicesOpacity __gc[] = new float __gc[this->m_OpacitySamples->get_Count()];

	 IDictionaryEnumerator* nextPos = this->m_OpacitySamples->GetEnumerator();
	 int index = 0;
			
	 while (nextPos->MoveNext())
		 sortedIndicesOpacity[index++] = Convert::ToSingle(nextPos->get_Key(), NumberFormatInfo::InvariantInfo);
	 Array::Sort(sortedIndicesOpacity);

	for (int i = 0; i < sortedIndicesOpacity->Length - 1; i++)
	{
		int x1 = Convert::ToInt32(Math::Round((float)sortedIndicesOpacity[i] * (n_DensityScope - 1)));
		int x2 = Convert::ToInt32(Math::Round((float)sortedIndicesOpacity[i+1] * (n_DensityScope - 1)));
		float key_x1 = sortedIndicesOpacity[i];
		float key_x2 = sortedIndicesOpacity[i+1];

		float y1 = Convert::ToSingle(this->m_OpacitySamples->get_Item(__box(key_x1)), NumberFormatInfo::InvariantInfo);
		float y2 = Convert::ToSingle(this->m_OpacitySamples->get_Item(__box(key_x2)), NumberFormatInfo::InvariantInfo);

		float k;
		if ((x2 - x1) != 0)
			k = (y2 - y1) / (x2 - x1);
		else 
			k = 0.0;

		for (int currentX = 0; currentX < x2 - x1 + 1; currentX++)
		{			
			this->m_OpacityFunction[currentX + x1] = (float)y1 + (float)(currentX * k);
		}
	}
}

/*
This function interpolates the color function for each density value.
Based upon the sample points of the Transferfunction::m_ColorSamples member the color transfer function
can be linearly interpolated.
*/
void::Transferfunction::calculateColorFrequency(unsigned short n_DensityScope)
{
	this->m_ColorFunction = new float __gc[3, n_DensityScope];
	float sortedIndicesColor __gc[];

	if (this->m_ColorSamples->get_Count() != 0)
		sortedIndicesColor = new float __gc[this->m_ColorSamples->get_Count()];
	else 
	{
		for (int i = 0; i < n_DensityScope; i++) 
		{
    		this->m_ColorFunction[0, i] = (float)this->m_BackColor.get_R() / 255.0f;
			this->m_ColorFunction[1, i] = (float)this->m_BackColor.get_G() / 255.0f;
			this->m_ColorFunction[2, i] = (float)this->m_BackColor.get_B() / 255.0f;
		}
		return;
	}

	IDictionaryEnumerator* nextPos = this->m_ColorSamples->GetEnumerator();
	int index = 0;
			
	while (nextPos->MoveNext())
		sortedIndicesColor[index++] = Convert::ToSingle(nextPos->get_Key(), NumberFormatInfo::InvariantInfo);		 
	Array::Sort(sortedIndicesColor);

	for (int i = 0; i <= sortedIndicesColor->Length; i++)
	{
		if (i == 0)
		{
			int x2 = Convert::ToInt32(Math::Round((float)sortedIndicesColor[i] * (n_DensityScope - 1)));

			interpolate_ColorScope(0, 
								   x2, 
								   this->m_BackColor, 
								   Color::FromArgb(Convert::ToInt32(this->m_ColorSamples->get_Item(__box(sortedIndicesColor[i])), NumberFormatInfo::InvariantInfo)));
		}
		else 
		{
			int x1 = Convert::ToInt32(Math::Round((float)sortedIndicesColor[i-1] * (n_DensityScope - 1)));
			if (i == sortedIndicesColor->Length)
			{
				interpolate_ColorScope(x1, 
									   n_DensityScope, 
									   Color::FromArgb(Convert::ToInt32(this->m_ColorSamples->get_Item(__box(sortedIndicesColor[i-1])), NumberFormatInfo::InvariantInfo)), 
									   this->m_BackColor);
				continue;
			}
			else if ( i > 0)
			{
				int x2 = Convert::ToInt32(Math::Round((float)sortedIndicesColor[i] * (n_DensityScope - 1)));
				interpolate_ColorScope(x1, 
									   x2, 
									   Color::FromArgb(Convert::ToInt32(this->m_ColorSamples->get_Item(__box(sortedIndicesColor[i-1])), NumberFormatInfo::InvariantInfo)), 
									   Color::FromArgb(Convert::ToInt32(this->m_ColorSamples->get_Item(__box(sortedIndicesColor[i])), NumberFormatInfo::InvariantInfo)));
			}
		}
	}
}

//This function interpolates the color transfer function linearly.	
void Transferfunction::interpolate_ColorScope(int x1, int x2, Color c1, Color c2)
{
	//mitte
	if(this->m_InterpolationFactor == 0.5f)
	{
		for (int step = x1; step < x2; step++)
		{
			float x = (float)(step - x1) / (x2 - x1);

			this->m_ColorFunction[RED,step]   = ((float)(1.0f - x)  * c1.get_R() + (float)c2.get_R() * x) / 255.0f;
			this->m_ColorFunction[GREEN,step] = ((float)(1.0f - x)  * c1.get_G() + (float)c2.get_G() * x) / 255.0f;
			this->m_ColorFunction[BLUE,step]  = ((float)(1.0f - x)  * c1.get_B()  + (float)c2.get_B() * x) / 255.0f;	
		}
	}

	//links
	else if(this->m_InterpolationFactor < 0.5f)
	{	
		float factor = this->m_InterpolationFactor * 2.0f;
		int length = Math::Abs(x2 - x1);
		for (int step = x1; step < x2; step++)			
		{	
			float x = (float)(step - x1) / ((x1 + length * factor) - x1);
			if(step < x1 + length * factor)
			{
				this->m_ColorFunction[RED,step]   = ((float)(1.0f - x)  * c1.get_R() + (float)c2.get_R() * x) / 255.0f;
				this->m_ColorFunction[GREEN,step] = ((float)(1.0f - x)  * c1.get_G() + (float)c2.get_G() * x) / 255.0f;
				this->m_ColorFunction[BLUE,step]  = ((float)(1.0f - x)  * c1.get_B()  + (float)c2.get_B() * x) / 255.0f;
			}
			else
			{	
				this->m_ColorFunction[RED,step]   = (float)c2.get_R() / 255.0f;
				this->m_ColorFunction[GREEN,step] = (float)c2.get_G() / 255.0f;
				this->m_ColorFunction[BLUE,step]  = (float)c2.get_B() / 255.0f;		
			}
		}
	}

	//rechts
	else if(this->m_InterpolationFactor > 0.5f)
	{		
		float factor = (this->m_InterpolationFactor - 0.5f) * 2.0f;
		int length = Math::Abs(x2 - x1 - 1);
		for (int step = x1; step < x2; step++)
		{	
			if (step == 4094)
				int ggg = 0;
			float x = (float)(step - (x1 + length * factor)) / (x2 - (x1 + length * factor));
			if(step < x1 + length * factor)
			{
				this->m_ColorFunction[RED,step]   = (float)c1.get_R() / 255.0f;
				this->m_ColorFunction[GREEN,step] = (float)c1.get_G() / 255.0f;
				this->m_ColorFunction[BLUE,step]  = (float)c1.get_B() / 255.0f;
			}
			else
			{	
				this->m_ColorFunction[RED,step]   = ((float)(1.0f - x)  * c1.get_R() + (float)c2.get_R() * x) / 255.0f;
				this->m_ColorFunction[GREEN,step] = ((float)(1.0f - x)  * c1.get_G() + (float)c2.get_G() * x) / 255.0f;
				this->m_ColorFunction[BLUE,step]  = ((float)(1.0f - x)  * c1.get_B()  + (float)c2.get_B() * x) / 255.0f;
			}
		}
	}	 
}

//Returns the current opacity transfer function.
float Transferfunction::get_OpacityFrequency(void) __gc[] 
{ 
	return this->m_OpacityFunction; 
}

//Returns the current color transfer function.
float Transferfunction::get_ColorFrequency(void) __gc[,]
{ 
	return this->m_ColorFunction; 
}
	
//Returns the current length of the transfer function.
int Transferfunction::get_FunctionLength(void) 
{ 
	return this->m_FunctionLength; 
}

//Returns the current color sample points.
Hashtable* Transferfunction::get_ColorSamples(void)
{
	return this->m_ColorSamples; 
}

//Returns the current opacity sample points.
Hashtable* Transferfunction::get_OpacitySamples(void)
{
	return this->m_OpacitySamples; 
}

//Returns the current back color of the transfer function.
Color Transferfunction::get_BackColor(void)
{
	return this->m_BackColor;
}

//Calculates the downsampled color transfer function.
float Transferfunction::getDownSampledColors(int sampleFactor, float colorValues __gc[,]) __gc[,]
{
	if (sampleFactor == 24)	
	{
		colorValues = getDownSampledColors(8, colorValues);
		sampleFactor = 3;
	}

	float downSampledColorValues __gc[,]= new float __gc[3, (colorValues->Length / 3) / sampleFactor];

	for (int i = 0, j = 0; i < colorValues->Length; i+=sampleFactor, j++)
	{
		try 
		{
			downSampledColorValues[RED,j] = colorValues[RED,i];
			downSampledColorValues[GREEN,j] = colorValues[GREEN,i];
			downSampledColorValues[BLUE,j] = colorValues[BLUE,i];
		}
		catch(IndexOutOfRangeException*)
		{
			break;
		}
	}
					
	return downSampledColorValues;
}

//Calculates the downsampled opacity transfer function.
float Transferfunction::getDownSampledOpacity(int sampleFactor, float opacityValues __gc[]) __gc[]
{
	if (sampleFactor == 24)	
	{
		opacityValues = getDownSampledOpacity(8, opacityValues);
		sampleFactor = 3;
	}

	float downSampledOpacityValues __gc[]= new float __gc[opacityValues->Length / sampleFactor];

	for (int i = 0, j = 0; i < opacityValues->Length; i+=sampleFactor, j++)
	{
		try 
		{
			downSampledOpacityValues[j] = opacityValues[i];
		}
		catch(IndexOutOfRangeException*)
		{
			break;
		}
	}

	return downSampledOpacityValues;
}

//Calculates the upsampled color transfer function.
float Transferfunction::getUpSampledColors(unsigned short n_DensityScope, int sampleFactor, float colorValues __gc[,]) __gc[,]
{
	float sampledColorValues __gc[,] = new float __gc[3, n_DensityScope];
	float delta_x, delta_y_Red, delta_y_Green, delta_y_Blue = 0;

	for (int i = 0; i < colorValues->Length / 3; i++)
	{
		try
		{
			delta_y_Red = colorValues[RED, i+1] - colorValues[RED, i];
			delta_y_Green= colorValues[GREEN, i+1] - colorValues[GREEN, i];
			delta_y_Blue = colorValues[BLUE, i+1] - colorValues[BLUE, i];
			delta_x = (float)(((i + 1) * sampleFactor) - (i * sampleFactor)); 
		}
		catch(IndexOutOfRangeException*)
		{
			delta_x = delta_y_Red = delta_y_Green = delta_y_Blue = 0;
		}

		if (delta_x == 0)
			for ( int step = 0; step < sampleFactor; step++)
			{
				sampledColorValues[RED, (i * sampleFactor)+ step] = colorValues[RED, i];
				sampledColorValues[GREEN, (i * sampleFactor)+ step] = colorValues[GREEN, i];
				sampledColorValues[BLUE, (i * sampleFactor)+ step] = colorValues[BLUE, i];
			}

		else 
		{  
			if (delta_y_Red == 0)
				for ( int step = 0; step < sampleFactor; step++)
					sampledColorValues[RED, (i * sampleFactor)+ step] = colorValues[RED, i];
			else
			{
				float factor = delta_y_Red / delta_x;

				for ( int step = 0; step < sampleFactor; step++)
					sampledColorValues[RED, (i * sampleFactor)+ step] = colorValues[RED, i] + (float)(factor * step);
			}

			if (delta_y_Green == 0)
				for ( int step = 0; step < sampleFactor; step++)
					sampledColorValues[GREEN, (i * sampleFactor)+ step] = colorValues[GREEN, i];
			else
			{
				float factor = delta_y_Green / delta_x;

				for ( int step = 0; step < sampleFactor; step++)
					sampledColorValues[GREEN, (i * sampleFactor)+ step] = colorValues[GREEN, i] + (float)(factor * step);
			}

			if (delta_y_Blue == 0)
				for ( int step = 0; step < sampleFactor; step++)
					sampledColorValues[BLUE, (i * sampleFactor)+ step] = colorValues[BLUE, i];
			else
			{
				float factor = delta_y_Blue / delta_x;

				for ( int step = 0; step < sampleFactor; step++)
					sampledColorValues[BLUE, (i * sampleFactor)+ step] = colorValues[BLUE, i] + (float)(factor * step);
			}
		}
	}

	return sampledColorValues;
}

//Calculates the upsampled opacity transfer function.
float Transferfunction::getUpSampledOpacity(unsigned short n_DensityScope, int sampleFactor, float opacityValues __gc[]) __gc[]
{
	float sampleOpacityValues __gc[] = new float __gc[n_DensityScope];
	float delta_x, delta_y = 0;

	for (int i = 0; i < opacityValues->Length; i++)
	{
		try
		{
			delta_y = opacityValues[i+1] - opacityValues[i];
			delta_x = (float)(((i + 1) * sampleFactor) - (i * sampleFactor)); 
		}
		catch(IndexOutOfRangeException*)
		{
			delta_x = delta_y = 0;
		}

		if (delta_x == 0 || delta_y == 0)
			for ( int step = 0; step < sampleFactor; step++)
				sampleOpacityValues[(i * sampleFactor)+ step] = opacityValues[i];
		else
		{
			float factor = delta_y / delta_x;

			for ( int step = 0; step < sampleFactor; step++)
				sampleOpacityValues[(i * sampleFactor)+ step] = opacityValues[i] + (float)(factor * step);
		}
		
	}

	return sampleOpacityValues;
}

/*
Creates a standard transfer function.
This means an linear opacity transfer function and a contant color transfer function with blue color.
*/
Transferfunction* Transferfunction::CreateGeneralTransfer(unsigned short n_DensityScope)
{
	float opacity_temp __gc[] = new float __gc[n_DensityScope];
	float color_temp __gc[,] = new float __gc[3, n_DensityScope];

	Hashtable *opacitySamples = new Hashtable();
	opacitySamples->Add(__box(0.0f), 
						__box(0.0f));
	opacitySamples->Add(__box(1.0f), 
						__box(1.0f));

	Transferfunction *newTF = new Transferfunction(opacity_temp, color_temp, n_DensityScope, new Hashtable(), opacitySamples);
	newTF->calculateOpacityFrequency(n_DensityScope);
	newTF->calculateColorFrequency(n_DensityScope);
	return newTF;
}

//Returns the current interpolation factor.
float Transferfunction::get_InterpolationFactor(void)
{
	return this->m_InterpolationFactor;
}

//Sets a new interpolation factor.
void Transferfunction::set_InterpolationFactor(float n_InterpolationFactor)
{
	this->m_InterpolationFactor = n_InterpolationFactor;
}

//Sets a new color transfer function.
void Transferfunction::set_ColorFrequency(float n_ColorFrequency __gc[,])
{
	this->m_ColorFunction = n_ColorFrequency;
}

//Sets a new opacity transfer function.
void Transferfunction::set_OpacityFrequency(float n_OpacityFrequency __gc[])
{
	this->m_OpacityFunction = n_OpacityFrequency;
}

//Sets a new function length.
void Transferfunction::set_FunctionLength(int n_functionLength)
{
	this->m_FunctionLength = n_functionLength;
}

//Sets new color sample points.
void Transferfunction::set_ColorSamples(Hashtable* n_ColorSamples)
{
	this->m_ColorSamples = n_ColorSamples;
}

//Sets new opacity sample points.
void Transferfunction::set_OpacitySamples(Hashtable* n_OpacitySamples)
{
	this->m_OpacitySamples = n_OpacitySamples;
}

//Sets a new back color for the current transfer function.
void Transferfunction::set_BackColor(Color n_BackColor)
{
	this->m_BackColor = n_BackColor;
}

/*
This static function converts the opacity sample point to drawable values.
This means that all opacity sample point (exactly the coordinates of each sample points) will be converted to the
original value scope between 0 and maximum height/width of the transfer function panel.
*/
Hashtable* Transferfunction::convertOpacitySamplesToDrawMode(Hashtable *n_OpacitySamples, int panelWidth, int panelHeight)
{
		Hashtable *convertedTable = new Hashtable();

		IDictionaryEnumerator *nextPos = n_OpacitySamples->GetEnumerator();
				
		while (nextPos->MoveNext())
		{
			int key = Convert::ToInt32(Math::Round(Convert::ToSingle(nextPos->get_Key()) * (double)panelWidth));
			int value = Convert::ToInt32(Math::Round(Convert::ToSingle(nextPos->get_Value()) * (double)panelHeight));
			value = panelHeight - value;

			convertedTable->set_Item(__box(key),__box(value));
		}

		return convertedTable;
}

/*
This static function converts the opacity sample point to normalzed values between 0 and 1.
This means that all opacity sample point (exactly the coordinates of each sample points) will be converted to the
value scope between 0 and 1.
*/
Hashtable* Transferfunction::convertOpacitySamplesToSaveMode(Hashtable *n_OpacitySamples, int panelWidth, int panelHeight)
{
		Hashtable *convertedTable = new Hashtable();

		IDictionaryEnumerator *nextPos = n_OpacitySamples->GetEnumerator();
				
		while (nextPos->MoveNext())
		{
			float key = Convert::ToSingle(nextPos->get_Key(), NumberFormatInfo::InvariantInfo) / (float)panelWidth;
			float value = Convert::ToSingle(nextPos->get_Value(), NumberFormatInfo::InvariantInfo) / (float)panelHeight;
			value = 1.0f - value;

			convertedTable->set_Item(__box(key), __box(value));
		}

		return convertedTable;
}

/*
This static function converts the color sample point to drawable values.
This means that all color sample point (exactly the coordinates of each sample points) will be converted to the
original value scope between 0 and maximum width of the transfer function panel.
*/
Hashtable* Transferfunction::convertColorSamplesToDrawMode(Hashtable *n_ColorSamples, int panelWidth)
{
		Hashtable *convertedTable = new Hashtable();

		IDictionaryEnumerator *nextPos = n_ColorSamples->GetEnumerator();
				
		while (nextPos->MoveNext())
		{
			int key = Convert::ToInt32(Math::Round(Convert::ToSingle(nextPos->get_Key()) * (float)panelWidth));
			convertedTable->set_Item(__box(key), nextPos->get_Value());
		}

		return convertedTable;
}

/*
This static function converts the color sample point to normalzed values between 0 and 1.
This means that all color sample point (exactly the coordinates of each sample points) will be converted to the
value scope between 0 and 1.
*/
Hashtable* Transferfunction::convertColorSamplesToSaveMode(Hashtable *n_ColorSamples, int panelWidth)
{
		Hashtable *convertedTable = new Hashtable();

		IDictionaryEnumerator *nextPos = n_ColorSamples->GetEnumerator();
				
		while (nextPos->MoveNext())
		{
			float key = Convert::ToSingle(nextPos->get_Key()) / (float)panelWidth;
			convertedTable->set_Item(__box(key), nextPos->get_Value());
		}

		return convertedTable;
}