#pragma once

using namespace System;

namespace IDVR {
namespace Tools {

	/**
	<B>Class for a 2 dimensional vector.</B>

	This class is part of the basic Tools namespace and contains a couple of methods for initialization and computing 
	various 2-element vectors (2 dimensional vectors).    

	For using vector with more elements (e.g. 3 or 4 elements) please see the following classes
	Vector3 and Vector4.

	@see Vector3 the three dimensional vector.
	@see Vector4 the four dimensional vector.
	@author <i>Bernhard Pflugfelder</i>
	@version 1.0
	@date 02-17-2005
	*/
public __gc class Vector2 : public System::Object
{
public:

	double m_X; /**< The x element of the vector.  The x element is the first element and is based on a double variable. */
	double m_Y; /**< The y element of the vector.  The y element is the second element and is based on a double variable. */

public:

	/**
	This is the standard constructor.
	This standard constructor initialzed both elements with zero.
	*/
	Vector2(void);
	/**
	Constructor initialization by two parameters.
	This constructor initialized both element of the vector with the specified parameters.
	@param n_X the value for the x element.
	@param n_Y the value for the y element.
	*/
	Vector2(double n_X, double n_Y);
	/**
	Constructor initialization by an array parameters.
	This constructor initialized both element of the vector with the specified array parameter.
	@param n_Value a two-element array for both elements.
	*/
	Vector2(double n_Value __gc[]);

	/**
	Returns the x element of the specific vector.
	@return The x element of the vector.
	*/
	__property double get_X(void);
	/**
	Returns the y element of the specific vector.
	@return The y element of the vector.
	*/
	__property double get_Y(void);
	/**
	Sets the x element of the existing vector.
	@param n_X the x element of the vector.
	*/
	__property void set_X(double n_X);
	/**
	Sets the y-element of the existing vector.
	@param n_Y the y element of the vector.
	*/
	__property void set_Y(double n_Y);

	/**
	Returns the magnitude of the specific vector.
	@return The magnitude of the vector.
	*/
	double magnitude(void);

	/**
	This static function created a normalized vector of the specified vector parameter.
	@param v the unnormalized vector.
	@return The normalized vector as a new instance.
	*/
	static Vector2* CreateNormalizedVector(Vector2* v);
	/**
	This static function returns the sum vector of the two specified vector parameters.
	@param v1 the first vector for addition.
	@param v2 the second vector for addition.
	@return The sum vector as a new instance.
	*/
	static Vector2* Addition(Vector2* v1, Vector2* v2);
	/**
	This static function returns the subtraction vector of the two specified vector parameters.
	@param v1 the first vector for subtraction.
	@param v2 the second vector for subtraction.
	@return The subtraction vector as a new instance.
	*/
	static Vector2* Subtraction(Vector2* v1, Vector2* v2);
};
}
}
