#include "StdAfx.h"
#include ".\VolumeData.h"
#using <mscorlib.dll>

using namespace System::Collections;
using namespace System::Drawing;
using namespace IDVR::Volume::DataStructures;
using namespace IDVR::VolumeRenderer;
using namespace IDVR::Interfaces;

/*
This is the standard constructor.
All member variables of this new instance will be set to zero/null.
*/
VolumeData::VolumeData(void)
{ 
	this->m_VoxelData = NULL;
	this->m_NextFreeObjectKey = 1;
    this->m_ObjectList = new Hashtable(); 
	this->m_GradientsPreCalculated = false;
}

/*
Constructor initialization with the valid volume data dimension.
All member variables will be set to zero.
*/
VolumeData::VolumeData(unsigned short n_VolumeDimension __gc[], unsigned short n_DensityScope) 
{		
	this->m_VoxelData = new VoxelData(n_VolumeDimension);
	this->m_DensityScope = n_DensityScope;
	this->m_ObjectList = new Hashtable(); 
	m_NextFreeObjectKey = 1;
	this->m_GradientsPreCalculated = false;
}

/*
This is the standard deconstructor.
All member variables will be deleted.
*/
VolumeData::~VolumeData(void)
{
	this->m_ObjectList = NULL;
	this->m_VoxelData = NULL;
	this->m_IIDVRCompositingModel = NULL;
}

/**********************************************************************************************************************/
/**********************************************************************************************************************/

//Returns the current number of objects.
unsigned short VolumeData::get_ObjectCount(void)
{ 
	return this->m_ObjectList->get_Count();
}

//Returns the current volume dimension.
unsigned short VolumeData::get_VolumeDimension(void) __gc[]
{ 
	return this->m_VoxelData->VolumeDimension; 
}

//Returns the overall count of voxel for the current dataset.
unsigned int VolumeData::get_VoxelCountOverall(void)
{
	return this->m_VoxelData->VoxelCountOverall;
}

//Returns the current voxel data set.
VoxelData* VolumeData::get_VoxelDataset(void)
{
	return this->m_VoxelData;
}

//Returns the flag value whether all gradient values are already calculated and saved or not.
bool VolumeData::get_GradientsPreCalculated(void)
{
	return this->m_GradientsPreCalculated;
}

//Sets the flag value whether all gradient values are already calculated and saved or not.
void VolumeData::set_GradientsPreCalculated(bool n_GradientsPreCalculated)
{
	this->m_GradientsPreCalculated = n_GradientsPreCalculated;
}

//Returns whether volume data objects are stored in the current data structure.
bool VolumeData::haveObjects(void) 
{ 
	if (this->get_ObjectCount() == NO_OBJECT) 
		return false;
	else return true;
}

/*
Creates the general volume data object (always with objectID = 0).
This general data object represents the entire voume data. Thus, this data object must be always generated
for correct representation of the data set.
*/
void VolumeData::addGeneralObject(String *n_ObjectDescription, unsigned short n_Importance, IRenderer* n_Renderer) 
{	
	DataObject *tempObject = new DataObject(n_ObjectDescription, n_Importance, this->m_DensityScope, n_Renderer);
	try
	{
		this->m_ObjectList->Add(__box(VolumeData::GENERALOBJECT), tempObject); 
	}
	catch (Exception* e) { throw new Exception(e->get_Message(), e); }
}

//Adds a new  volume data object object to the current data structure.
unsigned short VolumeData::addObject(String *n_ObjectDescription, unsigned short n_Importance, IRenderer* n_Renderer) 
{	
	DataObject *tempObject = new DataObject(n_ObjectDescription, n_Importance, this->m_DensityScope, n_Renderer);
	try
	{
		this->m_ObjectList->Add(__box(this->m_NextFreeObjectKey), tempObject); 
	}
	catch (Exception* e) 
	{ 
		throw new Exception(e->get_Message(), e); 
	}

	return this->m_NextFreeObjectKey++;
}

//Removes the specified volume data object of the current volume data set.
void VolumeData::removeObject(unsigned short n_ObjectID)
{
	try
	{
		for (int depth = 0; depth < this->m_VoxelData->VolumeDimension[Z]; depth++)
			for (int row = 0; row < this->m_VoxelData->VolumeDimension[Y]; row++)
				for (int column = 0; column < this->m_VoxelData->VolumeDimension[X]; column++)
				{
					if (this->m_VoxelData->getVoxelObjectID(column, row, depth) == n_ObjectID)
						this->m_VoxelData->setVoxelObjectID(column, row, depth, GENERALOBJECT);
				}
		this->m_ObjectList->Remove(__box(n_ObjectID));
		for (int index = n_ObjectID + 1; index < this->m_ObjectList->Count; index++)
		{
			try
			{
				this->m_ObjectList->Add(__box(index-1), this->m_ObjectList->get_Item(__box(index)));
			}
			catch (ArgumentException *a)
			{
				throw new Exception(a->Message, a);
			}
			catch (NotSupportedException *n)
			{
				throw new Exception(n->Message, n);
			}
		}
		this->m_NextFreeObjectKey--;

	}
	catch(Exception *e)
	{
		throw new Exception(e->Message, e);
		return;
	}
}

//Sets the specified voxel element with the choosen density value and object ID.
void VolumeData::addVoxel(unsigned short n_VoxelCoordinates __gc[], unsigned short n_Density, unsigned short n_ObjectID)
{
	try 
	{
		this->m_VoxelData->setVoxelValues(n_VoxelCoordinates, n_Density, n_ObjectID);
		getObjectMember(n_ObjectID)->get_DensityHistogram()->addNewValue(n_Density);
	}
	catch (Exception *e) { throw new Exception(e->get_Message(), e); }
}

/*
Sets the specified voxel element with the choosen density value.
The associated volume data object is the current general data object (ID = 0).
*/
void VolumeData::addVoxel(unsigned short n_VoxelCoordinates __gc[], unsigned short n_Density)
{
	try 
	{
		this->m_VoxelData->setVoxelValues(n_VoxelCoordinates, n_Density, GENERALOBJECT);
		getObjectMember(GENERALOBJECT)->get_DensityHistogram()->addNewValue(n_Density);
	}
	catch (Exception *e) { throw new Exception(e->get_Message(), e); }
}

//returns the data object width the specified objectID
DataObject* VolumeData::getObjectMember(unsigned short n_ObjectID)
{
	try
	{
		return __try_cast<IDVR::Volume::DataStructures::DataObject*>(this->m_ObjectList->get_Item(__box(n_ObjectID)));
	}
	catch (Exception* e) { throw new Exception(e->get_Message(), e); };
}

//Returns all used object IDs of the current data set.
Array* VolumeData::getObjectIDs(void)
{
	Array* keys = Array::CreateInstance(__typeof(Object),(this->m_ObjectList->get_Keys())->get_Count());  
	(this->m_ObjectList->get_Keys())->CopyTo(keys, 0);
	Array::Sort(keys);
	return keys;
}

//Returns the data object to which the specified voxel belongs.
DataObject* VolumeData::getObjectMember(unsigned short n_VoxelCoordinates __gc[])
{
	unsigned short objectID = this->m_VoxelData->getVoxelObjectID(n_VoxelCoordinates);
	try 
	{
		return __try_cast<DataObject*>(this->m_ObjectList->get_Item(__box(objectID)));
	}
	catch (Exception* e) { throw new Exception(e->get_Message(), e); }
}

//Returns the general data object (objectID = 0).
DataObject* VolumeData::getGeneralDataObject(void)
{ 
	return this->getObjectMember(VolumeData::GENERALOBJECT);
}	

//Returns the overall numer of voxels of the current volume data set.
int VolumeData::getMaxVoxelCount(void)
{ 
	return this->m_VoxelData->VolumeDimension[X] * this->m_VoxelData->VolumeDimension[Y] * this->m_VoxelData->VolumeDimension[Z]; 
}

//Returns the slice of the volume data set with the constant specified x-value.
unsigned short VolumeData::getVoxelSliceX(unsigned short n_SliceX) __gc[,]
{
	unsigned short sliceValues __gc[,] = new unsigned short __gc[this->m_VoxelData->VolumeDimension[Z], this->m_VoxelData->VolumeDimension[Y]];
	__int64 index = 0;

	try
	{
		for(int zDim = 0; zDim < this->m_VoxelData->VolumeDimension[Z]; zDim++)
			for (int yDim = 0; yDim < this->m_VoxelData->VolumeDimension[Y]; yDim++) 
				sliceValues[zDim,yDim] = this->m_VoxelData->getVoxelData(n_SliceX, yDim, zDim);
	}
	catch (Exception *e) 
	{ 
		throw new Exception(e->get_Message(), e); 
	}
	return sliceValues;
}

//Returns the slice of the volume data set with the constant specified y-value.
unsigned short VolumeData::getVoxelSliceY(unsigned short n_SliceY) __gc[,]
{
	unsigned short sliceValues __gc[,] = new unsigned short __gc[this->m_VoxelData->VolumeDimension[X], this->m_VoxelData->VolumeDimension[Z]];
	__int64 index = 0;

	try
	{
		for(int zDim = 0; zDim < this->m_VoxelData->VolumeDimension[Z]; zDim++)
			for (int xDim = 0; xDim < this->m_VoxelData->VolumeDimension[X]; xDim++) 
				sliceValues[xDim,zDim] = this->m_VoxelData->getVoxelData(xDim, n_SliceY, zDim);
	}
	catch (Exception *e) 
	{ 
		throw new Exception(e->get_Message(), e); 
	}
	return sliceValues;
}

//Returns the slice of the volume data set with the constant specified z-value.
unsigned short VolumeData::getVoxelSliceZ(unsigned short n_SliceZ) __gc[,]
{
	unsigned short sliceValues __gc[,] = new unsigned short __gc[this->m_VoxelData->VolumeDimension[X], this->m_VoxelData->VolumeDimension[Y]];
	__int64 index = 0;

	try
	{
		for(int yDim = 0; yDim < this->m_VoxelData->VolumeDimension[Y]; yDim++)
			for (int xDim = 0; xDim < this->m_VoxelData->VolumeDimension[X]; xDim++) 
				sliceValues[xDim,yDim] = this->m_VoxelData->getVoxelData(xDim, yDim, n_SliceZ);
	}
	catch (Exception *e) 
	{ 
		throw new Exception(e->get_Message(), e); 
	}
	return sliceValues;
}

//Sets a new IIDVRCompositingModel.
void  VolumeData::setIIDVRCompositingModel(IIDVRCompositingModel *n_IIDVRCompositingModel)
{
	this->m_IIDVRCompositingModel = n_IIDVRCompositingModel;
}

//Returns the IIDVRCompositingModel of the current volume data set.
IIDVRCompositingModel* VolumeData::getIIDVRCompositingModel(void)
{
	return this->m_IIDVRCompositingModel;
}

//Creates the general IIDVRCompositingModel (MImP with ColAndOp).
void VolumeData::createGeneralIDVRObjects(void)
{
	this->m_IIDVRCompositingModel = new IDVRMImPCompositingModelSW(new IDVRColAndOpModulatorSW());
}

//Returns the importance value of the choosen object ID.
unsigned short VolumeData::getVoxelImportance(unsigned short n_VoxelData)
{
	unsigned short voxelID = this->m_VoxelData->getVoxelObjectID(n_VoxelData);
	return getObjectMember(voxelID)->get_Importance();
}

//Returns the importance value of the choosen voxel coordinates.
unsigned short VolumeData::getVoxelImportance(unsigned short n_VoxelCoordinates __gc[])
{
	unsigned short voxelID = this->m_VoxelData->getVoxelObjectID(n_VoxelCoordinates);
	return getObjectMember(voxelID)->get_Importance();
}

//Returns the importance value of the choosen voxel coordinates.
unsigned short VolumeData::getVoxelImportance(unsigned short n_Column, unsigned short n_Row, unsigned short n_Depth)
{
	unsigned short voxelID = this->m_VoxelData->getVoxelObjectID(n_Column, n_Row, n_Depth);
	return getObjectMember(voxelID)->get_Importance();
}

/*
Resets the current volume data set to default status. 
That means all objectIDs will be set to GENERALOBJECT and all stored volume data objects will be removed.
*/
void VolumeData::resetDataset(void)
{
	unsigned short volumeDimension __gc[] = this->m_VoxelData->get_VolumeDimension();

	for (int depth = 0; depth < volumeDimension[Z]; depth++)
		for (int row = 0; row < volumeDimension[Y]; row++)
			for (int column = 0; column < volumeDimension[X]; column++)
			{
				this->m_VoxelData->setVoxelObjectID(column,row,depth,this->m_VoxelData->GENERALOBJECT);
			}
	DataObject *tempObject = getGeneralDataObject();

	this->m_ObjectList->Clear();
	this->m_NextFreeObjectKey = 1;
	try
	{
		this->m_ObjectList->Add(__box(VolumeData::GENERALOBJECT), tempObject); 
	}
	catch (Exception* e) 
	{ 
		throw new Exception(e->get_Message(), e); 
	}
	
}