#include "StdAfx.h"
#include ".\xmlparser.h"
#using <mscorlib.dll>

using namespace System::Globalization;
using namespace IDVR::Tools;

XMLParser::XMLParser(String* n_FilePath)
{
	this->m_FilePath = n_FilePath;
	int beginIndex = n_FilePath->LastIndexOf('\\');
	if (beginIndex != -1)
		this->m_FileName = n_FilePath->Substring(beginIndex + 1);
	else
		this->m_FileName = this->m_FilePath;
}

void XMLParser::load_ConfigFile(void)
{

	XmlDocument *configFile = new XmlDocument();

	try
	{
		configFile->Load( this->m_FilePath );

		XmlElement *firstNode = configFile->get_DocumentElement();

		if (0 != String::CompareOrdinal(firstNode->get_Name(), S"configuration"))
		{	
			throw new Exception(String::Concat(S"The file ", this->m_FileName, S" is not a valid IDVR configuration file!"));
		}

	
		XmlNodeReader *configReader = new XmlNodeReader( configFile );

		while ( configReader->Read() )
		{
			if (configReader->NodeType == XmlNodeType::Element)
	
			if (0 == String::CompareOrdinal(configReader->get_Name(), S"appSettings"))
				read_DatafileSettings( configReader );
			else if (0 == String::CompareOrdinal(configReader->get_Name(), S"phongShadingSettings"))
				read_PhongShadingSettings( configReader );
			else if (0 == String::CompareOrdinal(configReader->get_Name(), S"optimationSettings"))
				read_OptimationSettings( configReader );
			else if (0 == String::CompareOrdinal(configReader->get_Name(), S"depthCueingSettings"))
				read_DepthCueingSettings( configReader );
			else if (0 == String::CompareOrdinal(configReader->get_Name(), S"mipSettings"))
				read_MIPSettings( configReader );
			else if (0 == String::CompareOrdinal(configReader->get_Name(), S"contourSettings"))
				read_ContourSettings( configReader );
			else if (0 == String::CompareOrdinal(configReader->get_Name(), S"toneSettings"))
				read_ToneSettings( configReader );
		}	
	}
	catch(XmlException* xml)
	{
		throw new Exception(xml->get_Message(), xml);
	}
	catch(FormatException* format)
	{
		throw new Exception(format->get_Message(), format);
	}
	catch(Exception* e)
	{
		throw new Exception(e->get_Message(), e);
	}

}

Transferfunction* XMLParser::load_Transferfunction(void)
{

	XmlDocument *loadTransFunc = new XmlDocument();
	int functionLength = 0, backColor = 0;
	Transferfunction *transFunc = NULL;
	float interpolationfactor = 0.0f;
	Hashtable *colorSamples = new Hashtable();
	Hashtable *opactiySamples = new Hashtable();

	try
	{
		loadTransFunc->Load( this->m_FilePath );

		XmlElement *firstNode = loadTransFunc->get_DocumentElement();

		if (0 != String::CompareOrdinal(firstNode->get_Name(), S"Transferfunction"))
		{	
			throw new Exception(String::Concat(S"The file ", this->m_FileName, S" is not a valid IDVR transfer function file!"));
		}

		XmlNodeReader *transFuncReader = new XmlNodeReader( loadTransFunc );

		while ( transFuncReader->Read() )
		{
			switch(transFuncReader->NodeType)
			{
				case XmlNodeType::Element:

					if (0 == String::CompareOrdinal(transFuncReader->get_Name(), S"Transferfunction"))
					{
						functionLength = Convert::ToInt32(transFuncReader->GetAttribute(S"functionLength"));
					}

					else if (0 == String::CompareOrdinal(transFuncReader->get_Name(), S"opacitySamples"))
					{
						int sampleLength = Convert::ToInt32(transFuncReader->GetAttribute(S"length"));
						if (sampleLength != 0)
						{
							for (int i = 0; i < sampleLength; i++)
							{
								transFuncReader->Read();

								opactiySamples->Add(__box(Convert::ToSingle(transFuncReader->GetAttribute(NAME), NumberFormatInfo::InvariantInfo)), __box(Convert::ToSingle(transFuncReader->GetAttribute(VALUE), NumberFormatInfo::InvariantInfo)));
							}
						}
					}

					else if (0 == String::CompareOrdinal(transFuncReader->get_Name(), S"colorSamples"))
					{
						int sampleLength = Convert::ToInt32(transFuncReader->GetAttribute(S"length"));
						if (sampleLength != 0)
						{
							for (int i = 0; i < sampleLength; i++)
							{
								transFuncReader->Read();
								colorSamples->Add(__box(Convert::ToSingle(transFuncReader->GetAttribute(NAME), NumberFormatInfo::InvariantInfo)), __box(Convert::ToInt32(transFuncReader->GetAttribute(VALUE), NumberFormatInfo::InvariantInfo)));
							}
						}
					}

					else if (0 == String::CompareOrdinal(transFuncReader->get_Name(), S"interpolationFactor"))
					{
						interpolationfactor = Convert::ToSingle(transFuncReader->GetAttribute(NAME), NumberFormatInfo::InvariantInfo);
					}

					else if (0 == String::CompareOrdinal(transFuncReader->get_Name(), S"backColor"))
					{
						backColor = Convert::ToInt32(transFuncReader->GetAttribute(NAME), NumberFormatInfo::InvariantInfo);
					}
					break;
			}	
		}

		transFunc = new Transferfunction(DataLoader::DensityScope);
		transFunc->set_InterpolationFactor(interpolationfactor);
		transFunc->set_BackColor(Color::FromArgb(backColor));
		transFunc->set_OpacitySamples(opactiySamples);
		transFunc->set_ColorSamples(colorSamples);
	}
	catch(XmlException* xml)
	{
		throw new Exception(xml->get_Message(), xml);
		return NULL;
	}
	catch(FormatException* format)
	{
		throw new Exception(format->get_Message(), format);
		return NULL;
	}

	return transFunc;

}

void XMLParser::read_PhongShadingSettings(XmlNodeReader *n_XMLReader)
{
	n_XMLReader->Read();

	try
	{
		while (n_XMLReader->NodeType != XmlNodeType::EndElement)
		{
			if (n_XMLReader->NodeType != XmlNodeType::Comment) 
			{	
				if ( 0 == String::CompareOrdinal(n_XMLReader->GetAttribute(NAME), S"ambient") )
				{	
					float temp_value = Convert::ToSingle(n_XMLReader->GetAttribute(VALUE), NumberFormatInfo::InvariantInfo);
					if (temp_value >= 0.0 && temp_value <= 1.0)
						DataLoader::Default_Ambient = temp_value;
					else
						throw new FormatException();
				}
				else if ( 0 == String::CompareOrdinal(n_XMLReader->GetAttribute(NAME), S"diffuse") )
				{	
					float temp_value = Convert::ToSingle(n_XMLReader->GetAttribute(VALUE), NumberFormatInfo::InvariantInfo);
					if (temp_value >= 0.0 && temp_value <= 1.0)
						DataLoader::Default_Diffuse = temp_value;
					else
						throw new FormatException();
				}
				else if ( 0 == String::CompareOrdinal(n_XMLReader->GetAttribute(NAME), S"specular") )
				{	
					float temp_value = Convert::ToSingle(n_XMLReader->GetAttribute(VALUE), NumberFormatInfo::InvariantInfo);
					if (temp_value >= 0.0 && temp_value <= 1.0)
						DataLoader::Default_Specular = temp_value;
					else
						throw new FormatException();
				}
				else if ( 0 == String::CompareOrdinal(n_XMLReader->GetAttribute(NAME), S"spExponent") )
				{
					short temp_value = Convert::ToInt16(n_XMLReader->GetAttribute(VALUE), NumberFormatInfo::InvariantInfo);
					if (temp_value >= 0 && temp_value <= 999)
						DataLoader::Default_SpExponent = temp_value;
					else
						throw new FormatException();
				}
			}
			
			n_XMLReader->Read();
		}
	}
	catch(FormatException*)
			{
				throw new FormatException ( String::Concat(S"An error during reading ", this->m_FileName, S" has occured.\n The format of the ", n_XMLReader->GetAttribute(NAME), S" value is not correct."));
				return;
			}
}

void XMLParser::read_DatafileSettings(XmlNodeReader *n_XMLReader)
{
	n_XMLReader->Read();

	try
	{
		while (n_XMLReader->NodeType != XmlNodeType::EndElement)
		{
			if (n_XMLReader->NodeType != XmlNodeType::Comment) 
			{
				if ( 0 == String::CompareOrdinal(n_XMLReader->GetAttribute(NAME), S"densityScope") )
					DataLoader::DensityScope = Convert::ToUInt16(n_XMLReader->GetAttribute(VALUE), NumberFormatInfo::InvariantInfo);

				else if ( 0 == String::CompareOrdinal(n_XMLReader->GetAttribute(NAME), S"dataFileDirectory") )
					DataLoader::Default_DataFileDirectory = n_XMLReader->GetAttribute(VALUE);

				else if ( 0 == String::CompareOrdinal(n_XMLReader->GetAttribute(NAME), S"maskFileDirectory") )
					DataLoader::Default_MaskFileDirectory = n_XMLReader->GetAttribute(VALUE);

				else if ( 0 == String::CompareOrdinal(n_XMLReader->GetAttribute(NAME), S"transFuncFileDirectory") )
					DataLoader::Default_TransFuncFileDirectory = n_XMLReader->GetAttribute(VALUE);

				else if ( 0 == String::CompareOrdinal(n_XMLReader->GetAttribute(NAME), S"configFileDirectory") )
					DataLoader::Default_ConfigFileDirectory = n_XMLReader->GetAttribute(VALUE);

				else if ( 0 == String::CompareOrdinal(n_XMLReader->GetAttribute(NAME), S"maxUsableMemory") )
					DataLoader::MaxUsableMemory = Convert::ToUInt16(n_XMLReader->GetAttribute(VALUE), NumberFormatInfo::InvariantInfo);
			}

			n_XMLReader->Read();	
		}
	}
	catch(FormatException*)
	{
		throw new FormatException ( String::Concat(S"An error during reading ", this->m_FileName, S" has occured.\n The format of the ", n_XMLReader->GetAttribute(NAME), S" value is not correct."));
		return;
	}

}

void XMLParser::read_OptimationSettings(XmlNodeReader *n_XMLReader)
{
	n_XMLReader->Read();

	try
	{
		while (n_XMLReader->NodeType != XmlNodeType::EndElement)
		{
			if (n_XMLReader->NodeType != XmlNodeType::Comment) 
			{
				if ( 0 == String::CompareOrdinal(n_XMLReader->GetAttribute(NAME), S"samplingDistance") )
				{	
					float temp_value = Convert::ToSingle(n_XMLReader->GetAttribute(VALUE), NumberFormatInfo::InvariantInfo);
					if (temp_value >= 0.0 && temp_value <= 1.0)
						DataLoader::Default_SamplingDistance = temp_value;
					else
						throw new FormatException();
				}
				else if ( 0 == String::CompareOrdinal(n_XMLReader->GetAttribute(NAME), S"gradientThreshold") )
				{	
					float temp_value = Convert::ToSingle(n_XMLReader->GetAttribute(VALUE), NumberFormatInfo::InvariantInfo);
					if (temp_value >= 0.0 && temp_value <= 1.0)
						DataLoader::Default_GradientThreshold = temp_value;
					else
						throw new FormatException();
				}
				else if ( 0 == String::CompareOrdinal(n_XMLReader->GetAttribute(NAME), S"opacityThreshold") )
				{	
					float temp_value = Convert::ToSingle(n_XMLReader->GetAttribute(VALUE), NumberFormatInfo::InvariantInfo);
					if (temp_value >= 0.0 && temp_value <= 1.0)
						DataLoader::Default_OpacityThreshold = temp_value;
					else
						throw new FormatException();
				}
			}	
			n_XMLReader->Read();
		}
	}
	catch(FormatException*)
	{
		throw new FormatException ( String::Concat(S"An error during reading ", this->m_FileName, S" has occured.\n The format of the ", n_XMLReader->GetAttribute(NAME), S" value is not correct."));
		return;
	}

}

void XMLParser::read_DepthCueingSettings(XmlNodeReader *n_XMLReader)
{
	n_XMLReader->Read();

	try
	{
		while (n_XMLReader->NodeType != XmlNodeType::EndElement)
		{
			if (n_XMLReader->NodeType != XmlNodeType::Comment) 
			{
				if ( 0 == String::CompareOrdinal(n_XMLReader->GetAttribute(NAME), S"K1") )
				{	
					float temp_value = Convert::ToSingle(n_XMLReader->GetAttribute(VALUE), NumberFormatInfo::InvariantInfo);
					if (temp_value >= 0.01 && temp_value <= 1.0)
						DataLoader::Default_K1DepthCueing = temp_value;
					else
						throw new FormatException();
				}
				else if ( 0 == String::CompareOrdinal(n_XMLReader->GetAttribute(NAME), S"K2") )
				{	
					float temp_value = Convert::ToSingle(n_XMLReader->GetAttribute(VALUE), NumberFormatInfo::InvariantInfo);
					if (temp_value >= 0.0 && temp_value <= 1.0)
						DataLoader::Default_K2DepthCueing = temp_value;
					else
						throw new FormatException();
				}
			}
			n_XMLReader->Read();
		}
	}
	catch(FormatException*)
	{
		throw new FormatException ( String::Concat(S"An error during reading ", this->m_FileName, S" has occured.\n The format of the ", n_XMLReader->GetAttribute(NAME), S" value is not correct."));
		return;
	}

}

void XMLParser::read_MIPSettings(XmlNodeReader *n_XMLReader)
{
	n_XMLReader->Read();

	int mipColor __gc[] = new int __gc[3];

	try
	{
		while (n_XMLReader->NodeType != XmlNodeType::EndElement)
		{
			if (n_XMLReader->NodeType != XmlNodeType::Comment) 
			{
				if ( 0 == String::CompareOrdinal(n_XMLReader->GetAttribute(NAME), S"mipThreshold") )
				{	
					int temp_value = Convert::ToInt32(n_XMLReader->GetAttribute(VALUE), NumberFormatInfo::InvariantInfo);
					if (temp_value >= 0 && temp_value <= 4095)
						DataLoader::Default_MipThreshold = temp_value;
					else
						throw new FormatException();
				}
				else if ( 0 == String::CompareOrdinal(n_XMLReader->GetAttribute(NAME), S"mipColorRed") )
				{	
					int temp_value = Convert::ToInt32(n_XMLReader->GetAttribute(VALUE), NumberFormatInfo::InvariantInfo);
					if (temp_value >= 0 && temp_value <= 255)
						mipColor[0] = temp_value;
					else
						throw new FormatException();
				}
				else if ( 0 == String::CompareOrdinal(n_XMLReader->GetAttribute(NAME), S"mipColorGreen") )
				{	
					int temp_value = Convert::ToInt32(n_XMLReader->GetAttribute(VALUE), NumberFormatInfo::InvariantInfo);
					if (temp_value >= 0 && temp_value <= 255)
						mipColor[1] = temp_value;
					else
						throw new FormatException();
				}
				else if ( 0 == String::CompareOrdinal(n_XMLReader->GetAttribute(NAME), S"mipColorBlue") )
				{	
					int temp_value = Convert::ToInt32(n_XMLReader->GetAttribute(VALUE), NumberFormatInfo::InvariantInfo);
					if (temp_value >= 0 && temp_value <= 255)
						mipColor[2] = temp_value;
					else
						throw new FormatException();
				}
			}	
			n_XMLReader->Read();
		}

		DataLoader::Default_MipColor = Color::FromArgb(mipColor[0],mipColor[1],mipColor[2]);
	}
	catch(FormatException*)
	{
		throw new FormatException ( String::Concat(S"An error during reading ", this->m_FileName, S" has occured.\n The format of the ", n_XMLReader->GetAttribute(NAME), S" value is not correct."));
		return;
	}
}

void XMLParser::read_ContourSettings(XmlNodeReader *n_XMLReader)
{
	n_XMLReader->Read();

	int contourColor __gc[] = new int __gc[3];

	try
	{
		while (n_XMLReader->NodeType != XmlNodeType::EndElement)
		{
			if (n_XMLReader->NodeType != XmlNodeType::Comment) 
			{
				if ( 0 == String::CompareOrdinal(n_XMLReader->GetAttribute(NAME), S"contourThreshold") )
				{	
					float temp_value = Convert::ToSingle(n_XMLReader->GetAttribute(VALUE), NumberFormatInfo::InvariantInfo);
					if (temp_value >= 0.0f && temp_value <= 1.0f)
						DataLoader::Default_ContourThreshold = temp_value;
					else
						throw new FormatException();
				}
				else if ( 0 == String::CompareOrdinal(n_XMLReader->GetAttribute(NAME), S"contourGradientThreshold") )
				{	
					float temp_value = Convert::ToSingle(n_XMLReader->GetAttribute(VALUE), NumberFormatInfo::InvariantInfo);
					if (temp_value >= 0.0 && temp_value <= 1.0)
						DataLoader::Default_ContourGradientThreshold = temp_value;
					else
						throw new FormatException();
				}
				else if ( 0 == String::CompareOrdinal(n_XMLReader->GetAttribute(NAME), S"contourExponent") )
					DataLoader::Default_ContourExponent = Convert::ToInt32(n_XMLReader->GetAttribute(VALUE), NumberFormatInfo::InvariantInfo);
				else if ( 0 == String::CompareOrdinal(n_XMLReader->GetAttribute(NAME), S"ContourKTF") )
				{	
					float temp_value = Convert::ToSingle(n_XMLReader->GetAttribute(VALUE), NumberFormatInfo::InvariantInfo);
					if (temp_value >= 0.0 && temp_value <= 1.0)
						DataLoader::Default_ContourKTF = temp_value;
					else
						throw new FormatException();
				}
				else if ( 0 == String::CompareOrdinal(n_XMLReader->GetAttribute(NAME), S"contourColorRed") )
				{	
					int temp_value = Convert::ToInt32(n_XMLReader->GetAttribute(VALUE), NumberFormatInfo::InvariantInfo);
					if (temp_value >= 0 && temp_value <= 255)
						contourColor[0] = temp_value;
					else
						throw new FormatException();
				}
				else if ( 0 == String::CompareOrdinal(n_XMLReader->GetAttribute(NAME), S"contourColorGreen") )
				{	
					int temp_value = Convert::ToInt32(n_XMLReader->GetAttribute(VALUE), NumberFormatInfo::InvariantInfo);
					if (temp_value >= 0 && temp_value <= 255)
						contourColor[1] = temp_value;
					else
						throw new FormatException();
				}
				else if ( 0 == String::CompareOrdinal(n_XMLReader->GetAttribute(NAME), S"contourColorBlue") )
				{	
					int temp_value = Convert::ToInt32(n_XMLReader->GetAttribute(VALUE), NumberFormatInfo::InvariantInfo);
					if (temp_value >= 0 && temp_value <= 255)
						contourColor[2] = temp_value;
					else
						throw new FormatException();
				}
			}		
			n_XMLReader->Read();
		}

		DataLoader::Default_ContourColor = Color::FromArgb(contourColor[0],contourColor[1],contourColor[2]);
	}
	catch(FormatException*)
	{
		throw new FormatException ( String::Concat(S"An error during reading ", this->m_FileName, S" has occured.\n The format of the ", n_XMLReader->GetAttribute(NAME), S" value is not correct."));
		return;
	}
}

void XMLParser::read_ToneSettings(XmlNodeReader *n_XMLReader)
{
	n_XMLReader->Read();

	int warmColor __gc[] = new int __gc[3];
	int coldColor __gc[] = new int __gc[3];

	try
	{
		while (n_XMLReader->NodeType != XmlNodeType::EndElement)
		{
			if (n_XMLReader->NodeType != XmlNodeType::Comment) 
			{
				if ( 0 == String::CompareOrdinal(n_XMLReader->GetAttribute(NAME), S"toneThreshold") )
				{	
					float temp_value = Convert::ToSingle(n_XMLReader->GetAttribute(VALUE), NumberFormatInfo::InvariantInfo);
					if (temp_value >= 0.0f && temp_value <= 1.0f)
						DataLoader::Default_ToneThreshold = temp_value;
					else
						throw new FormatException();
				}
				else if ( 0 == String::CompareOrdinal(n_XMLReader->GetAttribute(NAME), S"toneMagnitudeThreshold") )
				{	
					float temp_value = Convert::ToSingle(n_XMLReader->GetAttribute(VALUE), NumberFormatInfo::InvariantInfo);
					if (temp_value >= 0.0 && temp_value <= 1.0)
						DataLoader::Default_ToneMagnitudeThreshold = temp_value;
					else
						throw new FormatException();
				}
				else if ( 0 == String::CompareOrdinal(n_XMLReader->GetAttribute(NAME), S"toneKWarm") )
				{	
					float temp_value = Convert::ToSingle(n_XMLReader->GetAttribute(VALUE), NumberFormatInfo::InvariantInfo);
					if (temp_value >= 0.0 && temp_value <= 1.0)
						DataLoader::Default_ToneKWarm = temp_value;
					else
						throw new FormatException();
				}
				else if ( 0 == String::CompareOrdinal(n_XMLReader->GetAttribute(NAME), S"toneKCold") )
				{	
					float temp_value = Convert::ToSingle(n_XMLReader->GetAttribute(VALUE), NumberFormatInfo::InvariantInfo);
					if (temp_value >= 0.0 && temp_value <= 1.0)
						DataLoader::Default_ToneKCold = temp_value;
					else
						throw new FormatException();
				}
				else if ( 0 == String::CompareOrdinal(n_XMLReader->GetAttribute(NAME), S"ToneKTF") )
				{	
					float temp_value = Convert::ToSingle(n_XMLReader->GetAttribute(VALUE), NumberFormatInfo::InvariantInfo);
					if (temp_value >= 0.0 && temp_value <= 1.0)
						DataLoader::Default_ToneKTF = temp_value;
					else
						throw new FormatException();
				}
				else if ( 0 == String::CompareOrdinal(n_XMLReader->GetAttribute(NAME), S"toneWarmRed") )
				{	
					int temp_value = Convert::ToInt32(n_XMLReader->GetAttribute(VALUE), NumberFormatInfo::InvariantInfo);
					if (temp_value >= 0 && temp_value <= 255)
						warmColor[0] = temp_value;
					else
						throw new FormatException();
				}
				else if ( 0 == String::CompareOrdinal(n_XMLReader->GetAttribute(NAME), S"toneWarmGreen") )
				{	
					int temp_value = Convert::ToInt32(n_XMLReader->GetAttribute(VALUE), NumberFormatInfo::InvariantInfo);
					if (temp_value >= 0 && temp_value <= 255)
						warmColor[1] = temp_value;
					else
						throw new FormatException();
				}
				else if ( 0 == String::CompareOrdinal(n_XMLReader->GetAttribute(NAME), S"toneWarmBlue") )
				{	
					int temp_value = Convert::ToInt32(n_XMLReader->GetAttribute(VALUE), NumberFormatInfo::InvariantInfo);
					if (temp_value >= 0 && temp_value <= 255)
						warmColor[2] = temp_value;
					else
						throw new FormatException();
				}
				else if ( 0 == String::CompareOrdinal(n_XMLReader->GetAttribute(NAME), S"toneColdRed") )
				{	
					int temp_value = Convert::ToInt32(n_XMLReader->GetAttribute(VALUE), NumberFormatInfo::InvariantInfo);
					if (temp_value >= 0 && temp_value <= 255)
						coldColor[0] = temp_value;
					else
						throw new FormatException();
				}
				else if ( 0 == String::CompareOrdinal(n_XMLReader->GetAttribute(NAME), S"toneColdGreen") )
				{	
					int temp_value = Convert::ToInt32(n_XMLReader->GetAttribute(VALUE), NumberFormatInfo::InvariantInfo);
					if (temp_value >= 0 && temp_value <= 255)
						coldColor[1] = temp_value;
					else
						throw new FormatException();
				}
				else if ( 0 == String::CompareOrdinal(n_XMLReader->GetAttribute(NAME), S"toneColdBlue") )
				{	
					int temp_value = Convert::ToInt32(n_XMLReader->GetAttribute(VALUE), NumberFormatInfo::InvariantInfo);
					if (temp_value >= 0 && temp_value <= 255)
						coldColor[2] = temp_value;
					else
						throw new FormatException();
				}
			}	
			n_XMLReader->Read();
		}

		DataLoader::Default_ToneWarmColor = Color::FromArgb(warmColor[0],warmColor[1],warmColor[2]);
		DataLoader::Default_ToneColdColor = Color::FromArgb(coldColor[0],coldColor[1],coldColor[2]);
	}
	catch(FormatException*)
	{
		throw new FormatException ( String::Concat(S"An error during reading ", this->m_FileName, S" has occured.\n The format of the ", n_XMLReader->GetAttribute(NAME), S" value is not correct."));
		return;
	}
}

void XMLParser::save_Transferfunction(Transferfunction* n_TransFunc)
{
	Hashtable *colorSamples = n_TransFunc->get_ColorSamples();
	Hashtable *opacitySamples = n_TransFunc->get_OpacitySamples();
	float interpolationFactor = n_TransFunc->get_InterpolationFactor();
	int backColor = n_TransFunc->get_BackColor().ToArgb();

	XmlTextWriter *saveTransFunc;

	try
	{

		saveTransFunc = new XmlTextWriter (this->m_FilePath, 0);

        saveTransFunc->Formatting = Formatting::Indented;
        saveTransFunc->WriteStartDocument(false);

		saveTransFunc->WriteComment(S"   This xml-File is built by the IDVR Volume Renderer System.");
        saveTransFunc->WriteComment(S"   This xml-File contains all data for a saved transfer function with opactiy and color values.");
		saveTransFunc->WriteComment(S"   Please do not change any of the following data to avoid further loading errors!");

		//Saving the main document element
        saveTransFunc->WriteStartElement(S"Transferfunction");
		saveTransFunc->WriteAttributeString(S"functionLength", Convert::ToString(n_TransFunc->get_FunctionLength(), NumberFormatInfo::InvariantInfo));

		//Saving opacity sample table of the TF
		saveTransFunc->WriteStartElement(S"opacitySamples");
		saveTransFunc->WriteAttributeString(S"length", Convert::ToString(opacitySamples->get_Count()));

		IDictionaryEnumerator *sampleEnumerator = opacitySamples->GetEnumerator();
        
        while (sampleEnumerator->MoveNext())
		{
			saveTransFunc->WriteStartElement(S"newElement");
			saveTransFunc->WriteAttributeString("key", Convert::ToString(sampleEnumerator->get_Key(), NumberFormatInfo::InvariantInfo));
			saveTransFunc->WriteAttributeString("value", Convert::ToString(sampleEnumerator->get_Value(), NumberFormatInfo::InvariantInfo));
			saveTransFunc->WriteEndElement();
		}
		saveTransFunc->WriteEndElement();

		//Saving color sample table of the TF
		saveTransFunc->WriteStartElement(S"colorSamples");
		saveTransFunc->WriteAttributeString(S"length", Convert::ToString(colorSamples->get_Count()));

		sampleEnumerator = colorSamples->GetEnumerator();
        
        while (sampleEnumerator->MoveNext())
		{
			saveTransFunc->WriteStartElement(S"newElement");
			saveTransFunc->WriteAttributeString("key", Convert::ToString(sampleEnumerator->get_Key(), NumberFormatInfo::InvariantInfo));
			saveTransFunc->WriteAttributeString("value", Convert::ToString(sampleEnumerator->get_Value(), NumberFormatInfo::InvariantInfo));
			saveTransFunc->WriteEndElement();
		}
		saveTransFunc->WriteEndElement();

		//Saving the interpolation factor of the transfer function
		saveTransFunc->WriteStartElement(S"interpolationFactor");
		saveTransFunc->WriteAttributeString(S"value", Convert::ToString(interpolationFactor, NumberFormatInfo::InvariantInfo));
		saveTransFunc->WriteEndElement();

		//Saving the back color of the transfer function
		saveTransFunc->WriteStartElement(S"backColor");
		saveTransFunc->WriteAttributeString(S"value", Convert::ToString(backColor, NumberFormatInfo::InvariantInfo));
		saveTransFunc->WriteEndElement();

		saveTransFunc->WriteEndElement();

        saveTransFunc->Flush();
        saveTransFunc->Close();

		
	}
	catch(XmlException* xml)
	{
		throw new Exception(xml->get_Message(), xml);
	}
	catch(FormatException* format)
	{
		throw new Exception(format->get_Message(), format);
	}
	__finally
	{
		saveTransFunc->Close();
	}
}

void XMLParser::save_ConfigFile(Array *n_NewValues)
{

	XmlTextWriter *n_SaveConfigFile;
	int n_Index = 0;

	try
	{

		n_SaveConfigFile = new XmlTextWriter (this->m_FilePath, 0);

        n_SaveConfigFile->Formatting = Formatting::Indented;
        n_SaveConfigFile->WriteStartDocument(false);

		n_SaveConfigFile->WriteComment(S"   This xml-File is built by the IDVR Volume Renderer System.");
        n_SaveConfigFile->WriteComment(S"   All default values for the initialization of IDVR ares saved in this file.");
		n_SaveConfigFile->WriteComment(S"   For changing specific values please mind the correct value format!");

		n_SaveConfigFile->WriteStartElement(S"configuration");
		save_DatafileSettings(n_SaveConfigFile, n_NewValues, &n_Index);
		save_PhongShadingSettings(n_SaveConfigFile, n_NewValues, &n_Index);
		save_OptimationSettings(n_SaveConfigFile, n_NewValues, &n_Index);
		save_DepthCueingSettings(n_SaveConfigFile, n_NewValues, &n_Index);
		save_MIPSettings(n_SaveConfigFile, n_NewValues, &n_Index);
		save_ContourSettings(n_SaveConfigFile, n_NewValues, &n_Index);
		save_ToneSettings(n_SaveConfigFile, n_NewValues, &n_Index);
		n_SaveConfigFile->WriteEndElement();
        

        n_SaveConfigFile->Flush();
        n_SaveConfigFile->Close();

		
	}
	catch(XmlException* xml)
	{
		throw new Exception(xml->get_Message(), xml);
	}
	catch(FormatException* format)
	{
		throw new Exception(format->get_Message(), format);
	}
	__finally
	{
		n_SaveConfigFile->Close();
	}

}

void XMLParser::save_DatafileSettings(XmlTextWriter *n_SaveConfigFile, Array *n_NewValues, int *n_Index)
{
	n_SaveConfigFile->WriteStartElement(S"appSettings");

	n_SaveConfigFile->WriteStartElement(S"add");
	n_SaveConfigFile->WriteAttributeString(S"key", S"grayLevels");
	n_SaveConfigFile->WriteAttributeString(S"value", Convert::ToString(n_NewValues->get_Item(*n_Index), NumberFormatInfo::InvariantInfo));
	*n_Index+=1;
	n_SaveConfigFile->WriteEndElement();

	n_SaveConfigFile->WriteStartElement(S"add");
	n_SaveConfigFile->WriteAttributeString(S"key", S"dataFileDirectory");
	n_SaveConfigFile->WriteAttributeString(S"value", Convert::ToString(n_NewValues->get_Item(*n_Index), NumberFormatInfo::InvariantInfo));
	*n_Index+=1;
	n_SaveConfigFile->WriteEndElement();

	n_SaveConfigFile->WriteStartElement(S"add");
	n_SaveConfigFile->WriteAttributeString(S"key", S"maskFileDirectory");
	n_SaveConfigFile->WriteAttributeString(S"value", Convert::ToString(n_NewValues->get_Item(*n_Index), NumberFormatInfo::InvariantInfo));
	*n_Index+=1;
	n_SaveConfigFile->WriteEndElement();

	n_SaveConfigFile->WriteStartElement(S"add");
	n_SaveConfigFile->WriteAttributeString(S"key", S"transFuncFileDirectory");
	n_SaveConfigFile->WriteAttributeString(S"value", Convert::ToString(n_NewValues->get_Item(*n_Index), NumberFormatInfo::InvariantInfo));
	*n_Index+=1;
	n_SaveConfigFile->WriteEndElement();

	n_SaveConfigFile->WriteStartElement(S"add");
	n_SaveConfigFile->WriteAttributeString(S"key", S"configFileDirectory");
	n_SaveConfigFile->WriteAttributeString(S"value", Convert::ToString(n_NewValues->get_Item(*n_Index), NumberFormatInfo::InvariantInfo));
	*n_Index+=1;
	n_SaveConfigFile->WriteEndElement();

	n_SaveConfigFile->WriteStartElement(S"add");
	n_SaveConfigFile->WriteAttributeString(S"key", S"maxUsableMemory");
	n_SaveConfigFile->WriteAttributeString(S"value", Convert::ToString(n_NewValues->get_Item(*n_Index), NumberFormatInfo::InvariantInfo));
	*n_Index+=1;
	n_SaveConfigFile->WriteEndElement();

	n_SaveConfigFile->WriteEndElement();
}

void XMLParser::save_PhongShadingSettings(XmlTextWriter *n_SaveConfigFile, Array *n_NewValues, int *n_Index)
{
	n_SaveConfigFile->WriteStartElement(S"phongShadingSettings");

	n_SaveConfigFile->WriteComment(S"  The ambient value must be between 0.0 and 1.0 ");
	n_SaveConfigFile->WriteStartElement(S"add");
	n_SaveConfigFile->WriteAttributeString(S"key", S"ambient");

	n_SaveConfigFile->WriteAttributeString(S"value", Convert::ToString(n_NewValues->get_Item(*n_Index), NumberFormatInfo::InvariantInfo));
	*n_Index+=1;
	n_SaveConfigFile->WriteEndElement();

	n_SaveConfigFile->WriteComment(S"  The diffuse value must be between 0.0 and 1.0 ");
	n_SaveConfigFile->WriteStartElement(S"add");
	n_SaveConfigFile->WriteAttributeString(S"key", S"diffuse");
	n_SaveConfigFile->WriteAttributeString(S"value", Convert::ToString(n_NewValues->get_Item(*n_Index), NumberFormatInfo::InvariantInfo));
	*n_Index+=1;
	n_SaveConfigFile->WriteEndElement();

	n_SaveConfigFile->WriteComment(S"  The specular value must be between 0.0 and 1.0 ");
	n_SaveConfigFile->WriteStartElement(S"add");
	n_SaveConfigFile->WriteAttributeString(S"key", S"specular");
	n_SaveConfigFile->WriteAttributeString(S"value", Convert::ToString(n_NewValues->get_Item(*n_Index), NumberFormatInfo::InvariantInfo));
	*n_Index+=1;
	n_SaveConfigFile->WriteEndElement();

	n_SaveConfigFile->WriteStartElement(S"add");
	n_SaveConfigFile->WriteAttributeString(S"key", S"spExponent");
	n_SaveConfigFile->WriteAttributeString(S"value", Convert::ToString(n_NewValues->get_Item(*n_Index), NumberFormatInfo::InvariantInfo));
	*n_Index+=1;
	n_SaveConfigFile->WriteEndElement();

	n_SaveConfigFile->WriteEndElement();
}

void XMLParser::save_OptimationSettings(XmlTextWriter *n_SaveConfigFile, Array *n_NewValues, int *n_Index)
{
	n_SaveConfigFile->WriteStartElement(S"optimationSettings");

	n_SaveConfigFile->WriteComment(S"  The samplingDistance value must be between 0.0 and 1.0 ");
	n_SaveConfigFile->WriteStartElement(S"add");
	n_SaveConfigFile->WriteAttributeString(S"key", S"samplingDistance");
	n_SaveConfigFile->WriteAttributeString(S"value", Convert::ToString(n_NewValues->get_Item(*n_Index), NumberFormatInfo::InvariantInfo));
	*n_Index+=1;
	n_SaveConfigFile->WriteEndElement();

	n_SaveConfigFile->WriteComment(S"  The gradientThreshold value must be between 0.0 and 1.0 ");
	n_SaveConfigFile->WriteStartElement(S"add");
	n_SaveConfigFile->WriteAttributeString(S"key", S"gradientThreshold");
	n_SaveConfigFile->WriteAttributeString(S"value", Convert::ToString(n_NewValues->get_Item(*n_Index), NumberFormatInfo::InvariantInfo));
	*n_Index+=1;
	n_SaveConfigFile->WriteEndElement();

	n_SaveConfigFile->WriteComment(S"  The opacityThreshold value must be between 0.0 and 1.0 ");
	n_SaveConfigFile->WriteStartElement(S"add");
	n_SaveConfigFile->WriteAttributeString(S"key", S"opacityThreshold");
	n_SaveConfigFile->WriteAttributeString(S"value", Convert::ToString(n_NewValues->get_Item(*n_Index), NumberFormatInfo::InvariantInfo));
	*n_Index+=1;
	n_SaveConfigFile->WriteEndElement();

	n_SaveConfigFile->WriteEndElement();
}

void XMLParser::save_DepthCueingSettings(XmlTextWriter *n_SaveConfigFile, Array *n_NewValues, int *n_Index)
{
	n_SaveConfigFile->WriteStartElement(S"depthCueingSettings");

	n_SaveConfigFile->WriteComment(S"  The K1 value must be between 0.01 and 1.0 ");
	n_SaveConfigFile->WriteStartElement(S"add");
	n_SaveConfigFile->WriteAttributeString(S"key", S"K1");
	n_SaveConfigFile->WriteAttributeString(S"value", Convert::ToString(n_NewValues->get_Item(*n_Index), NumberFormatInfo::InvariantInfo));
	*n_Index+=1;
	n_SaveConfigFile->WriteEndElement();

	n_SaveConfigFile->WriteComment(S"  The K2 value must be between 0.0 and 1.0 ");
	n_SaveConfigFile->WriteStartElement(S"add");
	n_SaveConfigFile->WriteAttributeString(S"key", S"K2");
	n_SaveConfigFile->WriteAttributeString(S"value", Convert::ToString(n_NewValues->get_Item(*n_Index), NumberFormatInfo::InvariantInfo));
	*n_Index+=1;
	n_SaveConfigFile->WriteEndElement();

	n_SaveConfigFile->WriteEndElement();
}

void XMLParser::save_MIPSettings(XmlTextWriter *n_SaveConfigFile, Array *n_NewValues, int *n_Index)
{
	n_SaveConfigFile->WriteStartElement(S"mipSettings");

	n_SaveConfigFile->WriteComment(S"  The mipThreshold value must be between 0 and 255 ");
	n_SaveConfigFile->WriteStartElement(S"add");
	n_SaveConfigFile->WriteAttributeString(S"key", S"mipThreshold");
	n_SaveConfigFile->WriteAttributeString(S"value", Convert::ToString(n_NewValues->get_Item(*n_Index), NumberFormatInfo::InvariantInfo));
	*n_Index+=1;
	n_SaveConfigFile->WriteEndElement();

	n_SaveConfigFile->WriteComment(S"  The mipColorRed value must be between 0 and 255 ");
	n_SaveConfigFile->WriteStartElement(S"add");
	n_SaveConfigFile->WriteAttributeString(S"key", S"mipColorRed");
	n_SaveConfigFile->WriteAttributeString(S"value", Convert::ToString(n_NewValues->get_Item(*n_Index), NumberFormatInfo::InvariantInfo));
	*n_Index+=1;
	n_SaveConfigFile->WriteEndElement();

	n_SaveConfigFile->WriteComment(S"  The mipColorGreen value must be between 0 and 255 ");
	n_SaveConfigFile->WriteStartElement(S"add");
	n_SaveConfigFile->WriteAttributeString(S"key", S"mipColorGreen");
	n_SaveConfigFile->WriteAttributeString(S"value", Convert::ToString(n_NewValues->get_Item(*n_Index), NumberFormatInfo::InvariantInfo));
	*n_Index+=1;
	n_SaveConfigFile->WriteEndElement();

	n_SaveConfigFile->WriteComment(S"  The mipColorBlue value must be between 0 and 255 ");
	n_SaveConfigFile->WriteStartElement(S"add");
	n_SaveConfigFile->WriteAttributeString(S"key", S"mipColorBlue");
	n_SaveConfigFile->WriteAttributeString(S"value", Convert::ToString(n_NewValues->get_Item(*n_Index), NumberFormatInfo::InvariantInfo));
	*n_Index+=1;
	n_SaveConfigFile->WriteEndElement();

	n_SaveConfigFile->WriteEndElement();
}

void XMLParser::save_ContourSettings(XmlTextWriter *n_SaveConfigFile, Array *n_NewValues, int *n_Index)
{
	n_SaveConfigFile->WriteStartElement(S"contourSettings");

	n_SaveConfigFile->WriteComment(S"  The contourThreshold value must be between 0.0 and 1.0 ");
	n_SaveConfigFile->WriteStartElement(S"add");
	n_SaveConfigFile->WriteAttributeString(S"key", S"contourThreshold");
	n_SaveConfigFile->WriteAttributeString(S"value", Convert::ToString(n_NewValues->get_Item(*n_Index), NumberFormatInfo::InvariantInfo));
	*n_Index+=1;
	n_SaveConfigFile->WriteEndElement();

	n_SaveConfigFile->WriteComment(S"  The contourGradientThreshold value must be between 0.0 and 1.0 ");
	n_SaveConfigFile->WriteStartElement(S"add");
	n_SaveConfigFile->WriteAttributeString(S"key", S"contourGradientThreshold");
	n_SaveConfigFile->WriteAttributeString(S"value", Convert::ToString(n_NewValues->get_Item(*n_Index), NumberFormatInfo::InvariantInfo));
	*n_Index+=1;
	n_SaveConfigFile->WriteEndElement();

	n_SaveConfigFile->WriteStartElement(S"add");
	n_SaveConfigFile->WriteAttributeString(S"key", S"contourExponent");
	n_SaveConfigFile->WriteAttributeString(S"value", Convert::ToString(n_NewValues->get_Item(*n_Index), NumberFormatInfo::InvariantInfo));
	*n_Index+=1;
	n_SaveConfigFile->WriteEndElement();

	n_SaveConfigFile->WriteComment(S"  The ContourKTF value must be between 0.0 and 1.0 ");
	n_SaveConfigFile->WriteStartElement(S"add");
	n_SaveConfigFile->WriteAttributeString(S"key", S"ContourKTF");
	n_SaveConfigFile->WriteAttributeString(S"value", Convert::ToString(n_NewValues->get_Item(*n_Index), NumberFormatInfo::InvariantInfo));
	*n_Index+=1;
	n_SaveConfigFile->WriteEndElement();

	n_SaveConfigFile->WriteComment(S"  The contourColorRed value must be between 0 and 255 ");
	n_SaveConfigFile->WriteStartElement(S"add");
	n_SaveConfigFile->WriteAttributeString(S"key", S"contourColorRed");
	n_SaveConfigFile->WriteAttributeString(S"value", Convert::ToString(n_NewValues->get_Item(*n_Index), NumberFormatInfo::InvariantInfo));
	*n_Index+=1;
	n_SaveConfigFile->WriteEndElement();

	n_SaveConfigFile->WriteComment(S"  The contourColorGreen value must be between 0 and 255 ");
	n_SaveConfigFile->WriteStartElement(S"add");
	n_SaveConfigFile->WriteAttributeString(S"key", S"contourColorGreen");
	n_SaveConfigFile->WriteAttributeString(S"value", Convert::ToString(n_NewValues->get_Item(*n_Index), NumberFormatInfo::InvariantInfo));
	*n_Index+=1;
	n_SaveConfigFile->WriteEndElement();

	n_SaveConfigFile->WriteComment(S"  The contourColorBlue value must be between 0 and 255 ");
	n_SaveConfigFile->WriteStartElement(S"add");
	n_SaveConfigFile->WriteAttributeString(S"key", S"contourColorBlue");
	n_SaveConfigFile->WriteAttributeString(S"value", Convert::ToString(n_NewValues->get_Item(*n_Index), NumberFormatInfo::InvariantInfo));
	*n_Index+=1;
	n_SaveConfigFile->WriteEndElement();

	n_SaveConfigFile->WriteEndElement();
}

void XMLParser::save_ToneSettings(XmlTextWriter *n_SaveConfigFile, Array *n_NewValues, int *n_Index)
{
	n_SaveConfigFile->WriteStartElement(S"toneSettings");

	n_SaveConfigFile->WriteComment(S"  The toneThreshold value must be between 0.0 and 1.0 ");
	n_SaveConfigFile->WriteStartElement(S"add");
	n_SaveConfigFile->WriteAttributeString(S"key", S"toneThreshold");
	n_SaveConfigFile->WriteAttributeString(S"value", Convert::ToString(n_NewValues->get_Item(*n_Index), NumberFormatInfo::InvariantInfo));
	*n_Index+=1;
	n_SaveConfigFile->WriteEndElement();

	n_SaveConfigFile->WriteComment(S"  The toneMagnitudeThreshold value must be between 0.0 and 1.0 ");
	n_SaveConfigFile->WriteStartElement(S"add");
	n_SaveConfigFile->WriteAttributeString(S"key", S"toneMagnitudeThreshold");
	n_SaveConfigFile->WriteAttributeString(S"value", Convert::ToString(n_NewValues->get_Item(*n_Index), NumberFormatInfo::InvariantInfo));
	*n_Index+=1;
	n_SaveConfigFile->WriteEndElement();

	n_SaveConfigFile->WriteComment(S"  The toneKWarm value must be between 0.0 and 1.0 ");
	n_SaveConfigFile->WriteStartElement(S"add");
	n_SaveConfigFile->WriteAttributeString(S"key", S"toneKWarm");
	n_SaveConfigFile->WriteAttributeString(S"value", Convert::ToString(n_NewValues->get_Item(*n_Index), NumberFormatInfo::InvariantInfo));
	*n_Index+=1;
	n_SaveConfigFile->WriteEndElement();

	n_SaveConfigFile->WriteComment(S"  The toneKCold value must be between 0.0 and 1.0 ");
	n_SaveConfigFile->WriteStartElement(S"add");
	n_SaveConfigFile->WriteAttributeString(S"key", S"toneKCold");
	n_SaveConfigFile->WriteAttributeString(S"value", Convert::ToString(n_NewValues->get_Item(*n_Index), NumberFormatInfo::InvariantInfo));
	*n_Index+=1;
	n_SaveConfigFile->WriteEndElement();

	n_SaveConfigFile->WriteComment(S"  The ToneKTF value must be between 0.0 and 1.0 ");
	n_SaveConfigFile->WriteStartElement(S"add");
	n_SaveConfigFile->WriteAttributeString(S"key", S"ToneKTF");
	n_SaveConfigFile->WriteAttributeString(S"value", Convert::ToString(n_NewValues->get_Item(*n_Index), NumberFormatInfo::InvariantInfo));
	*n_Index+=1;
	n_SaveConfigFile->WriteEndElement();

	n_SaveConfigFile->WriteComment(S"  The toneWarmRed value must be between 0 and 255 ");
	n_SaveConfigFile->WriteStartElement(S"add");
	n_SaveConfigFile->WriteAttributeString(S"key", S"toneWarmRed");
	n_SaveConfigFile->WriteAttributeString(S"value", Convert::ToString(n_NewValues->get_Item(*n_Index), NumberFormatInfo::InvariantInfo));
	*n_Index+=1;
	n_SaveConfigFile->WriteEndElement();

	n_SaveConfigFile->WriteComment(S"  The toneWarmGreen value must be between 0 and 255 ");
	n_SaveConfigFile->WriteStartElement(S"add");
	n_SaveConfigFile->WriteAttributeString(S"key", S"toneWarmGreen");
	n_SaveConfigFile->WriteAttributeString(S"value", Convert::ToString(n_NewValues->get_Item(*n_Index), NumberFormatInfo::InvariantInfo));
	*n_Index+=1;
	n_SaveConfigFile->WriteEndElement();

	n_SaveConfigFile->WriteComment(S"  The toneWarmBlue value must be between 0 and 255 ");
	n_SaveConfigFile->WriteStartElement(S"add");
	n_SaveConfigFile->WriteAttributeString(S"key", S"toneWarmBlue");
	n_SaveConfigFile->WriteAttributeString(S"value", Convert::ToString(n_NewValues->get_Item(*n_Index), NumberFormatInfo::InvariantInfo));
	*n_Index+=1;
	n_SaveConfigFile->WriteEndElement();

	n_SaveConfigFile->WriteComment(S"  The toneColdRed value must be between 0 and 255 ");
	n_SaveConfigFile->WriteStartElement(S"add");
	n_SaveConfigFile->WriteAttributeString(S"key", S"toneColdRed");
	n_SaveConfigFile->WriteAttributeString(S"value", Convert::ToString(n_NewValues->get_Item(*n_Index), NumberFormatInfo::InvariantInfo));
	*n_Index+=1;
	n_SaveConfigFile->WriteEndElement();

	n_SaveConfigFile->WriteComment(S"  The toneColdGreen value must be between 0 and 255 ");
	n_SaveConfigFile->WriteStartElement(S"add");
	n_SaveConfigFile->WriteAttributeString(S"key", S"toneColdGreen");
	n_SaveConfigFile->WriteAttributeString(S"value", Convert::ToString(n_NewValues->get_Item(*n_Index), NumberFormatInfo::InvariantInfo));
	*n_Index+=1;
	n_SaveConfigFile->WriteEndElement();

	n_SaveConfigFile->WriteComment(S"  The toneColdBlue value must be between 0 and 255 ");
	n_SaveConfigFile->WriteStartElement(S"add");
	n_SaveConfigFile->WriteAttributeString(S"key", S"toneColdBlue");
	n_SaveConfigFile->WriteAttributeString(S"value", Convert::ToString(n_NewValues->get_Item(*n_Index), NumberFormatInfo::InvariantInfo));
	*n_Index+=1;
	n_SaveConfigFile->WriteEndElement();

	n_SaveConfigFile->WriteEndElement();
}











